@@ -5,13 +5,13 @@
  *
  *      Description:  Utility string routines building upon bstrlib
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com.
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -68,8 +68,11 @@ bstr_to_doubleSize(const_bstring str, DataType type)
 {
     int ret;
     bstring unit = bmidstr(str, blength(str)-2, 2);
+    bstring single_unit = bmidstr(str, blength(str)-1, 1);
     bstring sizeStr = bmidstr(str, 0, blength(str)-2);
+    bstring single_sizeStr = bmidstr(str, 0, blength(str)-1);
     uint64_t sizeU = 0;
+    uint64_t single_sizeU = 0;
     uint64_t junk = 0;
     uint64_t bytesize = 0;
     if (blength(sizeStr) == 0)
@@ -85,6 +88,15 @@ bstr_to_doubleSize(const_bstring str, DataType type)
     {
         return 0;
     }
+    ret = str2int(bdata(single_sizeStr));
+    if (ret >= 0)
+    {
+        single_sizeU = str2int(bdata(single_sizeStr));
+    }
+    else
+    {
+        return 0;
+    }
 
     bytesize = allocator_dataTypeLength(type);
 
@@ -100,12 +112,14 @@ bstr_to_doubleSize(const_bstring str, DataType type)
     {
         junk = (sizeU *1000000000)/bytesize;
     }
-    else if (biseqcstr(unit, "B"))
+    else if (biseqcstr(single_unit, "B"))
     {
-        junk = (sizeU)/bytesize;
+        junk = (single_sizeU)/bytesize;
     }
     bdestroy(unit);
+    bdestroy(single_unit);
     bdestroy(sizeStr);
+    bdestroy(single_sizeStr);
     return junk;
 }
 
