@@ -12,13 +12,13 @@ my $lc = ' *';
 
 #my $VERSION   = '<VERSION>';
 #my $DATE   = '<DATE>';
-my $VERSION   = '4.2';
-my $DATE   = '22.12.2016';
-my $YEAR  = '2016';
+my $VERSION   = '4.3.1';
+my $DATE   = '04.01.2018';
+my $YEAR  = '2018';
 my $AUTHOR = 'RRZE, University Erlangen-Nuremberg';
 my $LICENSE = 'gpl';
 
-my @SKIPLIST = ('ghash.c','ghash.h','loadData.S','bstrlib.c','bstrlib.h', 'calculator_stack.h', 'calculator_stack.c');
+my @SKIPLIST = ('ghash.c','ghash.h','loadData.S','bstrlib.c','bstrlib.h');
 
 sub print_copyright
 {
