@@ -5,15 +5,15 @@
  *
  *      Description:  Implementation of access daemon.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Authors:  Michael Meier, michael.meier@rrze.fau.de
  *                Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -48,6 +48,7 @@
 #include <unistd.h>
 #include <sys/fsuid.h>
 #include <getopt.h>
+#include <dirent.h>
 
 #include <types.h>
 #include <registers.h>
@@ -57,6 +58,7 @@
 #include <perfmon_broadwelld_counters.h>
 #include <perfmon_broadwellEP_counters.h>
 #include <perfmon_knl_counters.h>
+#include <perfmon_skylakeX_counters.h>
 #include <topology.h>
 #include <cpuid.h>
 #include <lock.h>
@@ -96,9 +98,12 @@ static char* filepath;
 static const char* ident = "accessD";
 static AllowedPrototype allowed = NULL;
 static AllowedPciPrototype allowedPci = NULL;
-static int FD_MSR[MAX_NUM_THREADS];
-static int FD_PCI[MAX_NUM_NODES][MAX_NUM_PCI_DEVICES];
+/*static int FD_MSR[MAX_NUM_THREADS];*/
+/*static int FD_PCI[MAX_NUM_NODES][MAX_NUM_PCI_DEVICES];*/
+static int* FD_MSR = NULL;
+static int** FD_PCI = NULL;
 static int isPCIUncore = 0;
+static int isPCI64 = 0;
 static PciDevice* pci_devices_daemon = NULL;
 static char pci_filepath[MAX_PATH_LENGTH];
 static int num_pmc_counters = 0;
@@ -111,7 +116,68 @@ static int num_pmc_counters = 0;
  *   2                  0xbf
  *   3                  0xff
  */
-static char* socket_bus[MAX_NUM_NODES] = { [0 ... (MAX_NUM_NODES-1)] = NULL};
+//static char* socket_bus[MAX_NUM_NODES] = { [0 ... (MAX_NUM_NODES-1)] = NULL};
+static char** socket_bus = NULL;
+static int avail_sockets = 0;
+static int avail_cpus = 0;
+
+void __attribute__((constructor (101))) init_accessdaemon(void)
+{
+    FILE *fpipe = NULL;
+    char *ptr = NULL;
+    char cmd_cpu[] = "cat /proc/cpuinfo  | grep 'processor' | sort -u | wc -l";
+    char cmd_sock[] = "cat /proc/cpuinfo  | grep 'physical id' | sort -u | wc -l";
+    //static long avail_sockets = 0;
+    char buff[256];
+    //avail_cpus = sysconf(_SC_NPROCESSORS_CONF);
+
+    if ( !(fpipe = (FILE*)popen(cmd_cpu,"r")) )
+    {  // If fpipe is NULL
+        return;
+    }
+    ptr = fgets(buff, 256, fpipe);
+    if (pclose(fpipe))
+        return;
+    avail_cpus = atoi(buff);
+
+    if ( !(fpipe = (FILE*)popen(cmd_sock,"r")) )
+    {  // If fpipe is NULL
+        return;
+    }
+    ptr = fgets(buff, 256, fpipe);
+    if (pclose(fpipe))
+        return;
+    avail_sockets = atoi(buff);
+
+    FD_MSR = malloc(avail_cpus * sizeof(int));
+    //memset(FD_MSR, 0, avail_cpus * sizeof(int));
+
+    socket_bus = malloc(avail_sockets * sizeof(char*));
+    //memset(socket_bus, 0, avail_sockets * sizeof(char*));
+
+    FD_PCI = malloc(avail_sockets * sizeof(int*));
+    //memset(FD_PCI, 0, avail_sockets * sizeof(int*));
+    for (int i = 0; i < avail_sockets; i++)
+    {
+        FD_PCI[i] = malloc(MAX_NUM_PCI_DEVICES * sizeof(int));
+        //memset(FD_PCI[i], 0, MAX_NUM_PCI_DEVICES * sizeof(int));
+    }
+}
+
+void __attribute__((destructor (101))) close_accessdaemon(void)
+{
+    for (int i = 0; i < avail_sockets; i++)
+    {
+        free(FD_PCI[i]);
+        FD_PCI[i] = NULL;
+    }
+    free(FD_PCI);
+    FD_PCI = NULL;
+    free(socket_bus);
+    socket_bus = NULL;
+    free(FD_MSR);
+    FD_MSR = NULL;
+}
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
@@ -480,7 +546,8 @@ static int allowed_knl(uint32_t reg)
     {
         if (((reg & 0xF00U) == 0x700U) ||
             ((reg & 0xF00U) == 0xE00U) ||
-            ((reg & 0xF00U) == 0xF00U))
+            ((reg & 0xF00U) == 0xF00U) ||
+            (reg == MSR_PREFETCH_ENABLE))
             return 1;
     }
     return 0;
@@ -605,6 +672,84 @@ static int allowed_pci_knl(PciDeviceType type, uint32_t reg)
     return 0;
 }
 
+static int allowed_skx(uint32_t reg)
+{
+    if (allowed_sandybridge(reg))
+        return 1;
+    else
+    {
+        syslog(LOG_ERR, "Testing 0x%X %d %d\n", reg, (reg & 0xF00U), (reg & 0xA00U));
+        if (((reg & 0xF00U) == 0x700U) ||
+            ((reg & 0xF00U) == 0xE00U) ||
+            ((reg & 0xF00U) == 0xF00U) ||
+            (reg == MSR_PREFETCH_ENABLE) ||
+            ((reg & 0xA00U) == 0xA00U))
+            return 1;
+    }   
+    return 0;
+}
+
+
+static int allowed_pci_skx(PciDeviceType type, uint32_t reg)
+{
+    switch(type)
+    {
+        case HA:
+            if ((reg == MSR_UNC_SKX_M2M_PMON_CTL0)||
+                (reg == MSR_UNC_SKX_M2M_PMON_CTL1)||
+                (reg == MSR_UNC_SKX_M2M_PMON_CTL2)||
+                (reg == MSR_UNC_SKX_M2M_PMON_CTL3)||
+                (reg == MSR_UNC_SKX_M2M_PMON_CTR0) ||
+                (reg == MSR_UNC_SKX_M2M_PMON_CTR1) ||
+                (reg == MSR_UNC_SKX_M2M_PMON_CTR2) ||
+                (reg == MSR_UNC_SKX_M2M_PMON_CTR3) ||
+                (reg == MSR_UNC_SKX_M2M_PMON_BOX_CTL) ||
+                (reg == MSR_UNC_SKX_M2M_PMON_BOX_STATUS))
+                return 1;
+            break;
+        case IMC:
+            if ((reg == PCI_UNC_SKX_MC_PMON_CTL0) ||
+                (reg == PCI_UNC_SKX_MC_PMON_CTL1) ||
+                (reg == PCI_UNC_SKX_MC_PMON_CTL2) ||
+                (reg == PCI_UNC_SKX_MC_PMON_CTL3) ||
+                (reg == PCI_UNC_SKX_MC_PMON_CTR0) ||
+                (reg == PCI_UNC_SKX_MC_PMON_CTR1) ||
+                (reg == PCI_UNC_SKX_MC_PMON_CTR2) ||
+                (reg == PCI_UNC_SKX_MC_PMON_CTR3) ||
+                (reg == PCI_UNC_SKX_MC_PMON_FIXED_CTL) ||
+                (reg == PCI_UNC_SKX_MC_PMON_FIXED_CTR) ||
+                (reg == PCI_UNC_SKX_MC_PMON_BOX_CTL) ||
+                (reg == PCI_UNC_SKX_MC_PMON_BOX_STATUS))
+                return 1;
+            break;
+        case QPI:
+            if ((reg == MSR_UNC_SKX_UPI_PMON_CTL0) ||
+                (reg == MSR_UNC_SKX_UPI_PMON_CTL1) ||
+                (reg == MSR_UNC_SKX_UPI_PMON_CTL2) ||
+                (reg == MSR_UNC_SKX_UPI_PMON_CTL3) ||
+                (reg == MSR_UNC_SKX_UPI_PMON_CTR0) ||
+                (reg == MSR_UNC_SKX_UPI_PMON_CTR1) ||
+                (reg == MSR_UNC_SKX_UPI_PMON_CTR2) ||
+                (reg == MSR_UNC_SKX_UPI_PMON_CTR3) ||
+                (reg == MSR_UNC_SKX_UPI_PMON_BOX_CTL) ||
+                (reg == MSR_UNC_SKX_UPI_PMON_BOX_STATUS))
+                return 1;
+            break;
+        case R3QPI:
+            if ((reg == MSR_UNC_SKX_M3UPI_PMON_CTL0) ||
+                (reg == MSR_UNC_SKX_M3UPI_PMON_CTL1) ||
+                (reg == MSR_UNC_SKX_M3UPI_PMON_CTL2) ||
+                (reg == MSR_UNC_SKX_M3UPI_PMON_CTR0) ||
+                (reg == MSR_UNC_SKX_M3UPI_PMON_CTR1) ||
+                (reg == MSR_UNC_SKX_M3UPI_PMON_CTR2) ||
+                (reg == MSR_UNC_SKX_M3UPI_PMON_BOX_CTL) ||
+                (reg == MSR_UNC_SKX_M3UPI_PMON_BOX_STATUS))
+                return 1;
+            break;
+    }
+    return 0;
+}
+
 static int allowed_amd(uint32_t reg)
 {
     if ( (reg & 0xFFFFFFF0U) == 0xC0010000U)
@@ -646,6 +791,26 @@ allowed_amd16(uint32_t reg)
     }
 }
 
+static int
+allowed_amd17(uint32_t reg)
+{
+    if ((reg >= 0xC0010200 && reg <= 0xC0010207) ||
+        (reg >= 0xC0010230 && reg <= 0xC001023B) ||
+        (reg >= 0xC0010299 && reg <= 0xC001029B) ||
+        (reg >= 0xC00000E7 && reg <= 0xC00000E9) ||
+        (reg >= 0xC0010240 && reg <= 0xC0010247) ||
+        (reg == 0xC0010015) ||
+        (reg == 0xC0010010) ||
+        (reg == 0xC0000080))
+    {
+        return 1;
+    }
+    else
+    {
+        return 0;
+    }
+}
+
 static void
 msr_read(AccessDataRecord * dRecord)
 {
@@ -742,6 +907,59 @@ msr_check(AccessDataRecord * dRecord)
     return;
 }
 
+static int
+getBusFromSocket(PciDevice* dev, int socket, char** filepath)
+{
+    int cur_bus = 0;
+    uint32_t cur_socket = socket;
+    char pci_filepath[1024];
+    int fp;
+    int ret = 0;
+    while(socket >= 0)
+    {
+        while (cur_bus <= 255)
+        {
+            snprintf(pci_filepath, 1023, "/proc/bus/pci/%02x/%s", cur_bus, dev->path);
+            fp = open(pci_filepath, O_RDONLY);
+            if (fp < 0)
+            {
+                cur_bus++;
+                continue;
+            }
+            if (fp > 0)
+            {
+                uint32_t indev = 0;
+                ret = pread(fp, &indev, sizeof(uint32_t), 0x0);
+                indev = (indev >> 16);
+                if (indev != dev->devid)
+                {
+                    cur_bus++;
+                    continue;
+                }
+            }
+            if (fp > 0 && socket > 0)
+            {
+                cur_bus++;
+                socket--;
+                close(fp);
+                continue;
+            }
+            
+            break;
+        }
+        socket--;
+    }
+    if (fp > 0)
+    {
+        if (filepath)
+            strncpy(*filepath, pci_filepath, strlen(pci_filepath));
+        close(fp);
+        return cur_socket;
+    }
+    return -1;
+}
+
+
 static void
 pci_read(AccessDataRecord* dRecord)
 {
@@ -749,6 +967,8 @@ pci_read(AccessDataRecord* dRecord)
     uint32_t reg = dRecord->reg;
     uint32_t device = dRecord->device;
     uint32_t data;
+    uint64_t data64;
+    char* pcipath = NULL;
 
     dRecord->errorcode = ERR_NOERROR;
     dRecord->data = 0;
@@ -770,37 +990,65 @@ pci_read(AccessDataRecord* dRecord)
     {
         if (!allowedPci(pci_devices_daemon[device].type, reg))
         {
-        dRecord->errorcode = ERR_RESTREG;
-        return;
+            dRecord->errorcode = ERR_RESTREG;
+            return;
         }
     }
     if ( !FD_PCI[socketId][device] )
     {
-        snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%s%s", PCI_ROOT_PATH, socket_bus[socketId], pci_devices_daemon[device].path);
-        FD_PCI[socketId][device] = open( pci_filepath, O_RDWR);
-
-        if ( FD_PCI[socketId][device] < 0)
+        //snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%s%s", PCI_ROOT_PATH, socket_bus[socketId], pci_devices_daemon[device].path);
+        pcipath = malloc(1024 * sizeof(char));
+        memset(pcipath, '\0', 1024 * sizeof(char));
+        int sid = getBusFromSocket(&(pci_devices_daemon[device]), socketId, &pcipath);
+        if (sid == socketId)
         {
-            syslog(LOG_ERR, "Failed to open device file %s for device %s (%s) on socket %u", pci_filepath,
-                    pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
-            dRecord->errorcode = ERR_OPENFAIL;
+#ifdef DEBUG_LIKWID
+            syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pcipath,
+                        pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
+#endif
+            FD_PCI[socketId][device] = open( pcipath, O_RDWR);
+
+            if ( FD_PCI[socketId][device] < 0)
+            {
+                syslog(LOG_ERR, "Failed to open device file %s for device %s (%s) on socket %u", pcipath, pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
+                dRecord->errorcode = ERR_OPENFAIL;
+                if (pcipath)
+                    free(pcipath);
+                return;
+            }
+        }
+    }
+
+    if (!isPCI64)
+    {
+        if (FD_PCI[socketId][device] > 0 && pread(FD_PCI[socketId][device], &data, sizeof(data), reg) != sizeof(data))
+        {
+            syslog(LOG_ERR, "Failed to read data from pci device file %s for device %s (%s) on socket %u",
+                    pcipath,pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
+            dRecord->errorcode = ERR_RWFAIL;
+            if (pcipath)
+                free(pcipath);
             return;
         }
-#ifdef DEBUG_LIKWID
-        syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pci_filepath,
-                    pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
-#endif
-    }
 
-    if (FD_PCI[socketId][device] > 0 && pread(FD_PCI[socketId][device], &data, sizeof(data), reg) != sizeof(data))
+        dRecord->data = (uint64_t) data;
+    }
+    else
     {
-        syslog(LOG_ERR, "Failed to read data from pci device file %s for device %s (%s) on socket %u",
-                pci_filepath,pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name,socketId);
-        dRecord->errorcode = ERR_RWFAIL;
-        return;
-    }
+        if (FD_PCI[socketId][device] > 0 && pread(FD_PCI[socketId][device], &data64, sizeof(data64), reg) != sizeof(data64))
+        {
+            syslog(LOG_ERR, "Failed to read data from pci device file %s for device %s (%s) on socket %u",
+                    pcipath,pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
+            dRecord->errorcode = ERR_RWFAIL;
+            if (pcipath)
+                free(pcipath);
+            return;
+        }
 
-    dRecord->data = (uint64_t) data;
+        dRecord->data = data64;
+    }
+    if (pcipath)
+        free(pcipath);
 }
 
 static void
@@ -809,7 +1057,9 @@ pci_write(AccessDataRecord* dRecord)
     uint32_t socketId = dRecord->cpu;
     uint32_t reg = dRecord->reg;
     uint32_t device = dRecord->device;
-    uint32_t data = (uint32_t) dRecord->data;
+    uint32_t data = (uint32_t)dRecord->data;
+    uint64_t data64 = dRecord->data;
+    char* pcipath = NULL;
 
     dRecord->errorcode = ERR_NOERROR;
 
@@ -837,30 +1087,56 @@ pci_write(AccessDataRecord* dRecord)
 
     if ( !FD_PCI[socketId][device] )
     {
-        snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%s%s", PCI_ROOT_PATH, socket_bus[socketId], pci_devices_daemon[device].path);
-
-        FD_PCI[socketId][device] = open( pci_filepath, O_RDWR);
-
-        if ( FD_PCI[socketId][device] < 0)
+        //snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%s%s", PCI_ROOT_PATH, socket_bus[socketId], pci_devices_daemon[device].path);
+        pcipath = malloc(1024 * sizeof(char));
+        memset(pcipath, '\0', 1024 * sizeof(char));
+        int sid = getBusFromSocket(&(pci_devices_daemon[device]), socketId, &pcipath);
+        if (sid == socketId)
         {
-            syslog(LOG_ERR, "Failed to open device file %s for device %s (%s) on socket %u", pci_filepath,
+#ifdef DEBUG_LIKWID
+            syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pcipath,
                         pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
-            dRecord->errorcode = ERR_OPENFAIL;
+#endif
+            FD_PCI[socketId][device] = open( pcipath, O_RDWR);
+
+            if ( FD_PCI[socketId][device] < 0)
+            {
+                syslog(LOG_ERR, "Failed to open device file %s for device %s (%s) on socket %u", pcipath, pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
+                dRecord->errorcode = ERR_OPENFAIL;
+                if (pcipath)
+                    free(pcipath);
+                return;
+            }
+        }
+        
+    }
+
+    if (!isPCI64)
+    {
+        if (FD_PCI[socketId][device] > 0 && pwrite(FD_PCI[socketId][device], &data, sizeof data, reg) != sizeof data)
+        {
+            syslog(LOG_ERR, "Failed to write data to pci device file %s for device %s (%s) on socket %u",pci_filepath,
+                    pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
+            dRecord->errorcode = ERR_RWFAIL;
+            if (pcipath)
+                free(pcipath);
             return;
         }
-#ifdef DEBUG_LIKWID
-        syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pci_filepath,
-                    pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
-#endif
     }
-
-    if (FD_PCI[socketId][device] > 0 && pwrite(FD_PCI[socketId][device], &data, sizeof data, reg) != sizeof data)
+    else
     {
-        syslog(LOG_ERR, "Failed to write data to pci device file %s for device %s (%s) on socket %u",pci_filepath,
-                pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
-        dRecord->errorcode = ERR_RWFAIL;
-        return;
+        if (FD_PCI[socketId][device] > 0 && pwrite(FD_PCI[socketId][device], &data64, sizeof data64, reg) != sizeof data64)
+        {
+            syslog(LOG_ERR, "Failed to write data to pci device file %s for device %s (%s) on socket %u",pci_filepath,
+                    pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
+            dRecord->errorcode = ERR_RWFAIL;
+            if (pcipath)
+                free(pcipath);
+            return;
+        }
     }
+    if (pcipath)
+        free(pcipath);
 }
 
 static void
@@ -912,7 +1188,7 @@ stop_daemon(void)
 }
 
 static int
-getBusFromSocket(const uint32_t socket)
+getBusFromSocketOld(const uint32_t socket, const char* name, const uint32_t in_dev_id)
 {
     int cur_bus = 0;
     uint32_t cur_socket = 0;
@@ -921,7 +1197,7 @@ getBusFromSocket(const uint32_t socket)
     int ret = 0;
     while(cur_socket <= socket)
     {
-        snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%02x/05.0", PCI_ROOT_PATH, cur_bus);
+        snprintf(pci_filepath, MAX_PATH_LENGTH-1, "%s%02x/%s", PCI_ROOT_PATH, cur_bus, name);
         fp = open(pci_filepath, O_RDONLY);
         if (fp < 0)
         {
@@ -947,6 +1223,93 @@ getBusFromSocket(const uint32_t socket)
     return -1;
 }
 
+#define SKYLAKE_SERVER_SOCKETID_MBOX_DID 0x2042
+static int getBusFromSocketSysFS(const uint32_t socket, const char* name, const uint32_t in_dev_id)
+{
+    struct dirent *pDirent, *pDirentInner;
+    DIR *pDir, *pDirInner;
+    pDir = opendir ("/sys/devices");
+    FILE* fp = NULL;
+    char iPath[200], iiPath[200], buff[100];
+    char testDev[50];
+    size_t ret = 0;
+    int bus_id = -1;
+    int numa_ctr = 0;
+    if (pDir == NULL)
+    {
+        syslog(LOG_ERR, "Failed open directory /sys/devices");
+        return -1;
+    }
+    
+    while ((pDirent = readdir(pDir)) != NULL)
+    {
+        if (strncmp(pDirent->d_name, "pci0", 4) == 0)
+        {
+            sprintf(iPath, "/sys/devices/%s", pDirent->d_name);
+            char bus[4];
+            strncpy(bus, &(pDirent->d_name[strlen(pDirent->d_name)-2]), 2);
+            bus[2] = '\0';
+            
+            pDirInner = opendir (iPath);
+            if (pDir == NULL)
+            {
+                syslog(LOG_ERR, "Failed read file %s", iPath);
+                return -1;
+            }
+            while ((pDirentInner = readdir(pDirInner)) != NULL)
+            {
+                if (strncmp(pDirentInner->d_name, "0000", 4) == 0)
+                {
+                    uint32_t dev_id = 0x0;
+                    int numa_node = 0;
+                    sprintf(iiPath, "/sys/devices/%s/%s/device", pDirent->d_name, pDirentInner->d_name);
+                    fp = fopen(iiPath,"r");
+                    if( fp == NULL )
+                    {
+                        continue;
+                    }
+                    ret = fread(buff, sizeof(char), 99, fp);
+                    dev_id = strtoul(buff, NULL, 16);
+                    if (dev_id == in_dev_id)
+                    {
+                        fclose(fp);
+                        iiPath[0] = '\0';
+                        sprintf(iiPath, "/sys/devices/%s/%s/numa_node", pDirent->d_name, pDirentInner->d_name);
+                        fp = fopen(iiPath,"r");
+                        if( fp == NULL )
+                        {
+                            continue;
+                        }
+                        ret = fread(buff, sizeof(char), 99, fp);
+                        numa_node = atoi(buff);
+                        if (numa_node < 0)
+                        {
+                            numa_node = numa_ctr;
+                            numa_ctr++;
+                        }
+                        if (numa_node == socket)
+                        {
+                            bus_id = strtoul(bus, NULL, 16);
+                        }
+                    }
+                    fclose(fp);
+                    iiPath[0] = '\0';
+                    buff[0] = '\0';
+                    if (bus_id != -1)
+                        break;
+                }
+            }
+            closedir (pDirInner);
+            iPath[0] = '\0';
+            if (bus_id != -1)
+                break;
+        }
+    }
+    closedir (pDir);
+    return bus_id;
+}
+
+
 static void
 Signal_Handler(int sig)
 {
@@ -1027,7 +1390,7 @@ int main(void)
     mode_t oldumask;
     uint32_t numHWThreads = sysconf(_SC_NPROCESSORS_CONF);
     uint32_t model;
-    for (int i=0;i<MAX_NUM_THREADS;i++)
+    for (int i=0;i<avail_cpus;i++)
     {
         FD_MSR[i] = -1;
     }
@@ -1039,6 +1402,11 @@ int main(void)
         syslog(LOG_ERR,"Access to performance counters is locked.\n");
         stop_daemon();
     }
+    if (!FD_MSR || !FD_PCI || !socket_bus)
+    {
+        syslog(LOG_ERR,"Failed to allocate FD space.\n");
+        stop_daemon();
+    }
 
     daemonize(&pid);
     syslog(LOG_INFO, "AccessDaemon runs with UID %d, eUID %d\n", getuid(), geteuid());
@@ -1108,6 +1476,13 @@ int main(void)
                     allowed = allowed_sandybridge;
                     allowedPci = allowed_pci_haswell;
                 }
+                else if (model == SKYLAKEX)
+                {
+                    isPCIUncore = 1;
+                    allowed = allowed_skx;
+                    allowedPci = allowed_pci_skx;
+                    isPCI64 = 1;
+                }
                 else if ((model == ATOM_SILVERMONT_C) ||
                          (model == ATOM_SILVERMONT_E) ||
                          (model == ATOM_SILVERMONT_Z1) ||
@@ -1133,7 +1508,10 @@ int main(void)
                 break;
             case K16_FAMILY:
                 allowed = allowed_amd16;
-            break;
+                break;
+            case ZEN_FAMILY:
+                allowed = allowed_amd17;
+                break;
             default:
                 syslog(LOG_ERR, "ERROR - [%s:%d] - Unsupported processor. Exiting!  \n",
                         __FILE__, __LINE__);
@@ -1204,7 +1582,7 @@ int main(void)
 
         /* Open MSR device files for less overhead.
          * NOTICE: This assumes consecutive processor Ids! */
-        for ( uint32_t i=0; i < numHWThreads; i++ )
+        for ( uint32_t i=0; i < avail_cpus; i++ )
         {
             sprintf(msr_file_name,"/dev/cpu/%d/msr",i);
             FD_MSR[i] = open(msr_file_name, O_RDWR);
@@ -1251,6 +1629,11 @@ int main(void)
                 //testDevice = 0x80862f30;
                 pci_devices_daemon = broadwellEP_pci_devices;
             }
+            else if (model == SKYLAKEX)
+            {
+                //testDevice = 0x80862f30;
+                pci_devices_daemon = skylakeX_pci_devices;
+            }
             else if (model == XEON_PHI_KNL)
             {
                 pci_devices_daemon = knl_pci_devices;
@@ -1262,25 +1645,48 @@ int main(void)
                 goto LOOP;
             }
 
-            for (int j=0; j<MAX_NUM_NODES; j++)
+            for (int j=0; j<avail_sockets; j++)
             {
-                socket_bus[j] = (char*)malloc(4);
-                sprintf(socket_bus[j], "N-A");
+                //socket_bus[j] = (char*)malloc(4);
+                //sprintf(socket_bus[j], "N-A");
                 for (int i=0; i<MAX_NUM_PCI_DEVICES; i++)
                 {
                     FD_PCI[j][i] = -2;
                 }
             }
+            
+            for (int i=1; i<MAX_NUM_PCI_DEVICES; i++)
+            {
+                if (pci_devices_daemon[i].path && strlen(pci_devices_daemon[i].path) > 0)
+                {
+                    int socket_id = getBusFromSocket(&(pci_devices_daemon[i]), 0, NULL);
+                    if (socket_id == 0)
+                    {
+                        for (int j=0; j<avail_sockets; j++)
+                        {
+                            FD_PCI[j][i] = 0;
+                        }
+                        pci_devices_daemon[i].online = 1;
+                    }
+                    else
+                    {
+                        syslog(LOG_ERR, "Device %s for socket %d, excluded it from device list: %s\n",pci_devices_daemon[i].name, 0, strerror(errno));
+                    }
+                }
+            }
 
             /* determine PCI-BUSID mapping ... */
+            /*syslog(LOG_ERR, "Determine PCI-BUSID mapping");
             int sbus = -1;
             cntr = 0;
-            sbus = getBusFromSocket(cntr);
-            while (sbus != -1)
+            sbus = getSocketBus(cntr);
+            syslog(LOG_ERR, "Socket %d Bus %02x", cntr, sbus);
+            while (sbus != -1 && sbus < 0xFF)
             {
                 sprintf(socket_bus[cntr], "%02x/", sbus);
                 cntr++;
-                sbus = getBusFromSocket(cntr);
+                sbus = getSocketBus(cntr);
+                syslog(LOG_ERR, "Socket %d Bus %02x", cntr, sbus);
             }
 
             if ( cntr == 0 )
@@ -1308,13 +1714,13 @@ int main(void)
 #ifdef DEBUG_LIKWID
                             else if (j==0)
                             {
-                                syslog(LOG_NOTICE, "Device %s for socket %d not found at path %s, excluded it from device list: %s\n",pci_devices_daemon[i].name,j, pci_filepath, strerror(errno));
+                                syslog(LOG_ERR, "Device %s for socket %d not found at path %s, excluded it from device list: %s\n",pci_devices_daemon[i].name,j, pci_filepath, strerror(errno));
                             }
 #endif
                         }
                     }
                 }
-            }
+            }*/
         }
     }
 LOOP:
