@@ -5,13 +5,13 @@
  *
  *      Description:  Interface for the different register access modules.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -57,7 +57,7 @@
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 static int registeredCpus = 0;
-static int registeredCpuList[MAX_NUM_THREADS] = { [0 ... (MAX_NUM_THREADS-1)] = 0 };
+static int *registeredCpuList = NULL;
 static int (*access_read)(PciDeviceIndex dev, const int cpu, uint32_t reg, uint64_t *data) = NULL;
 static int (*access_write)(PciDeviceIndex dev, const int cpu, uint32_t reg, uint64_t data) = NULL;
 static int (*access_init) (int cpu_id) = NULL;
@@ -79,6 +79,12 @@ int
 HPMinit(void)
 {
     int ret = 0;
+    if (registeredCpuList == NULL)
+    {
+        registeredCpuList = malloc(cpuid_topology.numHWThreads* sizeof(int));
+        memset(registeredCpuList, 0, cpuid_topology.numHWThreads* sizeof(int));
+        registeredCpus = 0;
+    }
     if (access_init == NULL)
     {
 #if defined(__x86_64__) || defined(__i386__)
@@ -162,6 +168,11 @@ HPMfinalize()
                 registeredCpuList[i] = 0;
             }
         }
+        if (registeredCpuList && registeredCpus == 0)
+        {
+            free(registeredCpuList);
+            registeredCpuList = NULL;
+        }
     }
     if (access_init != NULL)
         access_init = NULL;
