@@ -9,14 +9,14 @@
  *                   sys interface of the Linux 2.6 kernel. This module
  *                   is based on the msr-util tools.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com.
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -59,13 +59,13 @@
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
-#define MAX_LENGTH_MSR_DEV_NAME  20
+#define MAX_LENGTH_MSR_DEV_NAME  24
 #define STRINGIFY(x) #x
 #define TOSTRING(x) STRINGIFY(x)
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
-static int FD[MAX_NUM_THREADS] = { [0 ... MAX_NUM_THREADS-1] = -1 };
+static int *FD = NULL;
 static int rdpmc_works_pmc = -1;
 static int rdpmc_works_fixed = -1;
 
@@ -147,6 +147,11 @@ access_x86_msr_init(const int cpu_id)
     int i = 0;
 
     char* msr_file_name;
+    if (!FD)
+    {
+        FD = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(FD, -1, cpuid_topology.numHWThreads * sizeof(int));
+    }
     if (FD[cpu_id] > 0)
     {
         return 0;
@@ -169,6 +174,12 @@ access_x86_msr_init(const int cpu_id)
         }
         else
         {
+            if(geteuid() != 0 && cpuid_info.supportUncore)
+            {
+                fprintf(stdout, "Using msr_safe kernel module. Currently, this deactivates the\n");
+                fprintf(stdout, "PCI-based Uncore monitoring.\n");
+                cpuid_info.supportUncore = 0;
+            }
             close(fd);
         }
     }
@@ -204,7 +215,17 @@ access_x86_msr_init(const int cpu_id)
     fd = open(msr_file_name, O_RDWR);
     if (fd < 0)
     {
-        sprintf(msr_file_name,"/dev/cpu/%d/msr",cpu_id);
+        sprintf(msr_file_name,"/dev/cpu/%d/msr_safe", cpu_id);
+        fd = open(msr_file_name, O_RDWR);
+        if (fd < 0)
+        {
+            sprintf(msr_file_name,"/dev/cpu/%d/msr", cpu_id);
+            printf("%s\n", msr_file_name);
+        }
+        else
+        {
+            close(fd);
+        }
     }
     else
     {
@@ -228,11 +249,16 @@ access_x86_msr_finalize(const int cpu_id)
 {
     int i = 0;
 
-    if (FD[cpu_id] > 0)
+    if (FD && FD[cpu_id] > 0)
     {
         close(FD[cpu_id]);
         FD[cpu_id] = 0;
     }
+    if (FD)
+    {
+        free(FD);
+        FD = NULL;
+    }
 }
 
 int
