@@ -5,14 +5,14 @@
  *
  *      Description:  Implementation of affinity module.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -49,13 +49,17 @@
 #include <likwid.h>
 #include <numa.h>
 #include <affinity.h>
+#include <lock.h>
 #include <tree.h>
 #include <topology.h>
+#include <topology_hwloc.h>
 
 /* #####   EXPORTED VARIABLES   ########################################### */
 
-int affinity_core2node_lookup[MAX_NUM_THREADS];
-int affinity_thread2core_lookup[MAX_NUM_THREADS];
+int *affinity_thread2core_lookup = NULL;
+int *affinity_thread2socket_lookup = NULL;
+int *affinity_thread2numa_lookup = NULL;
+int *affinity_thread2sharedl3_lookup = NULL;
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
@@ -75,8 +79,9 @@ static int
 getProcessorID(cpu_set_t* cpu_set)
 {
     int processorId;
+    topology_init();
 
-    for ( processorId = 0; processorId < MAX_NUM_THREADS; processorId++ )
+    for ( processorId = 0; processorId < cpuid_topology.numHWThreads; processorId++ )
     {
         if ( CPU_ISSET(processorId,cpu_set) )
         {
@@ -150,6 +155,126 @@ treeFillNextEntries(
     }
     return numberOfEntries-counter;
 }
+
+static int get_id_of_type(hwloc_obj_t base, hwloc_obj_type_t type)
+{
+    hwloc_obj_t walker = base->parent;
+    while (walker && walker->type != type)
+        walker = walker->parent;
+    if (walker && walker->type == type)
+        return walker->os_index;
+    return -1;
+}
+
+static int create_lookups()
+{
+    topology_init();
+    if (!affinity_thread2core_lookup)
+    {
+        affinity_thread2core_lookup = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(affinity_thread2core_lookup, -1, cpuid_topology.numHWThreads*sizeof(int));
+    }
+    if (!affinity_thread2socket_lookup)
+    {
+        affinity_thread2socket_lookup = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(affinity_thread2socket_lookup, -1, cpuid_topology.numHWThreads*sizeof(int));
+    }
+    if (!affinity_thread2sharedl3_lookup)
+    {
+        affinity_thread2sharedl3_lookup = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(affinity_thread2sharedl3_lookup, -1, cpuid_topology.numHWThreads*sizeof(int));
+    }
+    if (!affinity_thread2numa_lookup)
+    {
+        affinity_thread2numa_lookup = malloc(cpuid_topology.numHWThreads * sizeof(int));
+        memset(affinity_thread2numa_lookup, -1, cpuid_topology.numHWThreads*sizeof(int));
+    }
+
+    int num_pu = likwid_hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PU);
+    for (int pu_idx = 0; pu_idx < num_pu; pu_idx++)
+    {
+        hwloc_obj_t pu = likwid_hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_PU, pu_idx);
+        int hwthreadid = pu->os_index;
+        int coreid = get_id_of_type(pu, HWLOC_OBJ_CORE);
+        int sockid = get_id_of_type(pu, HWLOC_OBJ_PACKAGE);
+        int memid = get_id_of_type(pu, HWLOC_OBJ_NUMANODE);
+        affinity_thread2core_lookup[hwthreadid] = coreid;
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, affinity_thread2core_lookup[%d] = %d, hwthreadid, coreid);
+        affinity_thread2socket_lookup[hwthreadid] = sockid;
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, affinity_thread2socket_lookup[%d] = %d, hwthreadid, sockid);
+        affinity_thread2numa_lookup[hwthreadid] = memid;
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, affinity_thread2numa_lookup[%d] = %d, hwthreadid, memid);
+    }
+    int maxNumLevels = 0;
+    int depth = likwid_hwloc_topology_get_depth(hwloc_topology);
+    for (int d = 1; d <= depth; d++)
+    {
+        if (likwid_hwloc_get_depth_type(hwloc_topology, d) == HWLOC_OBJ_CACHE)
+            maxNumLevels++;
+    }
+    for(int d=depth-1;d >= 1; d--)
+    {
+        if (likwid_hwloc_get_depth_type(hwloc_topology, d) == HWLOC_OBJ_CACHE)
+        {
+            hwloc_obj_t cobj = likwid_hwloc_get_obj_by_depth(hwloc_topology, d, 0);
+            if (cobj->attr->cache.depth != 3)
+            {
+                continue;
+            }
+            int num_caches = likwid_hwloc_get_nbobjs_by_depth(hwloc_topology, d);
+            for (int c = 0; c < num_caches; c++)
+            {
+                cobj = likwid_hwloc_get_obj_by_depth(hwloc_topology, d, c);
+                for (int i = 0; i < num_pu; i++)
+                {
+                    if (likwid_hwloc_bitmap_isset(cobj->cpuset, i))
+                    {
+                        affinity_thread2sharedl3_lookup[i] = c;
+                        DEBUG_PRINT(DEBUGLEV_DEVELOP, affinity_thread2sharedl3_lookup[%d] = %d, i, c);
+                    }
+                }
+            }
+        }
+    }
+    return 0;
+}
+
+/*static int create_locks()*/
+/*{*/
+/*    numa_init();*/
+/*    if (!socket_lock)*/
+/*    {*/
+/*        socket_lock = malloc(cpuid_topology.numSockets * sizeof(int));*/
+/*        memset(socket_lock, LOCK_INIT, cpuid_topology.numSockets*sizeof(int));*/
+/*    }*/
+/*    if (!tile_lock)*/
+/*    {*/
+/*        tile_lock = malloc(cpuid_topology.numHWThreads * sizeof(int));*/
+/*        memset(tile_lock, LOCK_INIT, cpuid_topology.numHWThreads*sizeof(int));*/
+/*    }*/
+/*    if (!numa_lock)*/
+/*    {*/
+/*        numa_lock = malloc(numa_info.numberOfNodes * sizeof(int));*/
+/*        memset(numa_lock, LOCK_INIT, numa_info.numberOfNodes*sizeof(int));*/
+/*    }*/
+/*    if (!core_lock)*/
+/*    {*/
+/*        int cores = (cpuid_topology.numHWThreads/cpuid_topology.numThreadsPerCore);*/
+/*        core_lock = malloc(cores * sizeof(int));*/
+/*        memset(core_lock, LOCK_INIT, cores*sizeof(int));*/
+/*    }*/
+/*    if (!sharedl2_lock)*/
+/*    {*/
+/*        sharedl2_lock = malloc(cpuid_topology.numHWThreads * sizeof(int));*/
+/*        memset(sharedl2_lock, LOCK_INIT, cpuid_topology.numHWThreads*sizeof(int));*/
+/*    }*/
+/*    if (!sharedl3_lock)*/
+/*    {*/
+/*        sharedl3_lock = malloc(cpuid_topology.numHWThreads * sizeof(int));*/
+/*        memset(sharedl3_lock, LOCK_INIT, cpuid_topology.numHWThreads*sizeof(int));*/
+/*    }*/
+/*}*/
+
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
 void
@@ -165,7 +290,11 @@ affinity_init()
         return;
     }
     topology_init();
+
+    create_lookups();
+    //create_locks();
     int numberOfSocketDomains = cpuid_topology.numSockets;
+
     DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity: Socket domains %d, numberOfSocketDomains);
     numa_init();
     int numberOfNumaDomains = numa_info.numberOfNodes;
@@ -193,6 +322,7 @@ affinity_init()
     /* determine total number of domains */
     numberOfDomains += numberOfSocketDomains + numberOfCacheDomains + numberOfNumaDomains;
     DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity: All domains %d, numberOfDomains);
+
     domains = (AffinityDomain*) malloc(numberOfDomains * sizeof(AffinityDomain));
     if (!domains)
     {
@@ -200,6 +330,7 @@ affinity_init()
         return;
     }
 
+
     /* Node domain */
     domains[0].numberOfProcessors = cpuid_topology.activeHWThreads;
     domains[0].numberOfCores = cpuid_topology.numSockets * cpuid_topology.numCoresPerSocket;
@@ -209,12 +340,11 @@ affinity_init()
     if (!domains[0].processorList)
     {
         fprintf(stderr,"No more memory for %ld bytes for processor list of affinity domain %s\n",
-                cpuid_topology.numHWThreads*sizeof(int), 
+                cpuid_topology.numHWThreads*sizeof(int),
                 bdata(domains[0].tag));
         return;
     }
     offset = 0;
-
     if (numberOfSocketDomains > 1)
     {
         for (int i=0; i<numberOfSocketDomains; i++)
@@ -222,8 +352,7 @@ affinity_init()
           tmp = treeFillNextEntries(cpuid_topology.topologyTree,
                                     domains[0].processorList, offset,
                                     i, 0,
-                                    cpuid_topology.numCoresPerSocket,
-                                    numberOfProcessorsPerSocket);
+                                    cpuid_topology.numCoresPerSocket, numberOfProcessorsPerSocket);
           offset += tmp;
         }
     }
@@ -232,13 +361,13 @@ affinity_init()
         tmp = treeFillNextEntries(cpuid_topology.topologyTree,
                                   domains[0].processorList, 0,
                                   0, 0,
-                                  domains[0].numberOfCores,
-                                  domains[0].numberOfProcessors);
+                                  domains[0].numberOfCores, domains[0].numberOfProcessors);
         domains[0].numberOfProcessors = tmp;
     }
 
     /* Socket domains */
     currentDomain = 1;
+    tmp = 0;
     for (int i=0; i < numberOfSocketDomains; i++ )
     {
         domains[currentDomain + i].numberOfProcessors = numberOfProcessorsPerSocket;
@@ -246,6 +375,7 @@ affinity_init()
         domains[currentDomain + i].tag = bformat("S%d", i);
         DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain S%d: %d HW threads on %d cores, i, domains[currentDomain + i].numberOfProcessors, domains[currentDomain + i].numberOfCores);
         domains[currentDomain + i].processorList = (int*) malloc( domains[currentDomain + i].numberOfProcessors * sizeof(int));
+        memset(domains[currentDomain + i].processorList, 0, domains[currentDomain + i].numberOfProcessors * sizeof(int));
         if (!domains[currentDomain + i].processorList)
         {
             fprintf(stderr,"No more memory for %ld bytes for processor list of affinity domain %s\n",
@@ -253,16 +383,11 @@ affinity_init()
                     bdata(domains[currentDomain + i].tag));
             return;
         }
-
         tmp = treeFillNextEntries(cpuid_topology.topologyTree,
                                   domains[currentDomain + i].processorList, 0,
                                   i, 0, cpuid_topology.numCoresPerSocket,
                                   domains[currentDomain + i].numberOfProcessors);
         tmp = MIN(tmp, domains[currentDomain + i].numberOfProcessors);
-        for ( int j = 0; j < tmp; j++ )
-        {
-            affinity_core2node_lookup[domains[currentDomain + i].processorList[j]] = i;
-        }
         domains[currentDomain + i].numberOfProcessors = tmp;
     }
 
@@ -280,14 +405,13 @@ affinity_init()
             domains[currentDomain + subCounter].tag = bformat("C%d", subCounter);
             DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain C%d: %d HW threads on %d cores, subCounter, domains[currentDomain + subCounter].numberOfProcessors, domains[currentDomain + subCounter].numberOfCores);
             domains[currentDomain + subCounter].processorList = (int*) malloc(numberOfProcessorsPerCache*sizeof(int));
-            if (!domains[currentDomain + subCounter].processorList)   
+            if (!domains[currentDomain + subCounter].processorList)
             {
                 fprintf(stderr,"No more memory for %ld bytes for processor list of affinity domain %s\n",
                         numberOfProcessorsPerCache*sizeof(int),
                         bdata(domains[currentDomain + subCounter].tag));
                 return;
             }
-
             tmp = treeFillNextEntries(cpuid_topology.topologyTree,
                                       domains[currentDomain + subCounter].processorList, 0,
                                       i, offset, numberOfCoresPerCache,
@@ -331,14 +455,10 @@ affinity_init()
                             bdata(domains[currentDomain + subCounter].tag));
                     return;
                 }
-                // Skip memory domain if all CPUs are already attached to others
-                // This happens for example on Intel Xeon Phi (KNL) where the
-                // NUMA domains of the MCDRAM don't have CPUs attached.
                 if (offset >= cpuid_topology.numCoresPerSocket*cpuid_topology.numSockets)
                 {
                     continue;
                 }
-
                 tmp = treeFillNextEntries(cpuid_topology.topologyTree,
                                           domains[currentDomain + subCounter].processorList, 0,
                                           i, offset, domains[currentDomain + subCounter].numberOfCores,
@@ -363,7 +483,6 @@ affinity_init()
                 domains[currentDomain + subCounter].numberOfCores);
 
         domains[currentDomain + subCounter].processorList = (int*) malloc(NUMAthreads*sizeof(int));
-
         if (!domains[currentDomain + subCounter].processorList)
         {
             fprintf(stderr,"No more memory for %ld bytes for processor list of affinity domain %s\n",
@@ -419,7 +538,41 @@ affinity_finalize()
     if (affinityDomains.domains != NULL)
     {
         free(affinityDomains.domains);
+        affinityDomains.domains = NULL;
     }
+    if (affinity_thread2core_lookup)
+    {
+        free(affinity_thread2core_lookup);
+        affinity_thread2core_lookup = NULL;
+    }
+    if (affinity_thread2socket_lookup)
+    {
+        free(affinity_thread2socket_lookup);
+        affinity_thread2socket_lookup = NULL;
+    }
+    if (affinity_thread2sharedl3_lookup)
+    {
+        free(affinity_thread2sharedl3_lookup);
+        affinity_thread2sharedl3_lookup = NULL;
+    }
+    if (affinity_thread2numa_lookup)
+    {
+        free(affinity_thread2numa_lookup);
+        affinity_thread2numa_lookup = NULL;
+    }
+/*    if (socket_lock)*/
+/*        free(socket_lock);*/
+/*    if (core_lock)*/
+/*        free(core_lock);*/
+/*    if (tile_lock)*/
+/*        free(tile_lock);*/
+/*    if (numa_lock)*/
+/*        free(numa_lock);*/
+/*    if (sharedl2_lock)*/
+/*        free(sharedl2_lock);*/
+/*    if (sharedl3_lock)*/
+/*        free(sharedl3_lock);*/
+
     affinityDomains.domains = NULL;
     affinity_numberOfDomains = 0;
     affinityDomains.numberOfAffinityDomains = 0;
