@@ -7,13 +7,13 @@
  *      Description: A wrapper script to pin threads spawned by MPI processes and
  *                   measure hardware performance counters
  *
- *      Version:   <VERSION>
- *      Released:  <DATE>
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -80,6 +80,7 @@ local function usage()
     print_stdout("-m/-marker\t\t Activate marker API mode")
     print_stdout("-O\t\t\t Output easily parseable CSV instead of fancy tables")
     print_stdout("-f\t\t\t Force overwrite of registers if they are in use. You can also use environment variable LIKWID_FORCE")
+    print_stdout("-e, --env <key>=<value>\t Set environment variables for MPI processes")
     print_stdout("")
     print_stdout("Processes are pinned to physical CPU cores first. For syntax questions see likwid-pin")
     print_stdout("")
@@ -92,6 +93,7 @@ end
 
 local np = 0
 local ppn = 0
+local tpp = 1
 local nperdomain = nil
 local npernode = 0
 local cpuexprs = {}
@@ -103,8 +105,10 @@ local mpitype = nil
 local omptype = nil
 local skipStr = ""
 local executable = {}
+local envsettings = {}
 local mpiopts = {}
 local debug = false
+local likwiddebug = false
 local use_marker = false
 local use_csv = false
 local force = false
@@ -174,8 +178,14 @@ local function readHostfileOpenMPI(filename)
         if host["maxslots"] == nil or host["maxslots"] == 0 then
             host["maxslots"] = topo.numHWThreads
         end
-        if debug then
-            print_stdout(string.format("DEBUG: Read host %s with %d slots and %d slots maximally", host["hostname"], host["slots"], host["maxslots"]))
+    end
+    if debug then
+        print_stdout("Available hosts for scheduling:")
+        s = string.format("%-20s\t%s\t%s\t%s", "Host", "Slots", "MaxSlots", "Interface")
+        print_stdout(s)
+        for i, host in pairs(hostlist) do
+            s = string.format("%-20s\t%s\t%s\t%s", host["hostname"], host["slots"], host["maxslots"],"")
+            print_stdout (s)
         end
     end
     return hostlist
@@ -214,18 +224,16 @@ local function executeOpenMPI(wrapperscript, hostfile, env, nrNodes)
         wrapperscript = os.getenv("PWD").."/"..wrapperscript
     end
 
-    local f = io.popen(string.format("%s -V 2>&1", mpiexecutable), "r")
-    if f ~= nil then
-        local input = f:read("*a")
-        ver1,ver2,ver3 = input:match("(%d+)%.(%d+)%.(%d+)")
-        if ver1 == "1" then
-            if tonumber(ver2) >= 7 then
-                bindstr = "--bind-to none"
-            elseif ver2 == "6" then
-                bindstr = "--bind-to-none"
-            end
+
+    ver1,ver2 = getMpiVersion()
+    if ver1 == 1 then
+        if ver2 >= 7 then
+            bindstr = "--bind-to none"
+        elseif ver2 == "6" then
+            bindstr = "--bind-to-none"
         end
-        f:close()
+    elseif ver1 == 2 then
+        bindstr = "--bind-to none"
     end
 
     local cmd = string.format("%s -hostfile %s %s -np %d -npernode %d %s %s",
@@ -265,8 +273,12 @@ local function readHostfileIntelMPI(filename)
         end
     end
     if debug then
+        print_stdout("Available hosts for scheduling:")
+        s = string.format("%-20s\t%s\t%s\t%s", "Host", "Slots", "MaxSlots", "Interface")
+        print_stdout(s)
         for i, host in pairs(hostlist) do
-            print_stdout(string.format("DEBUG: Read host %s with %d slots and %d slots maximally", host["hostname"], host["slots"], host["maxslots"]))
+            s = string.format("%-20s\t%s\t%s\t%s", host["hostname"], host["slots"], host["maxslots"],"")
+            print_stdout (s)
         end
     end
     return hostlist
@@ -394,8 +406,14 @@ local function readHostfileMvapich2(filename)
         end
     end
     if debug then
-        for i, host in pairs(hostlist) do
-            print_stdout(string.format("DEBUG: Read host %s with %d slots and %d slots maximally", host["hostname"], host["slots"], host["maxslots"]))
+        if debug then
+            print_stdout("Available hosts for scheduling:")
+            s = string.format("%-20s\t%s\t%s\t%s", "Host", "Slots", "MaxSlots", "Interface")
+            print_stdout(s)
+            for i, host in pairs(hostlist) do
+                s = string.format("%-20s\t%s\t%s\t%s", host["hostname"], host["slots"], host["maxslots"],"")
+                print_stdout (s)
+            end
         end
     end
     return hostlist
@@ -487,8 +505,12 @@ local function readHostfilePBS(filename)
         end
     end
     if debug then
+        print_stdout("Available hosts for scheduling:")
+        s = string.format("%-20s\t%s\t%s\t%s", "Host", "Slots", "MaxSlots", "Interface")
+        print_stdout(s)
         for i, host in pairs(hostlist) do
-            print_stdout(string.format("DEBUG: Read host %s with %d slots and %d slots maximally", host["hostname"], host["slots"], host["maxslots"]))
+            s = string.format("%-20s\t%s\t%s\t%s", host["hostname"], host["slots"], host["maxslots"],"")
+            print_stdout (s)
         end
     end
     return hostlist
@@ -499,6 +521,15 @@ local function readHostfileSlurm(hostlist)
     if hostlist and nperhost then
         hostfile = write_hostlist_to_file(hostlist, nperhost)
         hosts = readHostfilePBS(hostfile)
+        if debug then
+            print_stdout("Available hosts for scheduling:")
+            s = string.format("%-20s\t%s\t%s\t%s", "Host", "Slots", "MaxSlots", "Interface")
+            print_stdout(s)
+            for i, host in pairs(hosts) do
+                s = string.format("%-20s\t%s\t%s\t%s", host["hostname"], host["slots"], host["maxslots"],"")
+                print_stdout (s)
+            end
+        end
         os.remove(hostfile)
     end
     return hosts
@@ -651,6 +682,40 @@ local function getMpiType()
     return mpitype
 end
 
+function getMpiVersion()
+    maj = nil
+    min = nil
+    intel_match = "Version (%d+) Update (%d+)"
+    openmpi_match = "(%d+)%.(%d+)%.%d+"
+    for i, exec in pairs({"mpiexec.hydra", "mpiexec", "mpirun"}) do
+        f = io.popen(string.format("which %s 2>/dev/null", exec), 'r')
+        if f ~= nil then
+            mpiexec = f:read("*line")
+            if mpiexec then
+                cmd = mpiexec .. " --version"
+                f:close()
+                f = io.popen(cmd, 'r')
+                if f ~= nil then
+                    local t = f:read("*all")
+                    f:close()
+                    for l in t:gmatch("[^\r\n]+") do
+                        if l:match(intel_match) then
+                            maj, min = l:match(intel_match)
+                            maj = tonumber(maj)
+                            min = tonumber(min)
+                        elseif l:match(openmpi_match) then
+                            maj, min = l:match(openmpi_match)
+                            maj = tonumber(maj)
+                            min = tonumber(min)
+                        end
+                    end
+                end
+            end
+        end
+    end
+    return maj, min
+end
+
 local function getMpiExec(mpitype)
     testing = {}
     if mpitype == "intelmpi" then
@@ -746,23 +811,18 @@ local function getOmpType()
     return omptype
 end
 
-local function assignHosts(hosts, np, ppn)
+local function assignHosts(hosts, np, ppn, tpp)
     tmp = np
     newhosts = {}
     current = 0
     if debug then
-        print_stdout(string.format("Assign %d processes with %d per node to %d hosts", np, ppn, #hosts))
-        print_stdout("Available hosts for scheduling:")
-        print_stdout("Host", "Slots", "MaxSlots", "Interface")
-        for i, h in pairs(hosts) do
-            print_stdout (h["hostname"], h["slots"], h["maxslots"],"", h["interface"])
-        end
+        print_stdout(string.format("DEBUG: Assign %d processes with %d per node and %d threads per process to %d hosts", np, ppn, tpp, #hosts))
     end
     local break_while = false
     while tmp > 0 and #hosts > 0 do
         for i, host in pairs(hosts) do
-            if host["slots"] and host["slots"] >= ppn then
-                if host["maxslots"] and host["maxslots"] < ppn then
+            if host["slots"] and host["slots"] >= ppn*tpp then
+                if host["maxslots"] and host["maxslots"] < ppn*tpp then
                     table.insert(newhosts, {hostname=host["hostname"],
                                             slots=host["maxslots"],
                                             maxslots=host["maxslots"],
@@ -774,17 +834,17 @@ local function assignHosts(hosts, np, ppn)
                     hosts[i] = nil
                 else
                     table.insert(newhosts, {hostname=host["hostname"],
-                                            slots=ppn,
+                                            slots=ppn*tpp,
                                             maxslots=host["slots"],
                                             interface=host["interface"]})
                     if debug then
-                        print_stdout(string.format("DEBUG: Add Host %s with %d slots to host list", host["hostname"], ppn))
+                        print_stdout(string.format("DEBUG: Add Host %s with %d slots to host list", host["hostname"], ppn*tpp))
                     end
-                    current = ppn
+                    current = ppn*tpp
                     hosts[i] = nil
                 end
             elseif host["slots"] then
-                --[[if host["maxslots"] then
+                --[[if host["maxslotsno"] then
                     if host["maxslots"] < ppn then
                         print_stderr(string.format("WARN: Oversubscription for host %s needed, but max-slots set to %d.",
                                                 host["hostname"], host["maxslots"]))
@@ -815,19 +875,19 @@ local function assignHosts(hosts, np, ppn)
                 os.exit(1)
             else
                 table.insert(newhosts, {hostname=host["hostname"],
-                                        slots=ppn,
+                                        slots=ppn*tpp,
                                         maxslots=host["slots"],
                                         interface=host["interface"]})
                 if debug then
-                    print_stdout(string.format("DEBUG: Add Host %s with %d slots to host list", host["hostname"], ppn))
+                    print_stdout(string.format("DEBUG: Add Host %s with %d slots to host list", host["hostname"], ppn*tpp))
                 end
-                current = ppn
+                current = ppn*tpp
             end
             tmp = tmp - current
             if tmp < 1 then
                 break_while = true
                 break
-            elseif tmp < ppn then
+            elseif tmp < ppn*tpp then
                 ppn = tmp
             end
         end
@@ -879,7 +939,7 @@ local function calculatePinExpr(cpuexprs)
         for _, c in pairs(list) do
             table.insert(strList, c)
         end
-        table.insert(newexprs, table.concat(strList,","))
+        table.insert(newexprs, strList)
     end
     return newexprs
 end
@@ -889,7 +949,10 @@ local function calculateCpuExprs(nperdomain, cpuexprs)
     local affinity = likwid.getAffinityInfo()
     local domainlist = {}
     local newexprs = {}
-    domainname, count = nperdomain:match("[E:]*(%g*):(%d+)")
+    domainname, count, threads = nperdomain:match("[E]*[:]*([NSCM]*):(%d+)[:]*(%d*)")
+    count = math.tointeger(count)
+    threads = math.tointeger(threads)
+    if threads == nil then threads = 1 end
 
     for i, domain in pairs(affinity["domains"]) do
         if domain["tag"]:match(domainname.."%d*") then
@@ -913,14 +976,18 @@ local function calculateCpuExprs(nperdomain, cpuexprs)
         end
         local tmplist = {}
         for j=1,count do
-            table.insert(newexprs, tostring(sortedlist[1]))
-            table.remove(sortedlist, 1)
+            local tmplist = {}
+            for t=1,threads do
+                table.insert(tmplist, tostring(sortedlist[1]))
+                table.remove(sortedlist, 1)
+            end
+            table.insert(newexprs, tmplist)
         end
     end
     if debug then
         local str = "DEBUG: Resolved NperDomain string "..nperdomain.." to CPUs: "
         for i, expr in pairs(newexprs) do
-            str = str .. expr .. " "
+            str = str .. "[" .. table.concat(expr,",") .. "]" .. " "
         end
         print_stdout(str)
     end
@@ -944,6 +1011,53 @@ local function createEventString(eventlist)
     return str
 end
 
+local function splitUncoreEvents(groupdata)
+    local core = {}
+    local uncore = {}
+    for i, e in pairs(groupdata["Events"]) do
+        if  not e["Counter"]:match("FIXC%d") and
+            not e["Counter"]:match("^PMC%d") and
+            not e["Counter"]:match("TMP%d") then
+            local event = e["Event"]..":"..e["Counter"]
+            table.insert(uncore, event)
+        else
+            local event = e["Event"]..":"..e["Counter"]
+            table.insert(core, event)
+        end
+    end
+    cevents = table.concat(core, ",")
+    uevents = table.concat(core, ",")
+    if #uncore > 0 then
+        uevents = uevents..","..table.concat(uncore,",")
+    end
+    return cevents, uevents
+end
+
+local function inList(value, list)
+    for _,l in pairs(list) do
+        if value == l then
+            return true
+        end
+    end
+    return false
+end
+
+local function uniqueList(list)
+    local newl = {}
+    for _,l in pairs(list) do
+        found = false
+        for _,k in pairs(newl) do
+            if l == k then
+                found = true
+            end
+        end
+        if not found then
+            table.insert(newl, l)
+        end
+    end
+    return newl
+end
+
 local function setPerfStrings(perflist, cpuexprs)
     local uncore = false
     local perfexprs = {}
@@ -964,11 +1078,9 @@ local function setPerfStrings(perflist, cpuexprs)
     end
 
     for k, perfStr in pairs(perflist) do
-        local coreevents = {}
-        local uncoreevents = {}
         local gdata = nil
         gdata = likwid.get_groupdata(perfStr)
-        if gdata == nil then
+        if gdata == nil or gdata["EventString"]:len() == 0 then
             print_stderr("Cannot get data for group "..perfStr..". Skipping...")
         else
             table.insert(grouplist, gdata)
@@ -976,47 +1088,38 @@ local function setPerfStrings(perflist, cpuexprs)
                 perfexprs[k] = {}
             end
 
-            for i, e in pairs(gdata["Events"]) do
-                if  not e["Counter"]:match("FIXC%d") and
-                    not e["Counter"]:match("^PMC%d") and
-                    not e["Counter"]:match("TMP%d") then
-                    table.insert(uncoreevents, e)
-                else
-                    table.insert(coreevents, e)
-                end
-            end
+            local coreevents = ""
+            local uncoreevents = ""
+            coreevents, uncoreevents = splitUncoreEvents(gdata)
+
             local tmpSocketFlags = {}
             for _,e in pairs(socketListFlags) do
                 table.insert(tmpSocketFlags, e)
             end
 
             for i,cpuexpr in pairs(cpuexprs) do
-                for j, cpu in pairs(likwid.stringsplit(cpuexpr,",")) do
-                    local uncore = false
-                    for sidx, socket in pairs(socketList) do
-                        local switchedFlag = false
-                        for _,c in pairs(socket) do
-                            if c == tonumber(cpu) then
-                                if tmpSocketFlags[sidx] == 1 then
-                                    local eventStr = createEventString(coreevents)
-                                    if #uncoreevents > 0 then
-                                        eventStr = eventStr .. ","..createEventString(uncoreevents)
-                                    end
-                                    table.insert(perfexprs[k], eventStr)
-                                    tmpSocketFlags[sidx] = 0
-                                    switchedFlag = true
-                                    uncore = true
-                                    break
-                                else
-                                    table.insert(perfexprs[k], createEventString(coreevents))
-                                    switchedFlag = true
-                                    uncore = true
-                                end
-                            end
+                local slist = {}
+                for j, cpu in pairs(cpuexpr) do
+                    for l, socklist in pairs(socketList) do
+                        if inList(cpu, socklist) then
+                            table.insert(slist, l)
                         end
-                        if switchedFlag then break end
                     end
-                    if uncore then break end
+                end
+                slist = uniqueList(slist)
+                local uncore = false
+                for _, s in pairs(slist) do
+                    if tmpSocketFlags[s] == 1 then
+                        tmpSocketFlags[s] = 0
+                        uncore = true
+                    end
+                end
+                if perfexprs[k][i] == nil then
+                    if uncore then
+                        perfexprs[k][i] = uncoreevents
+                    else
+                        perfexprs[k][i] = coreevents
+                    end
                 end
             end
 
@@ -1027,6 +1130,10 @@ local function setPerfStrings(perflist, cpuexprs)
             end
         end
     end
+    if #grouplist == 0 then
+        print_stderr("No group can be configured for measurments, exiting.")
+        os.exit(1)
+    end
     return perfexprs, grouplist
 end
 
@@ -1049,7 +1156,7 @@ local function checkLikwid()
     end
 end
 
-local function writeWrapperScript(scriptname, execStr, hosts, outputname)
+local function writeWrapperScript(scriptname, execStr, hosts, envsettings, outputname)
     if scriptname == nil or scriptname == "" then
         return
     end
@@ -1119,9 +1226,12 @@ local function writeWrapperScript(scriptname, execStr, hosts, outputname)
         if force and #perf > 0 then
             table.insert(cmd,"-f")
         end
+        if likwiddebug then
+            table.insert(cmd,"-V 3")
+        end
         table.insert(cmd,skipStr)
         table.insert(cmd,cpuexpr_opt)
-        table.insert(cmd,cpuexprs[i])
+        table.insert(cmd,table.concat(cpuexprs[i], ","))
         if #perf > 0 then
             for j, expr in pairs(perfexprs) do
                 table.insert(cmd,"-g")
@@ -1145,6 +1255,12 @@ local function writeWrapperScript(scriptname, execStr, hosts, outputname)
     if mpitype == "intelmpi" then
         f:write("export I_MPI_PIN=disable\n")
     end
+    for i, e in pairs(envsettings) do
+        if debug then
+            print_stdout(string.format("DEBUG: Environment variable %s", e))
+        end
+        f:write(string.format("export %s\n", e))
+    end
     f:write("LOCALSIZE="..losize_var.."\n\n")
 
     if mpitype == "openmpi" then
@@ -1171,14 +1287,14 @@ local function writeWrapperScript(scriptname, execStr, hosts, outputname)
 
     f:write("if [ \"$LOCALRANK\" -eq 0 ]; then\n")
     if debug then
-        print_stdout("NODE_EXEC: "..commands[1])
+        print_stdout(string.format("EXEC (Rank 0): %s",commands[1]))
     end
     f:write("\t"..commands[1].."\n")
 
     for i=2,#commands do
         f:write("elif [ \"$LOCALRANK\" -eq "..tostring(i-1).." ]; then\n")
         if debug then
-            print_stdout("NODE_EXEC: "..commands[i])
+            print_stdout(string.format("EXEC (Rank %d): %s", i-1,commands[i]))
         end
         f:write("\t"..commands[i].."\n")
     end
@@ -1298,6 +1414,7 @@ local function parseMarkerOutputFile(filename)
     local cpulist = {}
     local eventlist = {}
     local counterlist = {}
+    local regionlist = {}
     local idx = 1
     local results = {}
     local f = io.open(filename, "r")
@@ -1344,6 +1461,7 @@ local function parseMarkerOutputFile(filename)
                     results[current_region][gidx]["time"] = {}
                     results[current_region][gidx]["calls"] = {}
                 end
+                table.insert(regionlist, current_region)
             elseif parse_reg_info and line:match("^Region Info") then
                 linelist = likwid.stringsplit(line,",")
                 table.remove(linelist,1)
@@ -1394,7 +1512,12 @@ local function parseMarkerOutputFile(filename)
                         results[current_region][gidx][counter] = {}
                     end
                     for j, value in pairs(linelist) do
-                        results[current_region][gidx][counter][cpulist[j]] = tonumber(value)
+                        v = tonumber(value)
+                        if v then
+                            results[current_region][gidx][counter][cpulist[j]] = v
+                        else
+                            results[current_region][gidx][counter][cpulist[j]] = 0/0
+                        end
                     end
                     idx = idx + 1
                 end
@@ -1405,7 +1528,7 @@ local function parseMarkerOutputFile(filename)
         results[region]["clock"] = clock
     end
 
-    return host, tonumber(rank), results, cpulist
+    return host, tonumber(rank), results, cpulist, regionlist
 end
 
 
@@ -1538,7 +1661,8 @@ function printMpiOutput(group_list, all_results, regionname)
                     counterlist["inverseClock"] = 1.0/all_results[rank]["results"]["clock"]
                     tmpList = {all_results[rank]["hostname"]..":"..tostring(rank)..":"..tostring(cpu)}
                     for j=1,#groupdata["Metrics"] do
-                        local tmp = likwid.num2str(likwid.calculate_metric(gdata["Metrics"][j]["formula"], counterlist))
+                        local f = gdata["Metrics"][j]["formula"]
+                        local tmp = likwid.num2str(likwid.calculate_metric(f, counterlist))
                         table.insert(tmpList, tmp)
                     end
                     table.insert(secondtab,tmpList)
@@ -1596,7 +1720,20 @@ if #arg == 0 then
     os.exit(0)
 end
 
-for opt,arg in likwid.getopt(arg, {"n:","np:", "nperdomain:","pin:","hostfile:","h","help","v","g:","group:","mpi:","omp:","d","m","O","debug","marker","version","s:","skip:","f"}) do
+local cmd_options = {"h","help", -- default options for help message
+                     "v","version", -- default options for version message
+                     "d", "debug", -- activate debugging output
+                     "n:","np:", -- default options for number of MPI processes
+                     "t:","tpp:", -- default options for number of threads per process
+                     "mpi:","omp:", -- options to overwrite detection
+                     "s:","skip:", -- options to specify custom skip mask for threads
+                     "g:","group:", -- options to set group for performance measurements using likwid
+                     "m","marker", -- options to activate MarkerAPI
+                     "e:", "env:", -- options to forward environment variables
+                     "ld",         -- option to activate debugging in likwid-perfctr
+                     "nperdomain:","pin:","hostfile:","O","f"} -- other options
+
+for opt,arg in likwid.getopt(arg,  cmd_options) do
     if (type(arg) == "string") then
         local s,e = arg:find("-")
         if s == 1 then
@@ -1626,13 +1763,38 @@ for opt,arg in likwid.getopt(arg, {"n:","np:", "nperdomain:","pin:","hostfile:",
             print_stderr("Argument for -n/-np must be a number")
             os.exit(1)
         end
+    elseif opt == "t" or opt == "tpp" then
+        tpp = tonumber(arg)
+        if tpp == nil then
+            print_stderr("Argument for -t/-tpp must be a number")
+            os.exit(1)
+        end
+        if tpp == 0 then
+            print_stderr("Cannot run with 0 threads, at least 1 is required, sanitizing tpp to 1")
+            tpp = 1
+        end
     elseif opt == "nperdomain" then
-        nperdomain = arg
-        local domain, count = nperdomain:match("([NSCM]%d*):(%d+)")
-        if domain == nil then
+        local domain, count, threads = arg:match("([NSCM]):(%d+)[:]*(%d*)")
+        if domain == nil or count == nil then
             print_stderr("Invalid option to -nperdomain")
             os.exit(1)
         end
+        nperdomain = string.format("%s:%s", domain, count)
+        if threads ~= nil then
+            nperdomain = nperdomain .. ":" ..threads
+        end
+    elseif opt == "e" or opt == "env" then
+        name, val = arg:match("([%a%d_]+)[=]*([%a%d_\"\"']*)")
+        if name == nil and (val == nil or tostring(val):len() == 0) then
+            print_stderr("Invalid argument for -e/-env, must be varname=varvalue")
+        else
+            if (val == nil or tostring(val):len() == 0) then
+                val = os.getenv(name) or ''
+            end
+            if name:len() > 0 and val:len() > 0 then
+                table.insert(envsettings, string.format("%s=%s", name, val))
+            end
+        end
     elseif opt == "hostfile" then
         hostfile = arg
     elseif opt == "pin" then
@@ -1643,6 +1805,8 @@ for opt,arg in likwid.getopt(arg, {"n:","np:", "nperdomain:","pin:","hostfile:",
         mpitype = arg
     elseif opt == "omp" then
         omptype = arg
+    elseif opt == "ld" then
+        likwiddebug = true
     elseif opt == "s" or opt == "skip" then
         skipStr = "-s "..arg
     elseif opt == "?" then
@@ -1712,6 +1876,10 @@ end
 
 if mpitype == nil then
     mpitype = getMpiType()
+    if mpitype == nil then
+        print_stderr("ERROR: Cannot find MPI implementation")
+        os.exit(1)
+    end
     if debug then
         print_stdout("DEBUG: Using MPI implementation "..mpitype)
     end
@@ -1793,12 +1961,14 @@ if #cpuexprs > 0 then
     if debug then
         str = "["
         for i, expr in pairs(cpuexprs) do
-            str = str .. "["..expr.."], "
+            str = str .. "["..table.concat(expr,",").."], "
         end
         str = str:sub(1,str:len()-2) .. "]"
         print_stdout("DEBUG: Evaluated CPU expressions: ".. str)
     end
     ppn = #cpuexprs
+    tpp = #cpuexprs[1]
+
     if np == 0 then
         if debug then
             print_stdout(string.format("DEBUG: No -np given , setting according to pin expression and number of available hosts"))
@@ -1818,14 +1988,38 @@ if #cpuexprs > 0 then
         np = #cpuexprs*givenNrNodes
         ppn = #cpuexprs
     end
-    newhosts = assignHosts(hosts, np, ppn)
+    newhosts, ppn = assignHosts(hosts, np, ppn, tpp)
     if np > #cpuexprs*#newhosts and #perf > 0 then
         print_stderr("ERROR: Oversubsribing not allowed.")
         print_stderr(string.format("ERROR: You want %d processes but the pinning expression has only expressions for %d processes. There are only %d hosts in the host list.", np, #cpuexprs*#newhosts, #newhosts))
         os.exit(1)
     end
-elseif nperdomain ~= nil then
+else 
+    ppn = math.tointeger(np / givenNrNodes)
+    if nperdomain == nil then
+        nperdomain = "N:"..tostring(ppn)
+        if tpp > 0 then
+            nperdomain = nperdomain..":"..tostring(tpp)
+        end
+    end
+    domainname, count, threads = nperdomain:match("[E]*[:]*([NSCM]*):(%d+)[:]*(%d*)")
+    if math.tointeger(threads) == nil then
+        if tpp > 1 then
+            nperdomain = string.format("E:%s:%d:%d", domainname, count, tpp)
+        else
+            tpp = 1
+            nperdomain = string.format("E:%s:%d:%d", domainname, count, tpp)
+        end
+    else
+        tpp = math.tointeger(threads)
+        nperdomain = string.format("E:%s:%d:%d", domainname, count, tpp)
+    end
     cpuexprs = calculateCpuExprs(nperdomain, cpuexprs)
+    if debug then
+        for p, ex in pairs(cpuexprs) do
+            print_stdout(string.format("DEBUG: Process %d runs on CPUs %s", p, table.concat(ex, ",")))
+        end
+    end
     ppn = #cpuexprs
     if np == 0 then
         np = givenNrNodes * ppn
@@ -1846,73 +2040,10 @@ elseif nperdomain ~= nil then
         print_stderr(string.format("ERROR: You want %d processes with %d on each of the %d hosts", np, ppn, givenNrNodes))
         os.exit(1)
     end
-    newhosts, ppn = assignHosts(hosts, np, ppn)
-elseif ppn == 0 and np > 0 then
-    maxnp = 0
-    maxppn = 0
-    for i, host in pairs(hosts) do
-        maxnp = maxnp + host["slots"]
-        if host["slots"] > maxppn then
-            maxppn = host["slots"]
-        end
-    end
-    if ppn == 0 then
-        ppn = 1
-    end
-    if ppn > maxppn and np > maxppn then
-        ppn = maxppn
-    elseif np < maxppn then
-        ppn = np
-    elseif maxppn == np then
-        ppn = maxppn
-    end
-    if (ppn * givenNrNodes) < np then
-        if #perf == 0 then
-            print_stderr("ERROR: Processes cannot be equally distributed")
-            print_stderr(string.format("WARN: You want %d processes on %d hosts.", np, givenNrNodes))
-            ppn = np/givenNrNodes
-            print_stderr(string.format("WARN: Sanitizing number of processes per node to %d", ppn))
-        else
-            ppn = 0
-            os.exit(1)
-        end
-    end
-    local newexprs = calculateCpuExprs("E:N:"..tostring(ppn), cpuexprs)
-    local copynp = np
-    while copynp > 0 do
-        for i, expr in pairs(newexprs) do
-            local exprlist = likwid.stringsplit(expr, ",")
-            local seclength = math.ceil(#exprlist/ppn)
-            local offset = 0
-            for p=1, ppn do
-                local str = ""
-                for j=1, seclength do
-                    if exprlist[((p-1)*seclength) + j] then
-                        str = str .. exprlist[((p-1)*seclength) + j] ..","
-                    end
-                end
-                if str ~= "" then
-                    str = str:sub(1,#str-1)
-                    table.insert(cpuexprs, str)
-                    copynp = copynp - seclength
-                else
-                    break
-                end
-            end
-        end
-    end
-    newhosts, ppn = assignHosts(hosts, np, ppn)
-    if np < ppn*#newhosts then
-        np = 0
-        for i, host in pairs(newhosts) do
-            np = np + host["slots"]
-        end
-    end
-else
-    print_stderr("ERROR: Commandline settings are not supported.")
-    os.exit(1)
+    newhosts, ppn = assignHosts(hosts, np, ppn, tpp)
 end
 
+
 local grouplist = {}
 if #perf > 0 then
     perfexprs, grouplist = setPerfStrings(perf, cpuexprs)
@@ -1930,14 +2061,19 @@ end
 
 if skipStr == "" then
     if mpitype == "intelmpi" then
-        if omptype == "intel" and nrNodes > 1 then
-            skipStr = '-s 0x3'
-        elseif omptype == "intel" and nrNodes == 1 then
-            skipStr = '-s 0x3'
-        elseif omptype == "gnu" and nrNodes > 1 then
-            skipStr = '-s 0x1'
-        elseif omptype == "gnu" and nrNodes == 1 then
-            skipStr = '-s 0x0'
+        maj, min = getMpiVersion()
+        if maj < 2017 and min <= 1 then
+            if omptype == "intel" and nrNodes > 1 then
+                skipStr = '-s 0x3'
+            elseif omptype == "intel" and nrNodes == 1 then
+                skipStr = '-s 0x3'
+            elseif omptype == "gnu" and nrNodes > 1 then
+                skipStr = '-s 0x1'
+            elseif omptype == "gnu" and nrNodes == 1 then
+                skipStr = '-s 0x0'
+            end
+        else
+            skipStr = "-s 0x0"
         end
     elseif mpitype == "mvapich2" then
         if omptype == "intel" and nrNodes > 1 then
@@ -1972,7 +2108,7 @@ if writeHostfile == nil or getEnvironment == nil or executeCommand == nil then
 end
 
 writeHostfile(newhosts, hostfilename)
-writeWrapperScript(scriptfilename, table.concat(executable, " "), newhosts, outfilename)
+writeWrapperScript(scriptfilename, table.concat(executable, " "), newhosts, envsettings, outfilename)
 local env = getEnvironment()
 local exitvalue = executeCommand(scriptfilename, hostfilename, env, nrNodes)
 
@@ -2001,21 +2137,26 @@ if not use_marker then
 else
     local tmpList = {}
     local cpuCount = 0
+    local regionlist = {}
     for i, file in pairs(filelist) do
-        host, rank, results, cpulist = parseMarkerOutputFile(file)
+        host, rank, results, cpulist, rlist = parseMarkerOutputFile(file)
         if host ~= nil and rank ~= nil then
             if all_results[rank] == nil then
                 all_results[rank] = {}
             end
             all_results[rank]["hostname"] = host
             all_results[rank]["cpus"] = cpulist
+            for _, r in pairs(rlist) do
+                table.insert(regionlist, r)
+            end
             cpuCount = cpuCount + #cpulist
             tmpList[rank] = results
             os.remove(file)
         end
     end
+    regionlist = uniqueList(regionlist)
     if likwid.tablelength(all_results) > 0 then
-        for region, _ in pairs(tmpList[0]) do
+        for _,region in pairs(regionlist) do
             for rank,_ in pairs(all_results) do
                 all_results[rank]["results"] = tmpList[rank][region]
             end
@@ -2024,4 +2165,3 @@ else
     end
 end
 os.exit(exitvalue)
-
