@@ -7,13 +7,13 @@
  *      Description:  An application to read out performance counter registers
  *                    on x86 processors
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,7 +38,7 @@ print_stdout = print
 print_stderr = function(...) for k,v in pairs({...}) do io.stderr:write(v .. "\n") end io.stderr:flush() end
 
 local function version()
-    print_stdout(string.format("likwid-perfctr --  Version %d.%d",likwid.version,likwid.release))
+    print_stdout(string.format("likwid-perfctr -- Version %d.%d.%d (commit: %s)",likwid.version,likwid.release,likwid.minor,likwid.commit))
 end
 
 local function examples()
@@ -136,6 +136,9 @@ forceOverwrite = 0
 gotC = false
 markerFile = string.format("/tmp/likwid_%d.txt",likwid.getpid())
 cpuClock = 1
+execpid = false
+perfflags = nil
+perfpid = nil
 likwid.catchSignal()
 
 if #arg == 0 then
@@ -143,7 +146,7 @@ if #arg == 0 then
     os.exit(0)
 end
 
-for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "i", "m", "M:", "o:", "O", "P", "s:", "S:", "t:", "v", "V:", "T:", "f", "group:", "help", "info", "version", "verbose:", "output:", "skip:", "marker", "force", "stats"}) do
+for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "i", "m", "M:", "o:", "O", "P", "s:", "S:", "t:", "v", "V:", "T:", "f", "group:", "help", "info", "version", "verbose:", "output:", "skip:", "marker", "force", "stats", "execpid", "perfflags:", "perfpid:"}) do
     if (type(arg) == "string") then
         local s,e = arg:find("-");
         if s == 1 then
@@ -202,6 +205,12 @@ for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "
         print_groups = true
     elseif (opt == "e") then
         print_events = true
+    elseif (opt == "execpid") then
+        execpid = true
+    elseif (opt == "perfflags") then
+        perfflags = arg
+    elseif (opt == "perfpid") then
+        perfpid = arg
     elseif (opt == "E") then
         if arg ~= nil then
             print_event = arg
@@ -314,6 +323,10 @@ for opt,arg in likwid.getopt(arg, {"a", "c:", "C:", "e", "E:", "g:", "h", "H", "
         os.exit(1)
     end
 end
+local execList = {}
+for i=1, likwid.tablelength(arg)-2 do
+    table.insert(execList, arg[i])
+end
 
 io.stdout:setvbuf("no")
 cpuinfo = likwid.getCpuInfo()
@@ -327,12 +340,14 @@ if num_cpus == 0 and
    not print_groups and
    not print_group_help and
    not print_info then
-    print_stderr("Option -c <list> or -C <list> must be given on commandline")
-    usage()
-    if outfile and likwid.access(outfile..".tmp", "e") == 0 then
-        os.remove(outfile..".tmp")
+    cpulist = {}
+    pin_cpus = false
+    for cntr=0,cputopo["numHWThreads"]-1 do
+        if cputopo["threadPool"][cntr]["inCpuSet"] == 1 then
+            num_cpus = num_cpus + 1
+            table.insert(cpulist, cputopo["threadPool"][cntr]["apicId"])
+        end
     end
-    os.exit(1)
 elseif num_cpus == 0 and
        gotC and
        not print_events and
@@ -617,7 +632,15 @@ if pin_cpus then
     end
 end
 
-
+if use_marker == true then
+    likwid.setenv("LIKWID_FILEPATH", markerFile)
+    likwid.setenv("LIKWID_MODE", tostring(access_mode))
+    likwid.setenv("LIKWID_DEBUG", tostring(verbose))
+    local str = table.concat(event_string_list, "|")
+    likwid.setenv("LIKWID_EVENTS", str)
+    likwid.setenv("LIKWID_THREADS", table.concat(cpulist,","))
+    likwid.setenv("LIKWID_FORCE", "-1")
+end
 
 --[[for i, event_string in pairs(event_string_list) do
     local groupdata = likwid.get_groupdata(event_string)
@@ -647,11 +670,54 @@ if likwid.init(num_cpus, cpulist) < 0 then
     os.exit(1)
 end
 
+
+if verbose > 0 then
+    print_stdout(string.format("Executing: %s",table.concat(execList," ")))
+end
+local ldpath = os.getenv("LD_LIBRARY_PATH")
+local libpath = string.match(likwid.pinlibpath, "([/%a%d]+)/[%a%s%d]*")
+if ldpath == nil then
+    likwid.setenv("LD_LIBRARY_PATH", libpath)
+elseif not ldpath:match(libpath) then
+    likwid.setenv("LD_LIBRARY_PATH", libpath..":"..ldpath)
+end
+
+
+
+local pid = nil
+if #execList > 0 then
+    local execString = table.concat(execList," ")
+    if pin_cpus then
+        pid = likwid.startProgram(execString, #cpulist, cpulist)
+    else
+        pid = likwid.startProgram(execString, 0, cpulist)
+    end
+    if execpid then
+        perfpid = pid
+    end
+end
+if not pid and #execList > 0 then
+    print_stderr(string.format("Failed to execute command: %s", table.concat(execList," ")))
+    likwid.putTopology()
+    likwid.putNumaInfo()
+    likwid.putConfiguration()
+    os.exit(1)
+elseif #execList > 0 then
+    likwid.sendSignal(pid, 19)
+end
+
+
 if os.getenv("LIKWID_FORCE") == nil or (forceOverwrite == 1 and os.getenv("LIKWID_FORCE") ~= tostring(forceOverwrite)) then
     likwid.setenv("LIKWID_FORCE", tostring(forceOverwrite))
 end
 for i, event_string in pairs(event_string_list) do
     if event_string:len() > 0 then
+        if perfpid ~= nil then
+            likwid.setenv("LIKWID_PERF_PID", tostring(perfpid))
+        end
+        if perfpid ~= nil and perfflags ~= nil then
+            likwid.setenv("LIKWID_PERF_FLAGS", tostring(perfflags))
+        end
         local gid = likwid.addEventSet(event_string)
         if gid < 0 then
             likwid.putTopology()
@@ -671,32 +737,18 @@ if #group_ids == 0 then
 end
 
 activeGroup = group_ids[1]
-likwid.setupCounters(activeGroup)
+ret = likwid.setupCounters(activeGroup)
+if ret < 0 then
+    likwid.killProgram(pid)
+    os.exit(1)
+end
 if outfile == nil then
     print_stdout(likwid.hline)
 end
 
-if use_marker == true then
-    likwid.setenv("LIKWID_FILEPATH", markerFile)
-    likwid.setenv("LIKWID_MODE", tostring(access_mode))
-    likwid.setenv("LIKWID_DEBUG", tostring(verbose))
-    local str = table.concat(event_string_list, "|")
-    likwid.setenv("LIKWID_EVENTS", str)
-    likwid.setenv("LIKWID_THREADS", table.concat(cpulist,","))
-    likwid.setenv("LIKWID_FORCE", "-1")
-end
 
-execString = table.concat(arg," ",1, likwid.tablelength(arg)-2)
-if verbose == true then
-    print_stdout(string.format("Executing: %s",execString))
-end
-local ldpath = os.getenv("LD_LIBRARY_PATH")
-local libpath = string.match(likwid.pinlibpath, "([/%a%d]+)/[%a%s%d]*")
-if ldpath == nil then
-    likwid.setenv("LD_LIBRARY_PATH", libpath)
-elseif not ldpath:match(libpath) then
-    likwid.setenv("LD_LIBRARY_PATH", libpath..":"..ldpath)
-end
+
+
 
 
 if use_timeline == true then
@@ -704,7 +756,7 @@ if use_timeline == true then
     for i, cpu in pairs(cpulist) do
         cores_string = cores_string .. tostring(cpu) .. "|"
     end
-    print_stderr("# "..cores_string:sub(1,cores_string:len()-1))
+    print("# "..cores_string:sub(1,cores_string:len()-1))
     for i, gid in pairs(group_ids) do
         local strlist = {}
         if likwid.getNumberOfMetrics(gid) == 0 then
@@ -716,7 +768,7 @@ if use_timeline == true then
                 table.insert(strlist, likwid.getNameOfMetric(gid, m))
             end
         end
-        print_stderr("# "..table.concat(strlist, "|").."\n")
+        print("# "..table.concat(strlist, "|"))
     end
 end
 
@@ -725,6 +777,7 @@ end
 io.stdout:flush()
 local groupTime = {}
 local exitvalue = 0
+local twork = 0
 if use_wrapper or use_timeline then
     local start = likwid.startClock()
     local stop = 0
@@ -743,81 +796,70 @@ if use_wrapper or use_timeline then
         os.exit(1)
     end
 
-    local pid = nil
-    if execString:len() > 0 then
-        if pin_cpus then
-            pid = likwid.startProgram(execString, #cpulist, cpulist)
-        else
-            pid = likwid.startProgram(execString, 0, cpulist)
-        end
-    else
-        pid = likwid.getpid()
-    end
+    likwid.sendSignal(pid, 18)
 
-    if not pid then
-        print_stderr("Failed to execute command: ".. execString)
-        likwid.putTopology()
-        likwid.putNumaInfo()
-        likwid.putConfiguration()
-        os.exit(1)
-    else
-        start = likwid.startClock()
-        groupTime[activeGroup] = 0
-        timeline_delim = " "
-        if use_csv then
-            timeline_delim = ","
+    start = likwid.startClock()
+    groupTime[activeGroup] = 0
+    timeline_delim = " "
+    if use_csv then
+        timeline_delim = ","
+    end
+    while true do
+        if likwid.getSignalState() ~= 0 then
+            if #execList > 0 then
+                likwid.killProgram(pid)
+            end
+            break
         end
-        while true do
-            if likwid.getSignalState() ~= 0 then
-                if execString:len() > 0 then
-                    likwid.killProgram()
-                end
+        local remain = likwid.sleep(math.floor(duration-(twork*1E6)))
+        exitvalue = likwid.checkProgram(pid)
+        if remain > 0 or exitvalue >= 0 then
+            io.stdout:flush()
+            if #execList > 0 then
                 break
             end
-            local remain = likwid.sleep(duration)
-            exitvalue = likwid.checkProgram(pid)
-            if remain > 0 or exitvalue >= 0 then
-                io.stdout:flush()
-                if execString:len() > 0 then
-                    break
-                end
-            end
-            if use_timeline == true then
-                stop = likwid.stopClock()
-                likwid.readCounters()
-                local time = likwid.getClock(start, stop)
-                if likwid.getNumberOfMetrics(activeGroup) == 0 then
-                    results = likwid.getLastResults()
-                else
-                    results = likwid.getLastMetrics()
-                end
-                --str = tostring(math.tointeger(activeGroup)) .. " "..tostring(#results[activeGroup]).." "..tostring(#cpulist).." "..tostring(time)
-                local outList = {}
-                table.insert(outList, tostring(math.tointeger(activeGroup)))
-                table.insert(outList, tostring(#results[activeGroup]))
-                table.insert(outList, tostring(#cpulist))
-                table.insert(outList, tostring(time))
-                for i,l1 in pairs(results[activeGroup]) do
-                    for j, value in pairs(l1) do
-                        --str = str .. " " .. tostring(value)
-                        table.insert(outList, tostring(value))
-                    end
-                end
-                --io.stderr:write(str.."\n")
-                io.stderr:write(table.concat(outList, timeline_delim).."\n")
-                groupTime[activeGroup] = time
-            else
-                likwid.readCounters()
-            end
-            if #group_ids > 1 then
-                likwid.switchGroup(activeGroup + 1)
-                activeGroup = likwid.getIdOfActiveGroup()
-                if groupTime[activeGroup] == nil then
-                    groupTime[activeGroup] = 0
-                end
-                nr_events = likwid.getNumberOfEvents(activeGroup)
-            end
         end
+
+        if use_timeline == true then
+            stop = likwid.stopClock()
+            xstart = likwid.startClock()
+            likwid.readCounters()
+
+            local time = likwid.getClock(start, stop)
+            if likwid.getNumberOfMetrics(activeGroup) == 0 then
+                results = likwid.getLastResults()
+            else
+                results = likwid.getLastMetrics()
+            end
+            local outList = {}
+            table.insert(outList, tostring(math.tointeger(activeGroup)))
+            table.insert(outList, tostring(#results[activeGroup]))
+            table.insert(outList, tostring(#cpulist))
+            table.insert(outList, tostring(time))
+            for i,l1 in pairs(results[activeGroup]) do
+                for j, value in pairs(l1) do
+                    table.insert(outList, tostring(value))
+                end
+            end
+            print_stderr(table.concat(outList, timeline_delim))
+            groupTime[activeGroup] = time
+            xstop = likwid.stopClock()
+            twork = likwid.getClock(xstart, xstop)
+        else
+            xstart = likwid.startClock()
+            likwid.readCounters()
+            xstop = likwid.stopClock()
+            twork = likwid.getClock(xstart, xstop)
+        end
+        if #group_ids > 1 then
+            likwid.switchGroup(activeGroup + 1)
+            activeGroup = likwid.getIdOfActiveGroup()
+            if groupTime[activeGroup] == nil then
+                groupTime[activeGroup] = 0
+            end
+            nr_events = likwid.getNumberOfEvents(activeGroup)
+        end
+
         stop = likwid.stopClock()
     end
 elseif use_stethoscope then
@@ -828,11 +870,8 @@ elseif use_stethoscope then
     end
     likwid.sleep(duration)
 elseif use_marker then
-    local ret = os.execute(execString)
-    if ret == nil then
-        print_stderr("Failed to execute command: ".. execString)
-        exitvalue = 1
-    end
+    likwid.sendSignal(pid, 18)
+    exitvalue = likwid.waitpid(pid)
 end
 
 if not use_marker then
