@@ -7,13 +7,13 @@
  *      Description:  An application to get information about power
  *      consumption on architectures implementing the RAPL interface.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -37,7 +37,7 @@ print_stdout = print
 print_stderr = function(...) for k,v in pairs({...}) do io.stderr:write(v .. "\n") end end
 
 local function version()
-    print_stdout(string.format("likwid-powermeter --  Version %d.%d",likwid.version,likwid.release))
+    print_stdout(string.format("likwid-powermeter -- Version %d.%d.%d (commit: %s)",likwid.version,likwid.release,likwid.minor,likwid.commit))
 end
 
 local function examples()
@@ -84,7 +84,7 @@ time_interval = 2.E06
 time_orig = "2s"
 read_interval = 30.E06
 sockets = {}
-domainList = {"PKG", "PP0", "PP1", "DRAM"}
+domainList = {"PKG", "PP0", "PP1", "DRAM", "PLATFORM"}
 
 cpuinfo = likwid.getCpuInfo()
 cputopo = likwid.getCpuTopology()
@@ -226,7 +226,7 @@ end
 if (print_info) then
     for i, dname in pairs(domainList) do
         local domain = power["domains"][dname]
-        if domain["supportInfo"] then
+        if domain and domain["supportInfo"] then
             print_stdout(string.format("Info for RAPL domain %s:", dname));
             print_stdout(string.format("Thermal Spec Power: %g Watt",domain["tdp"]*1E-6))
             print_stdout(string.format("Minimum Power: %g Watt",domain["minPower"]*1E-6))
@@ -266,7 +266,7 @@ if use_perfctr then
     execString = string.format("<INSTALLED_PREFIX>/bin/likwid-perfctr -C %s -f -g CLOCK ",argString)
 end
 
-
+local execList = {}
 if #arg == 0 then
     if use_perfctr then
         execString = execString .. string.format(" -S %s ", time_orig)
@@ -275,10 +275,13 @@ if #arg == 0 then
         stethoscope = true
     end
 else
+    for i=1, likwid.tablelength(arg)-2 do
+        table.insert(execList, arg[i])
+    end
     if use_perfctr then
-        execString = execString .. table.concat(arg," ",1, likwid.tablelength(arg)-2)
+        execString = execString .. table.concat(execList," ")
     else
-        execString = table.concat(arg," ",1, likwid.tablelength(arg)-2)
+        execString = table.concat(execList," ")
     end
 end
 
@@ -312,9 +315,9 @@ if not print_info and not print_temp then
                 likwid.sleep(time_interval)
             end
         else
-            local pid = likwid.startProgram(execString, 0, {})
+            local pid = likwid.startProgram(table.concat(execList," "), 0, {})
             if not pid then
-                print_stderr(string.format("Failed to execute %s!",execString))
+                print_stderr(string.format("Failed to execute %s!",table.concat(execList," ")))
                 likwid.finalize()
                 os.exit(1)
             end
@@ -367,7 +370,7 @@ if not print_info and not print_temp then
     else
         err = os.execute(execString)
         if err == false then
-            print_stderr(string.format("Failed to execute %s!",execString))
+            print_stderr(string.format("Failed to execute %s!", execString))
             likwid.putPowerInfo()
             likwid.finalize()
             os.exit(1)
