@@ -7,13 +7,13 @@
  *      Description:  A application to determine the thread and cache topology
  *                    on x86 processors.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -37,7 +37,7 @@ print_stdout = print
 print_stderr = function(...) for k,v in pairs({...}) do io.stderr:write(v .. "\n") end end
 
 function version()
-    io.stdout:write(string.format("likwid-topology --  Version %d.%d\n",likwid.version,likwid.release))
+    io.stdout:write(string.format("likwid-topology -- Version %d.%d.%d (commit: %s)",likwid.version,likwid.release,likwid.minor,likwid.commit))
 end
 
 function usage()
@@ -399,4 +399,5 @@ likwid.putAffinityInfo()
 likwid.putNumaInfo()
 likwid.putTopology()
 likwid.putConfiguration()
+likwid.unsetenv("LIKWID_NO_ACCESS")
 os.exit(0)
