@@ -5,13 +5,13 @@
  *
  *      Description:  Lua LIKWID interface library
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -39,6 +39,8 @@ likwid.groupfolder = "<LIKWIDGROUPPATH>"
 
 likwid.version = <VERSION>
 likwid.release = <RELEASE>
+likwid.minor = <MINOR>
+likwid.commit = "<GITCOMMIT>"
 likwid.pinlibpath = "<LIBLIKWIDPIN>"
 likwid.dline = string.rep("=",80)
 likwid.hline =  string.rep("-",80)
@@ -107,7 +109,9 @@ likwid.readTemp = likwid_readTemp
 likwid.memSweep = likwid_memSweep
 likwid.memSweepDomain = likwid_memSweepDomain
 likwid.pinProcess = likwid_pinProcess
+likwid.pinThread = likwid_pinThread
 likwid.setenv = likwid_setenv
+likwid.unsetenv = likwid_unsetenv
 likwid.getpid = likwid_getpid
 likwid.setVerbosity = likwid_setVerbosity
 likwid.access = likwid_access
@@ -117,6 +121,7 @@ likwid.killProgram = likwid_killProgram
 likwid.catchSignal = likwid_catchSignal
 likwid.getSignalState = likwid_getSignalState
 likwid.waitpid = likwid_waitpid
+likwid.sendSignal = likwid_sendSignal
 likwid.cpustr_to_cpulist = likwid_cpustr_to_cpulist
 likwid.nodestr_to_nodelist = likwid_nodestr_to_nodelist
 likwid.sockstr_to_socklist = likwid_sockstr_to_socklist
@@ -145,14 +150,18 @@ likwid.markerRegionCount = likwid_markerRegionCount
 likwid.markerRegionResult = likwid_markerRegionResult
 likwid.markerRegionMetric = likwid_markerRegionMetric
 likwid.getCpuClockCurrent = likwid_getCpuClockCurrent
-likwid.setCpuClockCurrent = likwid_setCpuClockCurrent
 likwid.getCpuClockMin = likwid_getCpuClockMin
 likwid.setCpuClockMin = likwid_setCpuClockMin
 likwid.getCpuClockMax = likwid_getCpuClockMax
 likwid.setCpuClockMax = likwid_setCpuClockMax
 likwid.getGovernor = likwid_getGovernor
 likwid.setGovernor = likwid_setGovernor
-likwid.getDriver = likwid_getDriver
+likwid.setTurbo = likwid_setTurbo
+likwid.getTurbo = likwid_getTurbo
+likwid.setUncoreFreqMin = likwid_setUncoreFreqMin
+likwid.getUncoreFreqMin = likwid_getUncoreFreqMin
+likwid.setUncoreFreqMax = likwid_setUncoreFreqMax
+likwid.getUncoreFreqMax = likwid_getUncoreFreqMax
 likwid.getuid = likwid_getuid
 likwid.geteuid = likwid_geteuid
 likwid.setuid = likwid_setuid
@@ -166,6 +175,13 @@ likwid.cpuFeatures = { [0]="HW_PREFETCHER", [1]="CL_PREFETCHER", [2]="DCU_PREFET
                         [12]="MONITOR", [13]="SPEEDSTEP_LOCK", [14]="CPUID_MAX_VAL", [15]="XD_BIT",
                         [16]="DYN_ACCEL", [17]="TURBO_MODE", [18]="TM2" }
 
+likwid.signals = { [1] = "SIGHUP", [2] = "SIGINT", [3] = "SIGQUIT", [4] = "SIGILL",
+                   [5] = "SIGTRAP", [6] = "SIGABRT", [7] = "SIGBUS", [8] = "SIGFPE",
+                   [9] = "SIGKILL", [10] = "SIGUSR1", [11] = "SIGSEGV", [12] = "SIGUSR2",
+                   [13] = "SIGPIPE", [14] = "SIGALRM", [15] = "SIGTERM", [16] = "SIGSTKFLT",
+                   [17] = "SIGCHLD", [18] = "SIGCONT", [19] = "SIGSTOP", [20] = "SIGTSTP",
+                   [21] = "SIGTTIN", [22] = "SIGTTOU"}
+
 infinity = math.huge
 
 
@@ -295,8 +311,8 @@ local function get_spaces(str, min_space, max_space)
     local front = 0
     if tonumber(str) == nil then
         back = math.ceil((max_space-str:len()) /2)
-    else
-        back = 0
+--    else
+--        back = 0
     end
     front = max_space - back - str:len()
     if (front < back) then
@@ -312,7 +328,7 @@ local function calculate_metric(formula, counters_to_values)
         if a:len() > b:len() then return true end
         return false
     end
-    local result = "Nan"
+    local result = "nan"
     local err = false
     local clist = {}
     for counter,value in pairs(counters_to_values) do
@@ -320,7 +336,11 @@ local function calculate_metric(formula, counters_to_values)
     end
     table.sort(clist, cmp)
     for _,counter in pairs(clist) do
-        formula = string.gsub(formula, tostring(counter), tostring(counters_to_values[counter]))
+        if tostring(counters_to_values[counter]) ~= "-nan" then
+            formula = string.gsub(formula, tostring(counter), tostring(counters_to_values[counter]))
+        else
+            formula = string.gsub(formula, tostring(counter), tostring(0))
+        end
     end
     for c in formula:gmatch"." do
         if c ~= "+" and c ~= "-" and  c ~= "*" and  c ~= "/" and c ~= "(" and c ~= ")" and c ~= "." and c:lower() ~= "e" then
@@ -443,6 +463,8 @@ local function stringsplit(astr, sSeparator, nMax, bRegexp)
     assert(sSeparator ~= '')
     assert(nMax == nil or nMax >= 1)
     if astr == nil then return {} end
+    astr = tostring(astr)
+    local astrlen = astr:len() or 0
     local aRecord = {}
 
     if astr:len() > 0 then
@@ -457,7 +479,7 @@ local function stringsplit(astr, sSeparator, nMax, bRegexp)
             nStart = nLast+1
             nFirst,nLast = astr:find(sSeparator, nStart, bPlain)
             nMax = nMax-1
-            end
+        end
         aRecord[nField] = astr:sub(nStart)
     end
 
@@ -651,7 +673,9 @@ likwid.parse_time = parse_time
 
 local function num2str(value)
     local tmp = "0"
-    if value ~= 0 then
+    if value ~= value then
+        return "nan"
+    elseif type(value) == "number" and value ~= 0 then
         if tostring(value):match("%.0$") or value == math.tointeger(value) then
             tmp = tostring(math.tointeger(value))
         elseif string.format("%.4f", value):len() < 12 and
@@ -660,6 +684,8 @@ local function num2str(value)
         else
             tmp = string.format("%e", value)
         end
+    elseif type(value) == "string" then
+        tmp = value
     end
     return tmp
 end
@@ -705,16 +731,23 @@ local function tableMinMaxAvgSum(inputtable, skip_cols, skip_lines)
     for j=skip_cols+1,nr_columns do
         for i=skip_lines+1, nr_lines do
             local res = tonumber(inputtable[j][i])
-            if res ~= nil then
-                minOfLine[i-skip_lines+1] = math.min(res, minOfLine[i-skip_lines+1])
-                maxOfLine[i-skip_lines+1] = math.max(res, maxOfLine[i-skip_lines+1])
-                sumOfLine[i-skip_lines+1] = sumOfLine[i-skip_lines+1] + res
+            if inputtable[j][i] ~= "nan" then
+                if res ~= nil then
+                    minOfLine[i-skip_lines+1] = math.min(res, minOfLine[i-skip_lines+1])
+                    maxOfLine[i-skip_lines+1] = math.max(res, maxOfLine[i-skip_lines+1])
+                    sumOfLine[i-skip_lines+1] = sumOfLine[i-skip_lines+1] + res
+                else
+                    minOfLine[i-skip_lines+1] = 0
+                    maxOfLine[i-skip_lines+1] = 0
+                    sumOfLine[i-skip_lines+1] = 0
+                end
+                avgOfLine[i-skip_lines+1] = sumOfLine[i-skip_lines+1]/(nr_columns-skip_cols)
             else
-                minOfLine[i-skip_lines+1] = 0
-                maxOfLine[i-skip_lines+1] = 0
-                sumOfLine[i-skip_lines+1] = 0
+                minOfLine[i-skip_lines+1] = 0/0
+                maxOfLine[i-skip_lines+1] = 0/0
+                sumOfLine[i-skip_lines+1] = 0/0
+                avgOfLine[i-skip_lines+1] = 0/0
             end
-            avgOfLine[i-skip_lines+1] = sumOfLine[i-skip_lines+1]/(nr_columns-skip_cols)
         end
     end
     for i=2,#minOfLine do
@@ -1154,14 +1187,12 @@ local function llikwid_getAvailFreq(cpu)
     local freq_str = likwid_getAvailFreq(cpu)
     local freqs = {}
     if not freq_str then
-        return freqs, 0
+        return freqs
     end
     for item in freq_str:gmatch("[%d%.]+") do
         table.insert(freqs, item)
     end
-    local turbo = freqs[1]
-    table.remove(freqs, 1)
-    return freqs, turbo
+    return freqs
 end
 
 likwid.getAvailFreq = llikwid_getAvailFreq
