@@ -5,13 +5,13 @@
  *
  *      Description:  Configuration file module.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -43,7 +43,7 @@
 
 /* #####   EXPORTED VARIABLES   ########################################### */
 
-Configuration config = {NULL,NULL,NULL,NULL,-1,MAX_NUM_THREADS,MAX_NUM_NODES};
+Likwid_Configuration config = {NULL,NULL,NULL,NULL,-1,MAX_NUM_THREADS,MAX_NUM_NODES};
 int init_config = 0;
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
@@ -299,22 +299,29 @@ destroy_configuration(void)
     if (config.configFileName != NULL)
     {
         free(config.configFileName);
+        config.configFileName = NULL;
     }
     if (config.groupPath != NULL)
     {
         free(config.groupPath);
+        config.groupPath = NULL;
     }
     if (config.topologyCfgFileName != NULL)
     {
         free(config.topologyCfgFileName);
+        config.topologyCfgFileName = NULL;
     }
     if (config.daemonMode != ACCESSMODE_DIRECT)
     {
         if (config.daemonPath != NULL)
         {
             free(config.daemonPath);
+            config.daemonPath = NULL;
         }
     }
+    config.daemonMode = -1;
+    config.maxNumThreads = MAX_NUM_THREADS;
+    config.maxNumNodes = MAX_NUM_NODES;
     init_config = 0;
     return 0;
 }
