@@ -9,13 +9,13 @@
  *                  Allows to turn on and off the Hardware prefetcher
  *                  available.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -47,7 +47,7 @@
 
 /* #####   EXPORTED VARIABLES   ########################################### */
 
-static uint64_t cpuFeatureMask[MAX_NUM_THREADS] = {0x0ULL};
+static uint64_t *cpuFeatureMask = NULL;
 static int features_initialized = 0;
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
@@ -161,6 +161,7 @@ cpuFeatures_update(int cpu)
              (cpuid_info.model == BROADWELL_E) ||
              (cpuid_info.model == SKYLAKE1) ||
              (cpuid_info.model == SKYLAKE2) ||
+             (cpuid_info.model == SKYLAKEX) ||
              (cpuid_info.model == KABYLAKE1) ||
              (cpuid_info.model == KABYLAKE2))
     {
@@ -188,6 +189,7 @@ cpuFeatures_update(int cpu)
             (cpuid_info.model == BROADWELL_E) ||
             (cpuid_info.model == SKYLAKE1) ||
             (cpuid_info.model == SKYLAKE2) ||
+            (cpuid_info.model == SKYLAKEX) ||
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD))
@@ -204,6 +206,19 @@ cpuFeatures_update(int cpu)
         TEST_FLAG_INV(FEAT_CL_PREFETCHER,1);
         TEST_FLAG_INV(FEAT_HW_PREFETCHER,0);
     }
+
+    if (cpuid_info.model == XEON_PHI_KNL)
+    {
+        ret = HPMread(cpu, MSR_DEV, MSR_PREFETCH_ENABLE, &flags);
+        if (ret != 0)
+        {
+            fprintf(stderr,
+                    "Cannot read register 0x%X on cpu %d: err %d\n",
+                    MSR_PREFETCH_ENABLE, cpu, ret);
+        }
+        TEST_FLAG_INV(FEAT_DCU_PREFETCHER,0);
+        TEST_FLAG_INV(FEAT_HW_PREFETCHER,1);
+    }
 }
 
 static char*
@@ -229,6 +244,9 @@ cpuFeatureNames[CPUFEATURES_MAX] = {
     [FEAT_SPEEDSTEP_LOCK] = "Enhanced Intel SpeedStep Technology Select Lock",
 };
 
+static char*
+cpuFeatureNamesFixed[CPUFEATURES_MAX] = {};
+
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
 void
@@ -240,6 +258,12 @@ cpuFeatures_init()
     }
 
     topology_init();
+    if (!cpuFeatureMask)
+    {
+        cpuFeatureMask = malloc(cpuid_topology.numHWThreads*sizeof(uint64_t));
+        memset(cpuFeatureMask, 0, cpuid_topology.numHWThreads*sizeof(uint64_t));
+    }
+
     if (!HPMinitialized())
     {
         HPMinit();
@@ -301,6 +325,7 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
     uint64_t flags;
     uint32_t reg = MSR_IA32_MISC_ENABLE;
     int newOffsets = 0;
+    int knlOffsets = 0;
     if (IF_FLAG(type))
     {
         return 0;
@@ -326,6 +351,7 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == BROADWELL_E) ||
             (cpuid_info.model == SKYLAKE1) ||
             (cpuid_info.model == SKYLAKE2) ||
+            (cpuid_info.model == SKYLAKEX) ||
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD))
@@ -333,6 +359,17 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
         reg = MSR_PREFETCH_ENABLE;
         newOffsets = 1;
     }
+    if (cpuid_info.model == XEON_PHI_KNL)
+    {
+        reg = MSR_PREFETCH_ENABLE;
+        knlOffsets = 1;
+        if (type == FEAT_CL_PREFETCHER ||
+            type == FEAT_IP_PREFETCHER)
+        {
+            fprintf(stderr, "CL_PREFETCHER and IP_PREFETCHER not available on Intel Xeon Phi (KNL)");
+            return 0;
+        }
+    }
 
     ret = HPMread(cpu, MSR_DEV, reg, &flags);
     if (ret != 0)
@@ -350,6 +387,10 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
             {
                 flags &= ~(1ULL<<0);
             }
+            else if (knlOffsets)
+            {
+                flags &= ~(1ULL<<1);
+            }
             else
             {
                 flags &= ~(1ULL<<9);
@@ -376,6 +417,10 @@ cpuFeatures_enable(int cpu, CpuFeature type, int print)
             {
                 flags &= ~(1ULL<<2);
             }
+            else if (knlOffsets)
+            {
+                flags &= ~(1ULL<<0);
+            }
             else
             {
                 flags &= ~(1ULL<<37);
@@ -431,6 +476,7 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
     uint64_t flags;
     uint32_t reg = MSR_IA32_MISC_ENABLE;
     int newOffsets = 0;
+    int knlOffsets = 1;
     if (!IF_FLAG(type))
     {
         return 0;
@@ -456,6 +502,7 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
             (cpuid_info.model == BROADWELL_E) ||
             (cpuid_info.model == SKYLAKE1) ||
             (cpuid_info.model == SKYLAKE2) ||
+            (cpuid_info.model == SKYLAKEX) ||
             (cpuid_info.model == KABYLAKE1) ||
             (cpuid_info.model == KABYLAKE2) ||
             (cpuid_info.model == ATOM_SILVERMONT_GOLD))
@@ -463,6 +510,17 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
         reg = MSR_PREFETCH_ENABLE;
         newOffsets = 1;
     }
+    if (cpuid_info.model == XEON_PHI_KNL)
+    {
+        reg = MSR_PREFETCH_ENABLE;
+        knlOffsets = 1;
+        if (type == FEAT_CL_PREFETCHER ||
+            type == FEAT_IP_PREFETCHER)
+        {
+            fprintf(stderr, "CL_PREFETCHER and IP_PREFETCHER not available on Intel Xeon Phi (KNL)");
+            return 0;
+        }
+    }
     ret = HPMread(cpu, MSR_DEV, reg, &flags);
     if (ret != 0)
     {
@@ -479,6 +537,10 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
             {
                 flags |= (1ULL<<0);
             }
+            else if (knlOffsets)
+            {
+                flags |= (1ULL<<1);
+            }
             else
             {
                 flags |= (1ULL<<9);
@@ -505,6 +567,10 @@ cpuFeatures_disable(int cpu, CpuFeature type, int print)
             {
                 flags |= (1ULL<<2);
             }
+            else if (knlOffsets)
+            {
+                flags |= (1ULL<<0);
+            }
             else
             {
                 flags |= (1ULL<<37);
@@ -582,3 +648,12 @@ cpuFeatures_name(CpuFeature type)
     return NULL;
 }
 
+
+void __attribute__((destructor (104))) cpuFeatures_finalizeDestruct(void)
+{
+    if (cpuFeatureMask)
+    {
+        free(cpuFeatureMask);
+        cpuFeatureMask = NULL;
+    }
+}
