@@ -5,13 +5,13 @@
  *
  *      Description:  Parser for CPU selection strings
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -36,8 +36,25 @@
 
 #include <likwid.h>
 
+#define MAX(a, b)  (((a) > (b)) ? (a) : (b))
+#define MIN(a, b)  (((a) < (b)) ? (a) : (b))
+
+
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
+static int
+check_and_atoi(char* s)
+{
+    int i = 0;
+    int len = strlen(s);
+    for (i = 0; i < len ; i++)
+    {
+        if (s[i] < '0' || s[i] > '9')
+            return -1;
+    }
+    return atoi(s);
+}
+
 static int
 cpulist_sort(int* incpus, int* outcpus, int length)
 {
@@ -93,6 +110,21 @@ cpu_in_domain(int domainidx, int cpu)
     return 0;
 }
 
+static int
+get_domain_idx(bstring bdomain)
+{
+    affinity_init();
+    AffinityDomains_t affinity = get_affinityDomains();
+    for (int i=0;i<affinity->numberOfAffinityDomains; i++)
+    {
+        if (bstrcmp(affinity->domains[i].tag, bdomain) == BSTR_OK)
+        {
+            return i;
+        }
+    }
+    return -1;
+}
+
 static int
 cpuexpr_to_list(bstring bcpustr, bstring prefix, int* list, int length)
 {
@@ -104,6 +136,7 @@ cpuexpr_to_list(bstring bcpustr, bstring prefix, int* list, int length)
     strlist = bsplit(bcpustr, ',');
     int oldinsert = 0;
     int insert = 0;
+    int tmp = 0;
     for (int i=0;i < strlist->qty; i++)
     {
         bstring newstr = bstrcpy(prefix);
@@ -113,8 +146,12 @@ cpuexpr_to_list(bstring bcpustr, bstring prefix, int* list, int length)
         {
             if (bstrcmp(affinity->domains[j].tag, newstr) == 0)
             {
-                list[insert] = atoi(bdata(strlist->entry[i]));
-                insert++;
+                tmp = check_and_atoi(bdata(strlist->entry[i]));
+                if (tmp >= 0)
+                {
+                    list[insert] = tmp;
+                    insert++;
+                }
                 if (insert == length)
                     goto list_done;
                 break;
@@ -131,19 +168,32 @@ list_done:
     return insert;
 }
 
+
 static int
-cpustr_to_cpulist_scatter(bstring bcpustr, int* cpulist, int length)
+cpustr_to_cpulist_method(bstring bcpustr, int* cpulist, int length)
 {
     int max_procs = 0;
+    int given_procs = 0;
     topology_init();
     CpuTopology_t cpuid_topology = get_cpuTopology();
     affinity_init();
     AffinityDomains_t affinity = get_affinityDomains();
+    bstring scattercheck = bformat("scatter");
+    bstring balancedcheck = bformat("balanced");
+    bstring cbalancedcheck = bformat("cbalanced");
     char* cpustring = bstr2cstr(bcpustr, '\0');
     if (bstrchrp(bcpustr, ':', 0) != BSTR_ERR)
     {
         int insert = 0;
         int suitidx = 0;
+        struct bstrList* parts = bstrListCreate();
+        parts = bsplit(bcpustr, ':');
+        if (parts->qty == 3)
+        {
+            int tmp = check_and_atoi(bdata(parts->entry[2]));
+            if (tmp > 0)
+                given_procs = tmp;
+        }
         int* suitable = (int*)malloc(affinity->numberOfAffinityDomains*sizeof(int));
         if (!suitable)
         {
@@ -152,7 +202,7 @@ cpustr_to_cpulist_scatter(bstring bcpustr, int* cpulist, int length)
         }
         for (int i=0; i<affinity->numberOfAffinityDomains; i++)
         {
-            if (bstrchrp(affinity->domains[i].tag, cpustring[0], 0) != BSTR_ERR &&
+            if (binstr(affinity->domains[i].tag, 0, parts->entry[0]) != BSTR_ERR &&
                 affinity->domains[i].numberOfProcessors > 0)
             {
                 suitable[suitidx] = i;
@@ -174,7 +224,7 @@ cpustr_to_cpulist_scatter(bstring bcpustr, int* cpulist, int length)
             if (!sLists[i])
             {
                 free(suitable);
-                for (int j=0; i<i; j++)
+                for (int j=0; j<i; j++)
                 {
                     free(sLists[j]);
                 }
@@ -183,17 +233,125 @@ cpustr_to_cpulist_scatter(bstring bcpustr, int* cpulist, int length)
             }
             cpulist_sort(affinity->domains[suitable[i]].processorList, sLists[i], affinity->domains[suitable[i]].numberOfProcessors);
         }
-        for (int off=0;off<max_procs;off++)
+        if (binstr(bcpustr, 0, scattercheck) != BSTR_ERR)
         {
-            for(int i=0;i < suitidx; i++)
+            if (given_procs > 0)
+                length = given_procs;
+            for (int off=0;off<max_procs;off++)
             {
-                cpulist[insert] = sLists[i][off];
-                insert++;
-                if (insert == length)
-                    goto scatter_done;
+                for(int i=0;i < suitidx; i++)
+                {
+                    cpulist[insert] = sLists[i][off];
+                    insert++;
+                    if (insert == length)
+                        goto method_done;
+                }
             }
         }
-scatter_done:
+        else if (binstr(bcpustr, 0, cbalancedcheck) != BSTR_ERR)
+        {
+            if (given_procs > 0)
+                length = given_procs;
+            else
+                length = max_procs * suitidx;
+            int per_domain = length/suitidx;
+            int remain = length % suitidx;
+            int coresAllDomains = (max_procs*suitidx)/cpuid_topology->numThreadsPerCore;
+            int coresPerDomain = coresAllDomains/suitidx;
+            int threadsPerCore = cpuid_topology->numThreadsPerCore;
+            if ((per_domain+remain) > coresPerDomain)
+            {
+                for(int i=0;i < suitidx; i++)
+                {
+                    int with_ht = ((per_domain+remain)-coresPerDomain)*threadsPerCore;
+                    for (int j = 0; j < with_ht; j++)
+                    {
+                        int cpu = affinity->domains[suitable[i]].processorList[j];
+                        cpulist[insert] = cpu;
+                        insert++;
+                        for (int k=0; k< max_procs;k++)
+                        {
+                            if (sLists[i][k] == cpu)
+                            {
+                                sLists[i][k] = -1;
+                                break;
+                            }
+                        }
+                    }
+                    int wo_ht = (per_domain+remain) - with_ht;
+                    for (int j = 0; j < wo_ht; j++)
+                    {
+                        if (sLists[i][j] >= 0)
+                        {
+                            if (remain > 0)
+                            {
+                                remain--;
+                            }
+                            int cpu = sLists[i][j];
+                            cpulist[insert] = cpu;
+                            insert++;
+                            for (int k=0; k< max_procs;k++)
+                            {
+                                if (sLists[i][k] == cpu)
+                                {
+                                    sLists[i][k] = -1;
+                                    break;
+                                }
+                            }
+                        }
+                        else
+                        {
+                            wo_ht++;
+                        }
+                    }
+                }
+            }
+            else
+            {
+                for(int i=0;i < suitidx; i++)
+                {
+                    int new_domain = per_domain;
+                    if (remain > 0)
+                    {
+                        new_domain++;
+                        remain--;
+                    }
+                    for (int j = 0; j < new_domain; j++)
+                    {
+                        cpulist[insert] = sLists[i][j];
+                        insert++;
+                        if (insert == length)
+                            goto method_done;
+                    }
+                }
+            }
+        }
+        else if (binstr(bcpustr, 0, balancedcheck) != BSTR_ERR)
+        {
+            if (given_procs > 0)
+                length = given_procs;
+            else
+                length = max_procs * suitidx;
+            int per_domain = length/suitidx;
+            int remain = length % suitidx;
+            for(int i=0;i < suitidx; i++)
+            {
+                int new_domain = per_domain;
+                if (remain > 0)
+                {
+                    new_domain++;
+                    remain--;
+                }
+                for (int j = 0; j < new_domain; j++)
+                {
+                    cpulist[insert] = affinity->domains[suitable[i]].processorList[j];
+                    insert++;
+                    if (insert == length)
+                        goto method_done;
+                }
+            }
+        }
+method_done:
         bcstrfree(cpustring);
         for (int i = 0; i< suitidx; i++)
         {
@@ -203,6 +361,10 @@ scatter_done:
         free(suitable);
         return insert;
     }
+    else
+    {
+        fprintf(stderr, "Not a valid CPU expression\n");
+    }
     bcstrfree(cpustring);
     return 0;
 }
@@ -219,6 +381,7 @@ cpustr_to_cpulist_expression(bstring bcpustr, int* cpulist, int length)
     int count = 0;
     int stride = 0;
     int chunk = 0;
+    int off = 0;
     if (bstrchrp(bcpustr, 'E', 0) != 0)
     {
         fprintf(stderr, "Not a valid CPU expression\n");
@@ -226,28 +389,43 @@ cpustr_to_cpulist_expression(bstring bcpustr, int* cpulist, int length)
     }
     struct bstrList* strlist;
     strlist = bsplit(bcpustr, ':');
-    if (strlist->qty == 3)
+    if (strlist->qty == 2)
     {
         bdomain = bstrcpy(strlist->entry[1]);
-        count = atoi(bdata(strlist->entry[2]));
+        count = cpuid_topology->activeHWThreads;
+        stride = 1;
+        chunk = 1;
+    }
+    else if (strlist->qty == 3)
+    {
+        bdomain = bstrcpy(strlist->entry[1]);
+        count = check_and_atoi(bdata(strlist->entry[2]));
         stride = 1;
         chunk = 1;
     }
     else if (strlist->qty == 5)
     {
         bdomain = bstrcpy(strlist->entry[1]);
-        count = atoi(bdata(strlist->entry[2]));
-        chunk = atoi(bdata(strlist->entry[3]));
-        stride = atoi(bdata(strlist->entry[4]));
+        count = check_and_atoi(bdata(strlist->entry[2]));
+        chunk = check_and_atoi(bdata(strlist->entry[3]));
+        stride = check_and_atoi(bdata(strlist->entry[4]));
     }
-    for (int i=0; i<affinity->numberOfAffinityDomains; i++)
+    else if (strlist->qty == 6)
     {
-        if (bstrcmp(bdomain, affinity->domains[i].tag) == 0)
-        {
-            domainidx = i;
-            break;
-        }
+        bdomain = bstrcpy(strlist->entry[1]);
+        count = check_and_atoi(bdata(strlist->entry[2]));
+        chunk = check_and_atoi(bdata(strlist->entry[3]));
+        stride = check_and_atoi(bdata(strlist->entry[4]));
+        off = check_and_atoi(bdata(strlist->entry[5]));
     }
+    if (count < 0 || chunk < 0 || stride < 0 || off < 0)
+    {
+        fprintf(stderr, "CPU expression contains non-numerical characters\n");
+        bdestroy(bdomain);
+        bstrListDestroy(strlist);
+        return 0;
+    }
+    domainidx = get_domain_idx(bdomain);
     if (domainidx < 0)
     {
         fprintf(stderr, "Cannot find domain %s\n", bdata(bdomain));
@@ -261,13 +439,13 @@ cpustr_to_cpulist_expression(bstring bcpustr, int* cpulist, int length)
     {
         for (int j=0; j<chunk && offset+j<affinity->domains[domainidx].numberOfProcessors;j++)
         {
-            cpulist[insert] = affinity->domains[domainidx].processorList[offset + j];
+            cpulist[insert] = affinity->domains[domainidx].processorList[off + offset + j];
             insert++;
             if (insert == length || insert == count)
                 goto expression_done;
         }
         offset += stride;
-        if (offset >= affinity->domains[domainidx].numberOfProcessors)
+        if (off+offset >= affinity->domains[domainidx].numberOfProcessors)
         {
             offset = 0;
         }
@@ -303,21 +481,20 @@ cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
     strlist = bsplit(bcpustr, ':');
     if (strlist->qty != 3)
     {
-        fprintf(stderr, "ERROR: Invalid expression, should look like L:<domain>:<indexlist> or be in a cpuset\n");
+        fprintf(stderr, "ERROR: Invalid expression, should look like <domain>:<indexlist> or L:<domain>:<indexlist>\n");
+        bstrListDestroy(strlist);
+        return 0;
+    }
+    if (blength(strlist->entry[2]) == 0)
+    {
+        fprintf(stderr, "ERROR: Invalid expression, should look like <domain>:<indexlist> or L:<domain>:<indexlist>\n");
         bstrListDestroy(strlist);
         return 0;
     }
     bdomain = bstrcpy(strlist->entry[1]);
     blist = bstrcpy(strlist->entry[2]);
     bstrListDestroy(strlist);
-    for (int i=0; i<affinity->numberOfAffinityDomains; i++)
-    {
-        if (bstrcmp(bdomain, affinity->domains[i].tag) == 0)
-        {
-            domainidx = i;
-            break;
-        }
-    }
+    domainidx = get_domain_idx(bdomain);
     if (domainidx < 0)
     {
         fprintf(stderr, "ERROR: Cannot find domain %s\n", bdata(bdomain));
@@ -349,14 +526,24 @@ cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
         {
             struct bstrList* indexlist;
             indexlist = bsplit(strlist->entry[i], '-');
-            if (atoi(bdata(indexlist->entry[0])) <= atoi(bdata(indexlist->entry[1])))
+            int start = check_and_atoi(bdata(indexlist->entry[0]));
+            int end = check_and_atoi(bdata(indexlist->entry[1]));
+            if (start < 0 || end < 0)
             {
-                require += atoi(bdata(indexlist->entry[1])) - atoi(bdata(indexlist->entry[0])) + 1;
+                fprintf(stderr, "CPU expression %s contains non-numerical characters\n",
+                                 bdata(strlist->entry[i]));
+                bstrListDestroy(indexlist);
+                continue;
+            }
+            if (start <= end)
+            {
+                require += end - start + 1;
             }
             else
             {
-                require += atoi(bdata(indexlist->entry[0])) - atoi(bdata(indexlist->entry[1])) + 1;
+                require += start - end + 1;
             }
+            bstrListDestroy(indexlist);
         }
         else
         {
@@ -368,7 +555,7 @@ cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
         fprintf(stderr,
                 "WARN: Selected affinity domain %s has only %d hardware threads, but selection string evaluates to %d threads.\n",
                 bdata(affinity->domains[domainidx].tag), ret, require);
-        fprintf(stderr, "      This results in multiple threads on the same hardware thread.\n");
+        fprintf(stderr, "     This results in multiple threads on the same hardware thread.\n");
     }
 logical_redo:
     for (int i=0; i< strlist->qty; i++)
@@ -377,9 +564,18 @@ logical_redo:
         {
             struct bstrList* indexlist;
             indexlist = bsplit(strlist->entry[i], '-');
-            if (atoi(bdata(indexlist->entry[0])) <= atoi(bdata(indexlist->entry[1])))
+            int start = check_and_atoi(bdata(indexlist->entry[0]));
+            int end = check_and_atoi(bdata(indexlist->entry[1]));
+            if (start < 0 || end < 0)
             {
-                for (int j=atoi(bdata(indexlist->entry[0])); j<=atoi(bdata(indexlist->entry[1])) && (insert_offset+insert < require);j++)
+                fprintf(stderr, "CPU expression %s contains non-numerical characters\n",
+                                 bdata(strlist->entry[i]));
+                bstrListDestroy(indexlist);
+                continue;
+            }
+            if (start <= end)
+            {
+                for (int j=start; j<=end && (insert_offset+insert < require); j++)
                 {
                     inlist_idx = j;
                     cpulist[insert_offset + insert] = inlist[inlist_idx % ret];
@@ -396,8 +592,16 @@ logical_redo:
             }
             else
             {
-                for (int j=atoi(bdata(indexlist->entry[0]));
-                        j>=atoi(bdata(indexlist->entry[1])) && (insert_offset+insert < require); j--)
+                int start = check_and_atoi(bdata(indexlist->entry[0]));
+                int end = check_and_atoi(bdata(indexlist->entry[1]));
+                if (start < 0 || end < 0)
+                {
+                    fprintf(stderr, "CPU expression %s contains non-numerical characters\n",
+                                     bdata(strlist->entry[i]));
+                    bstrListDestroy(indexlist);
+                    continue;
+                }
+                for (int j=start; j>=end && (insert_offset+insert < require); j--)
                 {
                     inlist_idx = j;
                     cpulist[insert_offset + insert] = inlist[inlist_idx % ret];
@@ -416,7 +620,13 @@ logical_redo:
         }
         else
         {
-            cpulist[insert_offset + insert] = inlist[atoi(bdata(strlist->entry[i])) % ret];
+            int cpu = check_and_atoi(bdata(strlist->entry[i]));
+            if (cpu < 0)
+            {
+                fprintf(stderr, "CPU expression %s contains non-numerical characters\n",
+                                 bdata(strlist->entry[i]));
+            }
+            cpulist[insert_offset + insert] = inlist[cpu % ret];
             insert++;
             if (insert == ret)
             {
@@ -458,14 +668,7 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
         bdomain = bformat("N");
         blist = bstrcpy(bcpustr);
     }
-    for (int i=0; i<affinity->numberOfAffinityDomains; i++)
-    {
-        if (bstrcmp(bdomain, affinity->domains[i].tag) == 0)
-        {
-            domainidx = i;
-            break;
-        }
-    }
+    domainidx = get_domain_idx(bdomain);
     if (domainidx < 0)
     {
         fprintf(stderr, "Cannot find domain %s\n", bdata(bdomain));
@@ -473,6 +676,7 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
         bdestroy(blist);
         return 0;
     }
+    bstring domtag = affinity->domains[domainidx].tag;
 
     strlist = bsplit(blist, ',');
     int insert = 0;
@@ -482,9 +686,18 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
         {
             struct bstrList* indexlist;
             indexlist = bsplit(strlist->entry[i], '-');
-            if (atoi(bdata(indexlist->entry[0])) <= atoi(bdata(indexlist->entry[1])))
+            int start = check_and_atoi(bdata(indexlist->entry[0]));
+            int end = check_and_atoi(bdata(indexlist->entry[1]));
+            if (start < 0 || end < 0)
             {
-                for (int j=atoi(bdata(indexlist->entry[0])); j<=atoi(bdata(indexlist->entry[1]));j++)
+                fprintf(stderr, "CPU expression %s contains non-numerical characters\n",
+                                 bdata(strlist->entry[i]));
+                bstrListDestroy(indexlist);
+                continue;
+            }
+            if (start <= end)
+            {
+                for (int j = start; j <= end; j++)
                 {
                     if (cpu_in_domain(domainidx, j))
                     {
@@ -498,13 +711,29 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
                     }
                     else
                     {
-                        fprintf(stderr, "CPU %d not in domain %s\n", j, bdata(affinity->domains[domainidx].tag));
+                        int notInCpuSet = 0;
+                        for (int k=0;k<cpuid_topology->numHWThreads;k++)
+                        {
+                            if (cpuid_topology->threadPool[k].apicId == j && !cpuid_topology->threadPool[k].inCpuSet)
+                            {
+                                notInCpuSet = 1;
+                            }
+                        }
+                        fprintf(stderr, "CPU %d not in domain %s.", j, bdata(domtag));
+                        if (notInCpuSet)
+                        {
+                            fprintf(stderr, " It is not in the given cpuset\n");
+                        }
+                        else
+                        {
+                            fprintf(stderr, "\n");
+                        }
                     }
                 }
             }
             else
             {
-                for (int j=atoi(bdata(indexlist->entry[0])); j>=atoi(bdata(indexlist->entry[1]));j--)
+                for (int j = start; j >= end; j--)
                 {
                     if (cpu_in_domain(domainidx, j))
                     {
@@ -518,7 +747,23 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
                     }
                     else
                     {
-                        fprintf(stderr, "CPU %d not in domain %s\n", j, bdata(affinity->domains[domainidx].tag));
+                        int notInCpuSet = 0;
+                        for (int k=0;k<cpuid_topology->numHWThreads;k++)
+                        {
+                            if (cpuid_topology->threadPool[k].apicId == j && !cpuid_topology->threadPool[k].inCpuSet)
+                            {
+                                notInCpuSet = 1;
+                            }
+                        }
+                        fprintf(stderr, "CPU %d not in domain %s.", j, bdata(domtag));
+                        if (notInCpuSet)
+                        {
+                            fprintf(stderr, " It is not in the given cpuset\n");
+                        }
+                        else
+                        {
+                            fprintf(stderr, "\n");
+                        }
                     }
                 }
             }
@@ -526,8 +771,13 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
         }
         else
         {
-            int cpu = atoi(bdata(strlist->entry[i]));
-            if (cpu_in_domain(domainidx, cpu))
+            int cpu = check_and_atoi(bdata(strlist->entry[i]));
+            if (cpu < 0)
+            {
+                fprintf(stderr, "CPU expression %s contains non-numerical characters\n",
+                                 bdata(strlist->entry[i]));
+            }
+            else if (cpu_in_domain(domainidx, cpu))
             {
                 cpulist[insert] = cpu;
                 insert++;
@@ -538,7 +788,23 @@ cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
             }
             else
             {
-                fprintf(stderr, "CPU %d not in domain %s\n", cpu, bdata(affinity->domains[domainidx].tag));
+                int notInCpuSet = 0;
+                for (int k=0;k<cpuid_topology->numHWThreads;k++)
+                {
+                    if (cpuid_topology->threadPool[k].apicId == cpu && !cpuid_topology->threadPool[k].inCpuSet)
+                    {
+                        notInCpuSet = 1;
+                    }
+                }
+                fprintf(stderr, "CPU %d not in domain %s.", cpu, bdata(domtag));
+                if (notInCpuSet)
+                {
+                    fprintf(stderr, " It is not in the given cpuset\n");
+                }
+                else
+                {
+                    fprintf(stderr, "\n");
+                }
             }
         }
     }
@@ -560,6 +826,7 @@ cpustr_to_cpulist(const char* cpustring, int* cpulist, int length)
     bstring bcpustr = bfromcstr(cpustring);
     struct bstrList* strlist;
     bstring scattercheck = bformat("scatter");
+    bstring balancedcheck = bformat("balanced");
     topology_init();
     CpuTopology_t cpuid_topology = get_cpuTopology();
     strlist = bsplit(bcpustr, '@');
@@ -569,14 +836,17 @@ cpustr_to_cpulist(const char* cpustring, int* cpulist, int length)
     {
         bstrListDestroy(strlist);
         bdestroy(scattercheck);
+        bdestroy(balancedcheck);
         bdestroy(bcpustr);
         return -ENOMEM;
     }
+    memset(tmpList, 0, length * sizeof(int));
     for (int i=0; i< strlist->qty; i++)
     {
-        if (binstr(strlist->entry[i], 0, scattercheck) != BSTR_ERR)
+        if (binstr(strlist->entry[i], 0, scattercheck) != BSTR_ERR ||
+            binstr(strlist->entry[i], 0, balancedcheck) != BSTR_ERR)
         {
-            ret = cpustr_to_cpulist_scatter(strlist->entry[i], tmpList, length);
+            ret = cpustr_to_cpulist_method(strlist->entry[i], tmpList, length);
             insert += cpulist_concat(cpulist, insert, tmpList, ret);
         }
         else if (bstrchrp(strlist->entry[i], 'E', 0) == 0)
@@ -589,16 +859,13 @@ cpustr_to_cpulist(const char* cpustring, int* cpulist, int length)
             ret = cpustr_to_cpulist_logical(strlist->entry[i], tmpList, length);
             insert += cpulist_concat(cpulist, insert, tmpList, ret);
         }
-        else if (cpuid_topology->activeHWThreads < cpuid_topology->numHWThreads)
+
+        else if ((bstrchrp(strlist->entry[i], 'N', 0) == 0) ||
+                 (bstrchrp(strlist->entry[i], 'S', 0) == 0) ||
+                 (bstrchrp(strlist->entry[i], 'C', 0) == 0) ||
+                 (bstrchrp(strlist->entry[i], 'M', 0) == 0))
         {
-            fprintf(stdout,
-                    "INFO: You are running LIKWID in a cpuset with %d CPUs, only logical numbering allowed\n",
-                    cpuid_topology->activeHWThreads);
-            if (((bstrchrp(strlist->entry[i], 'N', 0) == 0) ||
-                (bstrchrp(strlist->entry[i], 'S', 0) == 0) ||
-                (bstrchrp(strlist->entry[i], 'C', 0) == 0) ||
-                (bstrchrp(strlist->entry[i], 'M', 0) == 0)) &&
-                (bstrchrp(strlist->entry[i], ':', 0) != BSTR_ERR))
+            if (bstrchrp(strlist->entry[i], ':', 0) != BSTR_ERR)
             {
                 bstring newstr = bformat("L:");
                 bconcat(newstr, strlist->entry[i]);
@@ -608,34 +875,50 @@ cpustr_to_cpulist(const char* cpustring, int* cpulist, int length)
             }
             else
             {
+                int dom = get_domain_idx(strlist->entry[i]);
+                if (dom >= 0)
+                {
+                    AffinityDomains_t affinity = get_affinityDomains();
+                    bstring newstr = bformat("E:");
+                    bconcat(newstr, strlist->entry[i]);
+                    length = MIN(length, affinity->domains[dom].numberOfProcessors);
+                    ret = cpustr_to_cpulist_expression(newstr, tmpList, length);
+                    insert += cpulist_concat(cpulist, insert, tmpList, ret);
+                    bdestroy(newstr);
+                }
+                else
+                {
+                    fprintf(stderr, "Cannot find domain %s\n", bdata(strlist->entry[i]));
+                    continue;
+                }
+            }
+        }
+        else
+        {
+            if (cpuid_topology->activeHWThreads < cpuid_topology->numHWThreads)
+            {
+                if (getenv("LIKWID_SILENT") == NULL)
+                {
+                    fprintf(stdout,
+                        "INFO: You are running LIKWID in a cpuset with %d CPUs. Taking given IDs as logical ID in cpuset\n",
+                        cpuid_topology->activeHWThreads);
+                }
                 bstring newstr = bformat("L:N:");
                 bconcat(newstr, strlist->entry[i]);
                 ret = cpustr_to_cpulist_logical(newstr, tmpList, length);
                 insert += cpulist_concat(cpulist, insert, tmpList, ret);
                 bdestroy(newstr);
             }
-        }
-        else if (((bstrchrp(strlist->entry[i], 'N', 0) == 0) ||
-            (bstrchrp(strlist->entry[i], 'S', 0) == 0) ||
-            (bstrchrp(strlist->entry[i], 'C', 0) == 0) ||
-            (bstrchrp(strlist->entry[i], 'M', 0) == 0)) &&
-            (bstrchrp(strlist->entry[i], ':', 0) != BSTR_ERR))
-        {
-            bstring newstr = bformat("L:");
-            bconcat(newstr, strlist->entry[i]);
-            ret = cpustr_to_cpulist_logical(newstr, tmpList, length);
-            insert += cpulist_concat(cpulist, insert, tmpList, ret);
-            bdestroy(newstr);
-        }
-
-        else
-        {
-            ret = cpustr_to_cpulist_physical(strlist->entry[i], tmpList, length);
-            insert += cpulist_concat(cpulist, insert, tmpList, ret);
+            else
+            {
+                ret = cpustr_to_cpulist_physical(strlist->entry[i], tmpList, length);
+                insert += cpulist_concat(cpulist, insert, tmpList, ret);
+            }
         }
     }
     free(tmpList);
     bdestroy(bcpustr);
+    bdestroy(balancedcheck);
     bdestroy(scattercheck);
     bstrListDestroy(strlist);
     return insert;
@@ -665,3 +948,4 @@ sockstr_to_socklist(const char* sockstr, int* sockets, int length)
     return ret;
 }
 
+
