@@ -5,14 +5,14 @@
  *
  *      Description:  Header File affinity Module
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -37,10 +37,16 @@
 int socket_lock[MAX_NUM_NODES];
 int core_lock[MAX_NUM_THREADS];
 int tile_lock[MAX_NUM_THREADS];
+int numa_lock[MAX_NUM_THREADS];
+int sharedl2_lock[MAX_NUM_THREADS];
+int sharedl3_lock[MAX_NUM_THREADS];
+
 extern AffinityDomains affinityDomains;
 
-extern int affinity_core2node_lookup[MAX_NUM_THREADS];
-extern int affinity_thread2core_lookup[MAX_NUM_THREADS];
+extern int *affinity_thread2core_lookup;
+extern int *affinity_thread2socket_lookup;
+extern int *affinity_thread2numa_lookup;
+extern int *affinity_thread2sharedl3_lookup;
 extern int affinity_processGetProcessorId();
 extern int affinity_threadGetProcessorId();
 extern const AffinityDomain* affinity_getDomain(bstring domain);
