@@ -5,13 +5,13 @@
  *
  *      Description:  Types file for libperfctr module.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,12 @@
 
 #include <bstrlib.h>
 
+typedef enum LikwidThreadStates {
+    MARKER_STATE_NEW,
+    MARKER_STATE_START,
+    MARKER_STATE_STOP
+} LikwidThreadStates;
+
 typedef struct LikwidThreadResults{
     bstring  label;
     uint32_t index;
@@ -43,6 +49,7 @@ typedef struct LikwidThreadResults{
     double StartPMcounters[NUM_PMC];
     int StartOverflows[NUM_PMC];
     double PMcounters[NUM_PMC];
+    LikwidThreadStates state;
 } LikwidThreadResults;
 
 typedef struct {
