@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of likwid API
  *
- *      Version:   <VERSION>
- *      Released:  <DATE>
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Authors:  Thomas Roehl (tr), thomas.roehl@googlemail.com
  *
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -42,6 +42,9 @@
 #define DEBUGLEV_DETAIL 2
 #define DEBUGLEV_DEVELOP 3
 
+#define LIKWID_VERSION "VERSION.RELEASE.MINORVERSION"
+#define LIKWID_COMMIT GITCOMMIT
+
 extern int perfmon_verbosity;
 
 /** \addtogroup MarkerAPI Marker API module
@@ -274,10 +277,10 @@ typedef struct {
     AccessMode daemonMode; /*!< \brief Access mode to the MSR and PCI registers */
     int maxNumThreads; /*!< \brief Maximum number of HW threads */
     int maxNumNodes; /*!< \brief Maximum number of NUMA nodes */
-} Configuration;
+} Likwid_Configuration;
 
 /** \brief Pointer for exporting the Configuration data structure */
-typedef Configuration* Configuration_t;
+typedef Likwid_Configuration* Configuration_t;
 /*! \brief Read the config file of LIKWID, if it exists
 
 Search for LIKWID config file and read the values in
@@ -1108,7 +1111,7 @@ extern void timer_finalize(void) __attribute__ ((visibility ("default") ));
 \def NUM_POWER_DOMAINS
 Amount of currently supported RAPL domains
 */
-#define NUM_POWER_DOMAINS 4
+#define NUM_POWER_DOMAINS 5
 /*! \brief List of all RAPL domain names
 */
 extern const char* power_names[NUM_POWER_DOMAINS] __attribute__ ((visibility ("default") ));
@@ -1155,7 +1158,8 @@ typedef enum {
     PKG = 0, /*!< \brief PKG domain, mostly one CPU socket/package */
     PP0 = 1, /*!< \brief PP0 domain, not clearly defined by Intel */
     PP1 = 2, /*!< \brief PP1 domain, not clearly defined by Intel */
-    DRAM = 3 /*!< \brief DRAM domain, the memory modules */
+    DRAM = 3, /*!< \brief DRAM domain, the memory modules */
+    PLATFORM = 4 /*!< \brief PLATFORM domain, the whole system (if powered through the main board) */
 } PowerType;
 
 /*! \brief Structure describing an RAPL power domain
@@ -1444,14 +1448,7 @@ Get the current clock frequency of a core
 @return Frequency or 0 in case of errors
 */
 extern uint64_t freq_getCpuClockCurrent(const int cpu_id ) __attribute__ ((visibility ("default") ));
-/*! \brief Set the current clock frequency of a core
 
-Set the current clock frequency of a core
-@param [in] cpu_id CPU ID
-@param [in] freq Frequency in kHz
-@return Frequency or 0 in case of errors
-*/
-extern uint64_t freq_setCpuClockCurrent(const int cpu_id, const uint64_t freq) __attribute__ ((visibility ("default") ));
 /*! \brief Get the maximal clock frequency of a core
 
 Get the maximal clock frequency of a core
@@ -1482,6 +1479,21 @@ Set the minimal clock frequency of a core
 @return Frequency or 0 in case of errors
 */
 extern uint64_t freq_setCpuClockMin(const int cpu_id, const uint64_t freq) __attribute__ ((visibility ("default") ));
+/*! \brief De/Activate turbo mode for core
+
+De/Activate turbo mode for core
+@param [in] cpu_id CPU ID
+@param [in] turbo (0=off, 1=on)
+@return 1 or 0 in case of errors
+*/
+extern int freq_setTurbo(const int cpu_id, int turbo) __attribute__ ((visibility ("default") ));
+/*! \brief Get state of turbo mode for core
+
+Get state of  turbo mode for core
+@param [in] cpu_id CPU ID
+@return 1=Turbo active or 0=Turbo inactive
+*/
+extern int freq_getTurbo(const int cpu_id) __attribute__ ((visibility ("default") ));
 /*! \brief Get the frequency governor of a core
 
 Get the frequency governor of a core. The returned string must be freed by the caller.
@@ -1511,13 +1523,40 @@ Get the available frequency governors of a core. The returned string must be fre
 @return String with available frequency governors or NULL in case of errors
 */
 extern char * freq_getAvailGovs(const int cpu_id ) __attribute__ ((visibility ("default") ));
-/*! \brief Get the name of the currently active cpufreq driver
 
-Get the name of the currently active cpufreq driver. The returned string must be freed by the caller.
-@param [in] cpu_id CPU ID
-@return String with active cpufreq driver or NULL in case of errors
+/*! \brief Set the minimal Uncore frequency
+
+Set the minimal Uncore frequency. Since the ranges are not documented, valid frequencies are from minimal CPU clock to maximal Turbo clock. If selecting a frequency at the borders, please check the result with the UNCORE_CLOCK event to be effective.
+@param [in] socket_id ID of socket
+@param [in] freq Frequency in MHz
+@return 0 for success, -ERROR at failure
 */
-extern char * freq_getDriver(const int cpu_id ) __attribute__ ((visibility ("default") ));
+extern int freq_setUncoreFreqMin(const int socket_id, const uint64_t freq) __attribute__ ((visibility ("default") ));
+
+/*! \brief Get the minimal Uncore frequency
+
+Get the minimal Uncore frequency.
+@param [in] socket_id ID of socket
+@return frequency in MHz or 0 at failure
+*/
+extern uint64_t freq_getUncoreFreqMin(const int socket_id) __attribute__ ((visibility ("default") ));
+
+/*! \brief Set the maximal Uncore frequency
+
+Set the maximal Uncore frequency. Since the ranges are not documented, valid frequencies are from minimal CPU clock to maximal Turbo clock. If selecting a frequency at the borders, please check the result with the UNCORE_CLOCK event to be effective.
+@param [in] socket_id ID of socket
+@param [in] freq Frequency in MHz
+@return 0 for success, -ERROR at failure
+*/
+extern int freq_setUncoreFreqMax(const int socket_id, const uint64_t freq) __attribute__ ((visibility ("default") ));
+
+/*! \brief Get the maximal Uncore frequency
+
+Get the maximal Uncore frequency.
+@param [in] socket_id ID of socket
+@return frequency in MHz or 0 at failure
+*/
+extern uint64_t freq_getUncoreFreqMax(const int socket_id) __attribute__ ((visibility ("default") ));
 /** @}*/
 
 #ifdef __cplusplus
