@@ -5,13 +5,13 @@
  *
  *      Description:  Header File hwloc based PCI lookup backend
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -31,5 +31,6 @@
 #define PCI_HWLOC_H
 
 extern int hwloc_pci_init(uint16_t testDevice, char** socket_bus, int* nrSockets);
+extern int sysfs_pci_init(uint16_t testDevice, char** socket_bus, int* nrSockets);
 
 #endif
