@@ -5,13 +5,13 @@
  *
  *      Description:  Counter Header File of perfmon module for Broadwell EP/EN/EX.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -366,3 +366,56 @@ static PciDevice broadwellEP_pci_devices[MAX_NUM_PCI_DEVICES] = {
  [PCI_R3QPI_DEVICE_LINK_1] = {R3QPI, "0b.2", "PCI_R3QPI_DEVICE_LINK_1", "RBOX1", 0x6F37},
  [PCI_R3QPI_DEVICE_LINK_2] = {R3QPI, "0b.5", "PCI_R3QPI_DEVICE_LINK_2", "RBOX1", 0x6F3E},
 };
+
+static char* broadwellEP_translate_types[NUM_UNITS] = {
+    [FIXED] = "/sys/bus/event_source/devices/cpu",
+    [PMC] = "/sys/bus/event_source/devices/cpu",
+    [POWER] = "/sys/bus/event_source/devices/power",
+    [MBOX0] = "/sys/bus/event_source/devices/uncore_imc_0",
+    [MBOX1] = "/sys/bus/event_source/devices/uncore_imc_1",
+    [MBOX2] = "/sys/bus/event_source/devices/uncore_imc_2",
+    [MBOX3] = "/sys/bus/event_source/devices/uncore_imc_3",
+    [MBOX4] = "/sys/bus/event_source/devices/uncore_imc_4",
+    [MBOX5] = "/sys/bus/event_source/devices/uncore_imc_5",
+    [MBOX6] = "/sys/bus/event_source/devices/uncore_imc_6",
+    [MBOX7] = "/sys/bus/event_source/devices/uncore_imc_7",
+    [CBOX0] = "/sys/bus/event_source/devices/uncore_cbox_0",
+    [CBOX1] = "/sys/bus/event_source/devices/uncore_cbox_1",
+    [CBOX2] = "/sys/bus/event_source/devices/uncore_cbox_2",
+    [CBOX3] = "/sys/bus/event_source/devices/uncore_cbox_3",
+    [CBOX4] = "/sys/bus/event_source/devices/uncore_cbox_4",
+    [CBOX5] = "/sys/bus/event_source/devices/uncore_cbox_5",
+    [CBOX6] = "/sys/bus/event_source/devices/uncore_cbox_6",
+    [CBOX7] = "/sys/bus/event_source/devices/uncore_cbox_7",
+    [CBOX8] = "/sys/bus/event_source/devices/uncore_cbox_8",
+    [CBOX9] = "/sys/bus/event_source/devices/uncore_cbox_9",
+    [CBOX10] = "/sys/bus/event_source/devices/uncore_cbox_10",
+    [CBOX11] = "/sys/bus/event_source/devices/uncore_cbox_11",
+    [CBOX12] = "/sys/bus/event_source/devices/uncore_cbox_12",
+    [CBOX13] = "/sys/bus/event_source/devices/uncore_cbox_13",
+    [CBOX14] = "/sys/bus/event_source/devices/uncore_cbox_14",
+    [CBOX15] = "/sys/bus/event_source/devices/uncore_cbox_15",
+    [CBOX16] = "/sys/bus/event_source/devices/uncore_cbox_16",
+    [CBOX17] = "/sys/bus/event_source/devices/uncore_cbox_17",
+    [CBOX18] = "/sys/bus/event_source/devices/uncore_cbox_18",
+    [CBOX19] = "/sys/bus/event_source/devices/uncore_cbox_19",
+    [CBOX20] = "/sys/bus/event_source/devices/uncore_cbox_20",
+    [CBOX21] = "/sys/bus/event_source/devices/uncore_cbox_21",
+    [CBOX22] = "/sys/bus/event_source/devices/uncore_cbox_22",
+    [CBOX23] = "/sys/bus/event_source/devices/uncore_cbox_23",
+    [BBOX0] = "/sys/bus/event_source/devices/uncore_ha_0",
+    [BBOX1] = "/sys/bus/event_source/devices/uncore_ha_1",
+    [WBOX] = "/sys/bus/event_source/devices/uncore_pcu",
+    [QBOX0] = "/sys/bus/event_source/devices/uncore_qpi_0",
+    [QBOX1] = "/sys/bus/event_source/devices/uncore_qpi_1",
+    [QBOX2] = "/sys/bus/event_source/devices/uncore_qpi_2",
+    [SBOX0] = "/sys/bus/event_source/devices/uncore_sbox_0",
+    [SBOX1] = "/sys/bus/event_source/devices/uncore_sbox_1",
+    [SBOX2] = "/sys/bus/event_source/devices/uncore_sbox_2",
+    [SBOX3] = "/sys/bus/event_source/devices/uncore_sbox_3",
+    [PBOX] = "/sys/bus/event_source/devices/uncore_r2pcie",
+    [RBOX0] = "/sys/bus/event_source/devices/uncore_r3qpi_0",
+    [RBOX1] = "/sys/bus/event_source/devices/uncore_r3qpi_1",
+    [UBOX] = "/sys/bus/event_source/devices/uncore_ubox",
+    [IBOX0] = "/sys/bus/event_source/devices/uncore_irp",
+};
