@@ -5,13 +5,13 @@
  *
  *      Description: Counter header file of perfmon module for Intel Atom (Silvermont)
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -446,30 +446,102 @@ static BoxMap knl_box_map[NUM_UNITS] = {
 
 static PciDevice knl_pci_devices[MAX_NUM_PCI_DEVICES] = {
  [MSR_DEV] = {NODEVTYPE, "", "", ""},
- [PCI_IMC_DEVICE_0_CH_0] = {IMC, "08.2", "PCI_IMC_DEVICE_CH_0", "MBOX0", 0x3cb0},
- [PCI_IMC_DEVICE_0_CH_1] = {IMC, "08.3", "PCI_IMC_DEVICE_CH_0", "MBOX1", 0x3cb0},
- [PCI_IMC_DEVICE_0_CH_2] = {IMC, "08.4", "PCI_IMC_DEVICE_CH_0", "MBOX2", 0x3cb0},
- [PCI_IMC_DEVICE_0_CH_3] = {IMC, "0a.0", "PCI_IMC_DEVICE_CH_0", "MBOX3", 0x3cb0},
- [PCI_IMC_DEVICE_1_CH_0] = {IMC, "09.2", "PCI_IMC_DEVICE_CH_1", "MBOX4", 0x3cb0},
- [PCI_IMC_DEVICE_1_CH_1] = {IMC, "09.3", "PCI_IMC_DEVICE_CH_1", "MBOX5", 0x3cb0},
- [PCI_IMC_DEVICE_1_CH_2] = {IMC, "09.4", "PCI_IMC_DEVICE_CH_1", "MBOX6", 0x3cb0},
- [PCI_IMC_DEVICE_1_CH_3] = {IMC, "0b.0", "PCI_IMC_DEVICE_CH_1", "MBOX7", 0x3cb0},
- [PCI_R2PCIE_DEVICE] = {R2PCIE, "0c.1", "PCI_R2PCIE_DEVICE", "R2PCIE", 0x0000},
- [PCI_IRP_DEVICE] = {IRP, "05.6", "PCI_IRP_DEVICE", "IRP", 0x0000},
- [PCI_EDC0_UCLK_DEVICE] = {EDC, "0f.0", "PCI_EDC0_UCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC0_DCLK_DEVICE] = {EDC, "18.2", "PCI_EDC0_DCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC1_UCLK_DEVICE] = {EDC, "10.0", "PCI_EDC1_UCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC1_DCLK_DEVICE] = {EDC, "19.2", "PCI_EDC1_DCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC2_UCLK_DEVICE] = {EDC, "11.0", "PCI_EDC2_UCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC2_DCLK_DEVICE] = {EDC, "1a.2", "PCI_EDC2_DCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC3_UCLK_DEVICE] = {EDC, "12.0", "PCI_EDC3_UCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC3_DCLK_DEVICE] = {EDC, "1b.2", "PCI_EDC3_DCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC4_UCLK_DEVICE] = {EDC, "13.0", "PCI_EDC4_UCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC4_DCLK_DEVICE] = {EDC, "1c.2", "PCI_EDC4_DCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC5_UCLK_DEVICE] = {EDC, "14.0", "PCI_EDC5_UCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC5_DCLK_DEVICE] = {EDC, "1d.2", "PCI_EDC5_DCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC6_UCLK_DEVICE] = {EDC, "15.0", "PCI_EDC6_UCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC6_DCLK_DEVICE] = {EDC, "1e.2", "PCI_EDC6_DCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC7_UCLK_DEVICE] = {EDC, "16.0", "PCI_EDC7_UCLK_DEVICE", "EDC", 0x0000},
- [PCI_EDC7_DCLK_DEVICE] = {EDC, "1f.2", "PCI_EDC7_DCLK_DEVICE", "EDC", 0x0000},
+ [PCI_IMC_DEVICE_0_CH_0] = {IMC, "08.2", "PCI_IMC_DEVICE_0_CH_0", "MBOX0", 0x7843},
+ [PCI_IMC_DEVICE_0_CH_1] = {IMC, "08.3", "PCI_IMC_DEVICE_0_CH_1", "MBOX1", 0x7843},
+ [PCI_IMC_DEVICE_0_CH_2] = {IMC, "08.4", "PCI_IMC_DEVICE_0_CH_2", "MBOX2", 0x7843},
+ [PCI_IMC_DEVICE_0_CH_3] = {IMC, "0a.0", "PCI_IMC_DEVICE_0_CH_3", "MBOX3", 0x7843},
+ [PCI_IMC_DEVICE_1_CH_0] = {IMC, "09.2", "PCI_IMC_DEVICE_1_CH_0", "MBOX4", 0x7843},
+ [PCI_IMC_DEVICE_1_CH_1] = {IMC, "09.3", "PCI_IMC_DEVICE_1_CH_1", "MBOX5", 0x7843},
+ [PCI_IMC_DEVICE_1_CH_2] = {IMC, "09.4", "PCI_IMC_DEVICE_1_CH_2", "MBOX6", 0x7843},
+ [PCI_IMC_DEVICE_1_CH_3] = {IMC, "0b.0", "PCI_IMC_DEVICE_1_CH_3", "MBOX7", 0x7843},
+ [PCI_R2PCIE_DEVICE] = {R2PCIE, "0c.1", "PCI_R2PCIE_DEVICE", "R2PCIE", 0x7817},
+ [PCI_IRP_DEVICE] = {IRP, "05.6", "PCI_IRP_DEVICE", "IRP", 0x7814},
+ [PCI_EDC0_UCLK_DEVICE] = {EDC, "0f.0", "PCI_EDC0_UCLK_DEVICE", "EDC", 0x7833},
+ [PCI_EDC0_DCLK_DEVICE] = {EDC, "18.2", "PCI_EDC0_DCLK_DEVICE", "EDC", 0x7835},
+ [PCI_EDC1_UCLK_DEVICE] = {EDC, "10.0", "PCI_EDC1_UCLK_DEVICE", "EDC", 0x7833},
+ [PCI_EDC1_DCLK_DEVICE] = {EDC, "19.2", "PCI_EDC1_DCLK_DEVICE", "EDC", 0x7835},
+ [PCI_EDC2_UCLK_DEVICE] = {EDC, "11.0", "PCI_EDC2_UCLK_DEVICE", "EDC", 0x7833},
+ [PCI_EDC2_DCLK_DEVICE] = {EDC, "1a.2", "PCI_EDC2_DCLK_DEVICE", "EDC", 0x7835},
+ [PCI_EDC3_UCLK_DEVICE] = {EDC, "12.0", "PCI_EDC3_UCLK_DEVICE", "EDC", 0x7833},
+ [PCI_EDC3_DCLK_DEVICE] = {EDC, "1b.2", "PCI_EDC3_DCLK_DEVICE", "EDC", 0x7835},
+ [PCI_EDC4_UCLK_DEVICE] = {EDC, "13.0", "PCI_EDC4_UCLK_DEVICE", "EDC", 0x7833},
+ [PCI_EDC4_DCLK_DEVICE] = {EDC, "1c.2", "PCI_EDC4_DCLK_DEVICE", "EDC", 0x7835},
+ [PCI_EDC5_UCLK_DEVICE] = {EDC, "14.0", "PCI_EDC5_UCLK_DEVICE", "EDC", 0x7833},
+ [PCI_EDC5_DCLK_DEVICE] = {EDC, "1d.2", "PCI_EDC5_DCLK_DEVICE", "EDC", 0x7835},
+ [PCI_EDC6_UCLK_DEVICE] = {EDC, "15.0", "PCI_EDC6_UCLK_DEVICE", "EDC", 0x7833},
+ [PCI_EDC6_DCLK_DEVICE] = {EDC, "1e.2", "PCI_EDC6_DCLK_DEVICE", "EDC", 0x7835},
+ [PCI_EDC7_UCLK_DEVICE] = {EDC, "16.0", "PCI_EDC7_UCLK_DEVICE", "EDC", 0x7833},
+ [PCI_EDC7_DCLK_DEVICE] = {EDC, "1f.2", "PCI_EDC7_DCLK_DEVICE", "EDC", 0x7835},
+};
+
+static char* knl_translate_types[NUM_UNITS] = {
+    [FIXED] = "/sys/bus/event_source/devices/cpu",
+    [PMC] = "/sys/bus/event_source/devices/cpu",
+    [POWER] = "/sys/bus/event_source/devices/power",
+    [MBOX0] = "/sys/bus/event_source/devices/uncore_imc_0",
+    [MBOX1] = "/sys/bus/event_source/devices/uncore_imc_1",
+    [MBOX2] = "/sys/bus/event_source/devices/uncore_imc_2",
+    [MBOX4] = "/sys/bus/event_source/devices/uncore_imc_3",
+    [MBOX5] = "/sys/bus/event_source/devices/uncore_imc_4",
+    [MBOX6] = "/sys/bus/event_source/devices/uncore_imc_5",
+    [CBOX0] = "/sys/bus/event_source/devices/uncore_cbox_0",
+    [CBOX1] = "/sys/bus/event_source/devices/uncore_cbox_1",
+    [CBOX2] = "/sys/bus/event_source/devices/uncore_cbox_2",
+    [CBOX3] = "/sys/bus/event_source/devices/uncore_cbox_3",
+    [CBOX4] = "/sys/bus/event_source/devices/uncore_cbox_4",
+    [CBOX5] = "/sys/bus/event_source/devices/uncore_cbox_5",
+    [CBOX6] = "/sys/bus/event_source/devices/uncore_cbox_6",
+    [CBOX7] = "/sys/bus/event_source/devices/uncore_cbox_7",
+    [CBOX8] = "/sys/bus/event_source/devices/uncore_cbox_8",
+    [CBOX9] = "/sys/bus/event_source/devices/uncore_cbox_9",
+    [CBOX10] = "/sys/bus/event_source/devices/uncore_cbox_10",
+    [CBOX11] = "/sys/bus/event_source/devices/uncore_cbox_11",
+    [CBOX12] = "/sys/bus/event_source/devices/uncore_cbox_12",
+    [CBOX13] = "/sys/bus/event_source/devices/uncore_cbox_13",
+    [CBOX14] = "/sys/bus/event_source/devices/uncore_cbox_14",
+    [CBOX15] = "/sys/bus/event_source/devices/uncore_cbox_15",
+    [CBOX16] = "/sys/bus/event_source/devices/uncore_cbox_16",
+    [CBOX17] = "/sys/bus/event_source/devices/uncore_cbox_17",
+    [CBOX18] = "/sys/bus/event_source/devices/uncore_cbox_18",
+    [CBOX19] = "/sys/bus/event_source/devices/uncore_cbox_19",
+    [CBOX20] = "/sys/bus/event_source/devices/uncore_cbox_20",
+    [CBOX21] = "/sys/bus/event_source/devices/uncore_cbox_21",
+    [CBOX22] = "/sys/bus/event_source/devices/uncore_cbox_22",
+    [CBOX23] = "/sys/bus/event_source/devices/uncore_cbox_23",
+    [CBOX24] = "/sys/bus/event_source/devices/uncore_cbox_24",
+    [CBOX25] = "/sys/bus/event_source/devices/uncore_cbox_25",
+    [CBOX26] = "/sys/bus/event_source/devices/uncore_cbox_26",
+    [CBOX27] = "/sys/bus/event_source/devices/uncore_cbox_27",
+    [CBOX28] = "/sys/bus/event_source/devices/uncore_cbox_28",
+    [CBOX29] = "/sys/bus/event_source/devices/uncore_cbox_29",
+    [CBOX30] = "/sys/bus/event_source/devices/uncore_cbox_30",
+    [CBOX31] = "/sys/bus/event_source/devices/uncore_cbox_31",
+    [CBOX32] = "/sys/bus/event_source/devices/uncore_cbox_32",
+    [CBOX33] = "/sys/bus/event_source/devices/uncore_cbox_33",
+    [CBOX34] = "/sys/bus/event_source/devices/uncore_cbox_34",
+    [CBOX35] = "/sys/bus/event_source/devices/uncore_cbox_35",
+    [CBOX36] = "/sys/bus/event_source/devices/uncore_cbox_36",
+    [CBOX37] = "/sys/bus/event_source/devices/uncore_cbox_37",
+    [WBOX] = "/sys/bus/event_source/devices/uncore_pcu",
+    [PBOX] = "/sys/bus/event_source/devices/uncore_m2pcie",
+    [UBOX] = "/sys/bus/event_source/devices/uncore_ubox",
+    [IBOX0] = "/sys/bus/event_source/devices/uncore_irp",
+    [EUBOX0] = "/sys/bus/event_source/devices/uncore_edc_uclk_0",
+    [EUBOX1] = "/sys/bus/event_source/devices/uncore_edc_uclk_1",
+    [EUBOX2] = "/sys/bus/event_source/devices/uncore_edc_uclk_2",
+    [EUBOX3] = "/sys/bus/event_source/devices/uncore_edc_uclk_3",
+    [EUBOX4] = "/sys/bus/event_source/devices/uncore_edc_uclk_4",
+    [EUBOX5] = "/sys/bus/event_source/devices/uncore_edc_uclk_5",
+    [EUBOX6] = "/sys/bus/event_source/devices/uncore_edc_uclk_6",
+    [EUBOX7] = "/sys/bus/event_source/devices/uncore_edc_uclk_7",
+    [EDBOX0] = "/sys/bus/event_source/devices/uncore_edc_eclk_0",
+    [EDBOX1] = "/sys/bus/event_source/devices/uncore_edc_eclk_1",
+    [EDBOX2] = "/sys/bus/event_source/devices/uncore_edc_eclk_2",
+    [EDBOX3] = "/sys/bus/event_source/devices/uncore_edc_eclk_3",
+    [EDBOX4] = "/sys/bus/event_source/devices/uncore_edc_eclk_4",
+    [EDBOX5] = "/sys/bus/event_source/devices/uncore_edc_eclk_5",
+    [EDBOX6] = "/sys/bus/event_source/devices/uncore_edc_eclk_6",
+    [EDBOX7] = "/sys/bus/event_source/devices/uncore_edc_eclk_7",
+    [MBOX3FIX] = "/sys/bus/event_source/devices/uncore_imc_uclk_0",
+    [MBOX7FIX] = "/sys/bus/event_source/devices/uncore_imc_uclk_1",
 };
