@@ -6,14 +6,14 @@
  *      Description:  Header File Power Module
  *                    Implements Intel RAPL Interface.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -37,33 +37,46 @@
 #include <bitUtil.h>
 #include <error.h>
 #include <access.h>
+#include <unistd.h>
 
-const char* power_names[NUM_POWER_DOMAINS] = {"PKG", "PP0", "PP1", "DRAM"};
+const char* power_names[NUM_POWER_DOMAINS] = {"PKG", "PP0", "PP1", "DRAM", "PLATFORM"};
+const char* perf_power_names[NUM_POWER_DOMAINS] = {
+    "/sys/devices/power/events/energy-pkg.scale",
+    "/sys/devices/power/events/energy-cores.scale",
+    "",
+    "/sys/devices/power/events/energy-ram.scale",
+    "/sys/devices/power/events/energy-platform.scale",
+};
 
 uint32_t power_regs[NUM_POWER_DOMAINS] = {MSR_PKG_ENERGY_STATUS,
                                 MSR_PP0_ENERGY_STATUS,
                                 MSR_PP1_ENERGY_STATUS,
-                                MSR_DRAM_ENERGY_STATUS};
+                                MSR_DRAM_ENERGY_STATUS,
+                                MSR_PLATFORM_ENERGY_STATUS};
 
 uint32_t limit_regs[NUM_POWER_DOMAINS] = {MSR_PKG_RAPL_POWER_LIMIT,
                                 MSR_PP0_RAPL_POWER_LIMIT,
                                 MSR_PP1_RAPL_POWER_LIMIT,
-                                MSR_DRAM_RAPL_POWER_LIMIT};
+                                MSR_DRAM_RAPL_POWER_LIMIT,
+                                MSR_PLATFORM_POWER_LIMIT};
 
 uint32_t policy_regs[NUM_POWER_DOMAINS] = {0,
                                 MSR_PP0_ENERGY_POLICY,
                                 MSR_PP1_ENERGY_POLICY,
+                                0,
                                 0};
 
 uint32_t perf_regs[NUM_POWER_DOMAINS] = {MSR_PKG_PERF_STATUS,
                                 MSR_PP0_PERF_STATUS,
                                 0,
-                                MSR_DRAM_PERF_STATUS};
+                                MSR_DRAM_PERF_STATUS,
+                                0};
 
 uint32_t info_regs[NUM_POWER_DOMAINS] = {MSR_PKG_POWER_INFO,
                                 0,
                                 0,
-                                MSR_DRAM_POWER_INFO};
+                                MSR_DRAM_POWER_INFO,
+                                MSR_PLATFORM_INFO};
 
 
 double
@@ -202,6 +215,23 @@ power_tread(int socket_fd, int cpuId, uint64_t reg, uint32_t *data)
 double
 power_getEnergyUnit(int domain)
 {
+#ifdef LIKWID_USE_PERFEVENT
+    FILE* fd = NULL;
+    char out[512];
+    if (power_info.domains[domain].energyUnit == 0)
+    {
+        if (!access(perf_power_names[domain], R_OK))
+        {
+            fd = fopen(perf_power_names[domain], "r");
+            if (fd != NULL)
+            {
+                fread(out, sizeof(char), 512, fd);
+                fclose(fd);
+                power_info.domains[domain].energyUnit = atof(out);
+            }
+        }
+    }
+#endif
     return power_info.domains[domain].energyUnit;
 }
 
