@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of registers.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -85,6 +85,14 @@ typedef enum {
     PMC283, PMC284, PMC285, PMC286, PMC287, PMC288,
     PMC289, PMC290, PMC291, PMC292, PMC293, PMC294,
     PMC295, PMC296, PMC297, PMC298, PMC299, PMC300,
+    PMC301, PMC302, PMC303, PMC304, PMC305, PMC306,
+    PMC307, PMC308, PMC309, PMC310, PMC311, PMC312,
+    PMC313, PMC314, PMC315, PMC316, PMC317, PMC318,
+    PMC319, PMC320, PMC321, PMC322, PMC323, PMC324,
+    PMC325, PMC326, PMC327, PMC328, PMC329, PMC330,
+    PMC331, PMC332, PMC333, PMC334, PMC335, PMC336,
+    PMC337, PMC338, PMC339, PMC340, PMC341, PMC342,
+    PMC343, PMC344, PMC345, PMC346, PMC347, PMC348,
     NUM_PMC
 } RegisterIndex;
 
@@ -115,10 +123,11 @@ typedef enum {
     CBOX30, CBOX31, CBOX32,
     CBOX33, CBOX34, CBOX35,
     CBOX36, CBOX37,
-    PBOX,
+    PBOX, PBOX1, PBOX2, PBOX3,
     UBOX,
     UBOXFIX,
-    IBOX0, IBOX1,
+    IBOX0, IBOX1, IBOX2, IBOX3, IBOX4, IBOX5,
+    IBOX0FIX, IBOX1FIX, IBOX2FIX, IBOX3FIX, IBOX4FIX, IBOX5FIX,
     QBOX0, QBOX1, QBOX2,
     QBOX0FIX, QBOX1FIX, QBOX2FIX,
     EUBOX0, EUBOX0FIX, EUBOX1, EUBOX1FIX,
@@ -209,10 +218,17 @@ static char* RegisterTypeNames[MAX_UNITS] = {
     [CBOX36] = "Caching Agent box 36",
     [CBOX37] = "Caching Agent box 37",
     [PBOX] = "Physical Layer box",
+    [PBOX1] = "Physical Layer box",
+    [PBOX2] = "Physical Layer box",
+    [PBOX3] = "Physical Layer box",
     [UBOX] = "System Configuration box",
     [UBOXFIX] = "System Configuration box fixed counter",
     [IBOX0] = "Coherency Maintainer for IIO traffic",
     [IBOX1] = "Coherency Maintainer for IIO traffic",
+    [IBOX2] = "Coherency Maintainer for IIO traffic",
+    [IBOX3] = "Coherency Maintainer for IIO traffic",
+    [IBOX4] = "Coherency Maintainer for IIO traffic",
+    [IBOX5] = "Coherency Maintainer for IIO traffic",
     [QBOX0] = "QPI Link Layer 0",
     [QBOX1] = "QPI Link Layer 1",
     [QBOX0FIX] = "QPI Link Layer rate status 0",
