@@ -5,14 +5,14 @@
  *
  *      Description:  Header File of topology module.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -45,8 +45,6 @@
 #define MAX_FEATURE_STRING_LENGTH 512
 #define MAX_MODEL_STRING_LENGTH 512
 
-extern int affinity_thread2tile_lookup[MAX_NUM_THREADS];
-extern int affinity_thread2tile_lookup[MAX_NUM_THREADS];
 struct topology_functions {
     void (*init_cpuInfo) (cpu_set_t cpuSet);
     void (*init_cpuFeatures) (void);
@@ -73,6 +71,8 @@ struct topology_functions {
 #define ATOM_SILVERMONT_F    0x5DU
 #define ATOM_SILVERMONT_AIR  0x4CU
 #define ATOM_SILVERMONT_GOLD 0x5CU
+#define ATOM_DENVERTON       0x5FU
+#define ATOM_GOLDMONT_PLUS   0x7AU
 #define NEHALEM              0x1AU
 #define NEHALEM_BLOOMFIELD   0x1AU
 #define NEHALEM_LYNNFIELD    0x1EU
@@ -91,23 +91,27 @@ struct topology_functions {
 #define WESTMERE_EX          0x2FU
 #define XEON_MP              0x1DU
 #define BROADWELL            0x3DU
-#define BROADWELL_E3         0x47U
 #define BROADWELL_E          0x4FU
 #define BROADWELL_D          0x56U
+#define BROADWELL_E3         0x47U
 #define SKYLAKE1             0x4EU
 #define SKYLAKE2             0x5EU
+#define SKYLAKEX             0x55U
 #define KABYLAKE1            0x8EU
 #define KABYLAKE2            0x9EU
+#define CANNONLAKE           0x66U
 
 /* Intel MIC */
 #define XEON_PHI           0x01U
 #define XEON_PHI_KNL       0x57U
+#define XEON_PHI_KML       0x85U
 
 /* AMD K10 */
 #define BARCELONA      0x02U
 #define SHANGHAI       0x04U
 #define ISTANBUL       0x08U
 #define MAGNYCOURS     0x09U
+#define THUBAN         0x0AU
 
 /* AMD K8 */
 #define OPTERON_SC_1MB  0x05U
@@ -120,11 +124,13 @@ struct topology_functions {
 #define ATHLON64_X2_G   0x6BU
 #define ATHLON64_G1     0x6FU
 #define ATHLON64_G2     0x7FU
+#define ZEN_RYZEN       0x01
 
 
 #define  P6_FAMILY        0x6U
 #define  MIC_FAMILY       0xBU
 #define  NETBURST_FAMILY  0xFFU
+#define  ZEN_FAMILY       0x17
 #define  K15_FAMILY       0x15U
 #define  K16_FAMILY       0x16U
 #define  K10_FAMILY       0x10U
