@@ -6,14 +6,14 @@
  *      Description:  Types file for topology module. External definitions are
  *                    in likwid.h
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -67,6 +67,7 @@ typedef enum {
     HLE, /*!< \brief Hardware Lock Elision */
     HTT, /*!< \brief Hyper-Threading Technology */
     RDSEED, /*!< \brief Non-deterministic random bit generator */
+    AVX512, /*!< \brief 512-bit wide vector registers for Advanced Vector Extensions */
 } FeatureBit;
 /** @}*/
 #endif /*CPUID_TYPES_H*/
