@@ -5,14 +5,14 @@
  *
  *      Description:  Marker API interface of module perfmon
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -40,6 +40,7 @@
 #include <sched.h>
 #include <pthread.h>
 #include <inttypes.h>
+#include <math.h>
 
 #include <likwid.h>
 #include <bitUtil.h>
@@ -50,13 +51,11 @@
 #include <registers.h>
 #include <error.h>
 #include <access.h>
-
+#include <affinity.h>
 #include <perfmon.h>
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
-int socket_lock[MAX_NUM_NODES];
-int core_lock[MAX_NUM_THREADS];
 static int likwid_init = 0;
 static int numberOfGroups = 0;
 static int* groups;
@@ -181,10 +180,9 @@ likwid_markerInit(void)
     affinity_init();
     hashTable_init();
 
-    for(int i=0; i<MAX_NUM_NODES; i++) socket_lock[i] = LOCK_INIT;
-
+//#ifndef LIKWID_USE_PERFEVENT
     HPMmode(atoi(modeStr));
-
+//#endif
     if (getenv("LIKWID_DEBUG") != NULL)
     {
         perfmon_verbosity = atoi(getenv("LIKWID_DEBUG"));
@@ -257,13 +255,14 @@ likwid_markerInit(void)
     {
         likwid_init = 1;
     }
-    groupSet->activeGroup = 0;
     threads2Pthread[registered_cpus] = pthread_self();
     registered_cpus++;
 
+    groupSet->activeGroup = 0;
+//#ifdef LIKWID_USE_PERFEVENT
     perfmon_setupCounters(groupSet->activeGroup);
     perfmon_startCounters();
-
+//#endif
 }
 
 void
@@ -290,7 +289,6 @@ likwid_markerThreadInit(void)
         threads2Pthread[registered_cpus] = t;
         myID = registered_cpus++;
     }
-    //myID = registered_cpus++;
     pthread_mutex_unlock(&globalLock);
 
     if (getenv("LIKWID_PIN") != NULL)
@@ -301,7 +299,7 @@ likwid_markerThreadInit(void)
         if ((CPU_COUNT(&cpuset) > 1) || (likwid_getProcessorId() != threads2Cpu[myID % num_cpus]))
         {
             likwid_pinThread(threads2Cpu[myID % num_cpus]);
-            DEBUG_PRINT(DEBUGLEV_DEVELOP, "Pin thread %lu to CPU %d currently %d\n", gettid(), threads2Cpu[myID % num_cpus], sched_getcpu());
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, Pin thread %lu to CPU %d currently %d, gettid(), threads2Cpu[myID % num_cpus], sched_getcpu());
         }
     }
 }
@@ -343,13 +341,14 @@ likwid_markerClose(void)
     char* markerfile = NULL;
     int lineidx = 0;
     char line[1024];
+    int* validRegions = NULL;
 
     if ( ! likwid_init )
     {
         return;
     }
     hashTable_finalize(&numberOfThreads, &numberOfRegions, &results);
-    if ((numberOfThreads == 0)||(numberOfThreads == 0))
+    if ((numberOfThreads == 0)||(numberOfRegions == 0))
     {
         fprintf(stderr, "No threads or regions defined in hash table\n");
         return;
@@ -361,33 +360,68 @@ likwid_markerClose(void)
                 "Is the application executed with LIKWID wrapper? No file path for the Marker API output defined.\n");
         return;
     }
+    validRegions = (int*)malloc(numberOfRegions*sizeof(int));
+    if (!validRegions)
+    {
+        return;
+    }
+    for (int i=0; i<numberOfRegions; i++)
+    {
+        validRegions[i] = 0;
+    }
     file = fopen(markerfile,"w");
 
     if (file != NULL)
     {
-        DEBUG_PRINT(DEBUGLEV_DEVELOP,
-                Creating Marker file %s with %d regions %d groups and %d threads,
-                markerfile, numberOfRegions, numberOfGroups, numberOfThreads);
-        fprintf(file,"%d %d %d\n",numberOfThreads, numberOfRegions, numberOfGroups);
-        DEBUG_PRINT(DEBUGLEV_DEVELOP, %d %d %d, numberOfThreads, numberOfRegions, numberOfGroups);
-
-        for (int i=0; i<numberOfRegions; i++)
-        {
-            fprintf(file,"%d:%s\n",i,bdata(results[i].tag));
-            DEBUG_PRINT(DEBUGLEV_DEVELOP, %d:%s, i,bdata(results[i].tag));
-        }
+        int newNumberOfRegions = 0;
+        int newRegionID = 0;
         for (int i=0; i<numberOfRegions; i++)
         {
             for (int j=0; j<numberOfThreads; j++)
             {
-                fprintf(file,"%d ",i);
+                validRegions[i] += results[i].count[j];
+            }
+            if (validRegions[i] > 0)
+                newNumberOfRegions++;
+            else
+                fprintf(stderr, "WARN: Skipping region %s for evaluation.\n", bdata(results[i].tag));
+        }
+        if (newNumberOfRegions < numberOfRegions)
+        {
+            fprintf(stderr, "WARN: Regions are skipped because:\n");
+            fprintf(stderr, "      - The region was only registered\n");
+            fprintf(stderr, "      - The region was started but never stopped\n");
+            fprintf(stderr, "      - The region was never started but stopped\n");
+        }
+        DEBUG_PRINT(DEBUGLEV_DEVELOP,
+                Creating Marker file %s with %d regions %d groups and %d threads,
+                markerfile, newNumberOfRegions, numberOfGroups, numberOfThreads);
+        fprintf(file,"%d %d %d\n",numberOfThreads, newNumberOfRegions, numberOfGroups);
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, %d %d %d, numberOfThreads, newNumberOfRegions, numberOfGroups);
+
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            if (validRegions[i] == 0)
+                continue;
+            fprintf(file,"%d:%s\n",newRegionID,bdata(results[i].tag));
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, %d:%s, newRegionID,bdata(results[i].tag));
+            newRegionID++;
+        }
+        newRegionID = 0;
+        for (int i=0; i<numberOfRegions; i++)
+        {
+            if (validRegions[i] == 0)
+                continue;
+            for (int j=0; j<numberOfThreads; j++)
+            {
+                fprintf(file,"%d ",newRegionID);
                 fprintf(file,"%d ",results[i].groupID);
                 fprintf(file,"%d ",results[i].cpulist[j]);
                 fprintf(file,"%u ",results[i].count[j]);
                 fprintf(file,"%e ",results[i].time[j]);
                 fprintf(file,"%d ",groupSet->groups[results[i].groupID].numberOfEvents);
                 lineidx = sprintf(&(line[0]), "%d %d %d %u %e %d ",
-                        i, results[i].groupID,results[i].cpulist[j],results[i].count[j],
+                        newRegionID, results[i].groupID,results[i].cpulist[j],results[i].count[j],
                         results[i].time[j],groupSet->groups[results[i].groupID].numberOfEvents);
 
                 for (int k=0; k<groupSet->groups[results[i].groupID].numberOfEvents; k++)
@@ -398,6 +432,7 @@ likwid_markerClose(void)
                 fprintf(file,"\n");
                 DEBUG_PRINT(DEBUGLEV_DEVELOP, %s,line);
             }
+            newRegionID++;
         }
         fclose(file);
     }
@@ -406,7 +441,7 @@ likwid_markerClose(void)
         fprintf(stderr, "Cannot open file %s\n", markerfile);
         fprintf(stderr, "%s", strerror(errno));
     }
-
+    free(validRegions);
 }
 
 void __attribute__((destructor (101))) likwid_markerCloseDestruct(void)
@@ -457,7 +492,11 @@ likwid_markerRegisterRegion(const char* regionTag)
     bcatcstr(tag, groupSuffix);
     int cpu_id = hashTable_get(tag, &results);
     bdestroy(tag);
+//#ifdef LIKWID_USE_PERFEVENT
     return HPMaddThread(cpu_id);
+//#else
+//    return 0;
+//#endif
 }
 
 int
@@ -481,19 +520,32 @@ likwid_markerStartRegion(const char* regionTag)
 
     int cpu_id = hashTable_get(tag, &results);
     int thread_id = getThreadID(cpu_id);
+    if (results->state == MARKER_STATE_START)
+    {
+        fprintf(stderr, "WARN: Region %s was already started\n", regionTag);
+    }
     perfmon_readCountersCpu(cpu_id);
     results->cpuID = cpu_id;
     for(int i=0;i<groupSet->groups[groupSet->activeGroup].numberOfEvents;i++)
     {
-        DEBUG_PRINT(DEBUGLEV_DEVELOP, START [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu,
-                regionTag, thread_id, cpu_id, i,
-                LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
-        //groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].startData =
-        //        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData;
+        if (groupSet->groups[groupSet->activeGroup].events[i].type != NOTYPE)
+        {
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, START [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu,
+                    regionTag, thread_id, cpu_id, i,
+                    LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
+            //groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].startData =
+            //        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData;
 
-        results->StartPMcounters[i] = groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData;
-        results->StartOverflows[i] = groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows;
+            results->StartPMcounters[i] = groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData;
+            results->StartOverflows[i] = groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows;
+        }
+        else
+        {
+            results->StartPMcounters[i] = NAN;
+            results->StartOverflows[i] = -1;
+        }
     }
+    results->state = MARKER_STATE_START;
 
     bdestroy(tag);
     timer_start(&(results->startTime));
@@ -530,6 +582,11 @@ likwid_markerStopRegion(const char* regionTag)
 
     cpu_id = hashTable_get(tag, &results);
     thread_id = getThreadID(cpu_id);
+    if (results->state != MARKER_STATE_START)
+    {
+        fprintf(stderr, "WARN: Stopping an unknown/not-started region %s\n", regionTag);
+        return -EFAULT;
+    }
     results->groupID = groupSet->activeGroup;
     results->startTime.stop.int64 = timestamp.stop.int64;
     results->time += timer_print(&(results->startTime));
@@ -540,21 +597,29 @@ likwid_markerStopRegion(const char* regionTag)
 
     for(int i=0;i<groupSet->groups[groupSet->activeGroup].numberOfEvents;i++)
     {
-        DEBUG_PRINT(DEBUGLEV_DEVELOP, STOP [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu, regionTag, thread_id, cpu_id, i,
-                        LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
-        result = calculateMarkerResult(groupSet->groups[groupSet->activeGroup].events[i].index, results->StartPMcounters[i],
-                                        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData,
-                                        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows -
-                                        results->StartOverflows[i]);
-        if (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != THERMAL)
+        if (groupSet->groups[groupSet->activeGroup].events[i].type != NOTYPE)
         {
-            results->PMcounters[i] += result;
+            DEBUG_PRINT(DEBUGLEV_DEVELOP, STOP [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu, regionTag, thread_id, cpu_id, i,
+                            LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
+            result = calculateMarkerResult(groupSet->groups[groupSet->activeGroup].events[i].index, results->StartPMcounters[i],
+                                            groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData,
+                                            groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows -
+                                            results->StartOverflows[i]);
+            if (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != THERMAL)
+            {
+                results->PMcounters[i] += result;
+            }
+            else
+            {
+                results->PMcounters[i] = result;
+            }
         }
         else
         {
-            results->PMcounters[i] = result;
+            results->PMcounters[i] = NAN;
         }
     }
+    results->state = MARKER_STATE_STOP;
     if (use_locks == 1)
     {
         pthread_mutex_unlock(&threadLocks[myCPU]);
