@@ -5,13 +5,13 @@
  *
  *      Description:  C part of the Likwid Lua interface
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -41,6 +41,7 @@
 #include <sched.h>
 #include <sys/types.h>
 #include <pwd.h>
+#include <sys/syscall.h>
 
 #include <lua.h>                               /* Always include this */
 #include <lauxlib.h>                           /* Always include this */
@@ -55,6 +56,8 @@
 #include <textcolor.h>
 #endif
 
+#define gettid() syscall(SYS_gettid)
+
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 static int topology_isInitialized = 0;
@@ -1333,13 +1336,19 @@ lua_likwid_cpustr_to_cpulist(lua_State* L)
 {
     int ret = 0;
     char* cpustr = (char *)luaL_checkstring(L, 1);
-    int* cpulist = (int*) malloc(MAX_NUM_THREADS * sizeof(int));
+    if (!cputopo)
+    {
+        topology_init();
+        cputopo = get_cpuTopology();
+        topology_isInitialized = 1;
+    }
+    int* cpulist = (int*) malloc(cputopo->numHWThreads * sizeof(int));
     if (cpulist == NULL)
     {
         lua_pushstring(L,"Cannot allocate data for the CPU list");
         lua_error(L);
     }
-    ret = cpustr_to_cpulist(cpustr, cpulist, MAX_NUM_THREADS);
+    ret = cpustr_to_cpulist(cpustr, cpulist, cputopo->numHWThreads);
     if (ret <= 0)
     {
         lua_pushstring(L,"Cannot parse cpustring");
@@ -1362,13 +1371,21 @@ lua_likwid_nodestr_to_nodelist(lua_State* L)
 {
     int ret = 0;
     char* nodestr = (char *)luaL_checkstring(L, 1);
-    int* nodelist = (int*) malloc(MAX_NUM_NODES * sizeof(int));
+    if (!numainfo)
+    {
+        topology_init();
+        numa_init();
+        numainfo = get_numaTopology();
+        topology_isInitialized = 1;
+        numa_isInitialized = 1;
+    }
+    int* nodelist = (int*) malloc(numainfo->numberOfNodes * sizeof(int));
     if (nodelist == NULL)
     {
         lua_pushstring(L,"Cannot allocate data for the node list");
         lua_error(L);
     }
-    ret = nodestr_to_nodelist(nodestr, nodelist, MAX_NUM_NODES);
+    ret = nodestr_to_nodelist(nodestr, nodelist, numainfo->numberOfNodes);
     if (ret <= 0)
     {
         lua_pushstring(L,"Cannot parse node string");
@@ -1391,13 +1408,19 @@ lua_likwid_sockstr_to_socklist(lua_State* L)
 {
     int ret = 0;
     char* sockstr = (char *)luaL_checkstring(L, 1);
-    int* socklist = (int*) malloc(MAX_NUM_NODES * sizeof(int));
+    if (!cputopo)
+    {
+        topology_init();
+        cputopo = get_cpuTopology();
+        topology_isInitialized = 1;
+    }
+    int* socklist = (int*) malloc(cputopo->numSockets * sizeof(int));
     if (socklist == NULL)
     {
         lua_pushstring(L,"Cannot allocate data for the socket list");
         lua_error(L);
     }
-    ret = nodestr_to_nodelist(sockstr, socklist, MAX_NUM_NODES);
+    ret = nodestr_to_nodelist(sockstr, socklist, cputopo->numSockets);
     if (ret <= 0)
     {
         lua_pushstring(L,"Cannot parse socket string");
@@ -1476,7 +1499,6 @@ lua_likwid_getPowerInfo(lua_State* L)
             return 0;
         }
     }
-
     lua_newtable(L);
     lua_pushstring(L,"hasRAPL");
     lua_pushboolean(L,power_hasRAPL);
@@ -1626,7 +1648,7 @@ lua_likwid_startPower(lua_State* L)
 #else
     PowerType type = (PowerType) ((lua_Unsigned)lua_tointegerx(L,2, NULL));
 #endif
-    luaL_argcheck(L, type >= PKG+1 && type <= DRAM+1, 2, "Type not valid");
+    luaL_argcheck(L, type >= PKG+1 && type <= NUM_POWER_DOMAINS, 2, "Type not valid");
     power_start(&pwrdata, cpuId, type-1);
     lua_pushnumber(L,pwrdata.before);
     return 1;
@@ -1643,7 +1665,7 @@ lua_likwid_stopPower(lua_State* L)
 #else
     PowerType type = (PowerType) ((lua_Unsigned)lua_tointegerx(L,2, NULL));
 #endif
-    luaL_argcheck(L, type >= PKG+1 && type <= DRAM+1, 2, "Type not valid");
+    luaL_argcheck(L, type >= PKG+1 && type <= NUM_POWER_DOMAINS, 2, "Type not valid");
     power_stop(&pwrdata, cpuId, type-1);
     lua_pushnumber(L,pwrdata.after);
     return 1;
@@ -1858,6 +1880,22 @@ lua_likwid_return_signal_state(lua_State* L)
     return 1;
 }
 
+static int
+lua_likwid_send_signal(lua_State* L)
+{
+    int err = 0;
+#if LUA_VERSION_NUM == 501
+    pid_t pid = ((lua_Integer)lua_tointeger(L,1));
+    int signal = ((lua_Integer)lua_tointeger(L,2));
+#else
+    pid_t pid = ((lua_Unsigned)lua_tointegerx(L,1, NULL));
+    int signal = ((lua_Unsigned)lua_tointegerx(L,2, NULL));
+#endif
+    err = kill(pid, signal);
+    lua_pushnumber(L, err);
+    return 1;
+}
+
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
 void
@@ -1891,13 +1929,23 @@ lua_likwid_startProgram(lua_State* L)
     char *argv[4096];
     exec = (char *)luaL_checkstring(L, 1);
     int nrThreads = luaL_checknumber(L,2);
-    int cpus[MAX_NUM_THREADS];
+    CpuTopology_t cputopo = get_cpuTopology();
+    if (nrThreads > cputopo->numHWThreads)
+    {
+        lua_pushstring(L,"Number of threads greater than available HW threads");
+        lua_error(L);
+        return 0;
+    }
+    int *cpus = malloc(cputopo->numHWThreads * sizeof(int));
+    if (!cpus)
+        return 0;
     cpu_set_t cpuset;
     if (nrThreads > 0)
     {
         if (!lua_istable(L, -1)) {
           lua_pushstring(L,"No table given as second argument");
           lua_error(L);
+          free(cpus);
         }
         for (status = 1; status<=nrThreads; status++)
         {
@@ -1921,6 +1969,7 @@ lua_likwid_startProgram(lua_State* L)
     pid = fork();
     if (pid < 0)
     {
+        free(cpus);
         return 0;
     }
     else if ( pid == 0)
@@ -1940,6 +1989,7 @@ lua_likwid_startProgram(lua_State* L)
     else
     {
         signal(SIGCHLD, catch_sigchild);
+        free(cpus);
         lua_pushnumber(L, pid);
     }
     return 1;
@@ -1982,8 +2032,10 @@ lua_likwid_waitpid(lua_State* L)
     pid_t retpid = waitpid(pid, &status, 0);
     if (pid == retpid)
     {
-        if (WIFEXITED(status))
+        if (WIFEXITED(status) || WIFSIGNALED(status))
+        {
             ret = WEXITSTATUS(status);
+        }
     }
     lua_pushinteger(L, ret);
     return 1;
@@ -2050,6 +2102,37 @@ lua_likwid_pinProcess(lua_State* L)
     return 0;
 }
 
+static int
+lua_likwid_pinThread(lua_State* L)
+{
+    int cpuID = luaL_checknumber(L,-2);
+    int silent = luaL_checknumber(L,-1);
+#ifdef HAS_SCHEDAFFINITY
+    luaL_argcheck(L, cpuID >= 0, 1, "CPU ID must be greater or equal 0");
+    if (affinity_isInitialized == 0)
+    {
+        affinity_init();
+        affinity_isInitialized = 1;
+        affinity = get_affinityDomains();
+    }
+    affinity_pinThread(cpuID);
+    if (!silent)
+    {
+#ifdef COLOR
+            color_on(BRIGHT, COLOR);
+#endif
+            printf("[likwid-pin] PID %lu -> core %d - OK", gettid(), cpuID);
+#ifdef COLOR
+            color_reset();
+#endif
+            printf("\n");
+    }
+#else
+    printf("Pinning of threads is not supported by your system\n")
+#endif
+    return 0;
+}
+
 static int
 lua_likwid_setenv(lua_State* L)
 {
@@ -2059,6 +2142,14 @@ lua_likwid_setenv(lua_State* L)
     return 0;
 }
 
+static int
+lua_likwid_unsetenv(lua_State* L)
+{
+    const char* element = (const char*)luaL_checkstring(L, -1);
+    unsetenv(element);
+    return 0;
+}
+
 static int
 lua_likwid_getpid(lua_State* L)
 {
@@ -2401,15 +2492,6 @@ lua_likwid_getCpuClockCurrent(lua_State* L)
     return 1;
 }
 
-static int
-lua_likwid_setCpuClockCurrent(lua_State* L)
-{
-    const int cpu_id = lua_tointeger(L,-2);
-    const unsigned long freq = lua_tointeger(L,-1);
-    lua_pushnumber(L, freq_setCpuClockCurrent(cpu_id, freq));
-    return 1;
-}
-
 static int
 lua_likwid_getCpuClockMin(lua_State* L)
 {
@@ -2444,13 +2526,33 @@ lua_likwid_setCpuClockMax(lua_State* L)
     return 1;
 }
 
+static int
+lua_likwid_setTurbo(lua_State* L)
+{
+    const int cpu_id = lua_tointeger(L,-2);
+    const int turbo = lua_tointeger(L,-1);
+    lua_pushnumber(L, freq_setTurbo(cpu_id, turbo));
+    return 1;
+}
+
+static int
+lua_likwid_getTurbo(lua_State* L)
+{
+    const int cpu_id = lua_tointeger(L,-1);
+    lua_pushnumber(L, freq_getTurbo(cpu_id));
+    return 1;
+}
+
 static int
 lua_likwid_getGovernor(lua_State* L)
 {
     const int cpu_id = lua_tointeger(L,-1);
     char *gov = freq_getGovernor(cpu_id);
     if (gov)
+    {
         lua_pushstring(L, gov);
+        free(gov);
+    }
     else
         lua_pushnil(L);
     return 1;
@@ -2471,7 +2573,10 @@ lua_likwid_getAvailFreq(lua_State* L)
     const int cpu_id = lua_tointeger(L,-1);
     char* avail = freq_getAvailFreq(cpu_id);
     if (avail)
+    {
         lua_pushstring(L, avail);
+        free(avail);
+    }
     else
         lua_pushnil(L);
     return 1;
@@ -2489,15 +2594,42 @@ lua_likwid_getAvailGovs(lua_State* L)
     return 1;
 }
 
+
 static int
-lua_likwid_getDriver(lua_State* L)
+lua_likwid_setUncoreFreqMin(lua_State* L)
 {
-    const int cpu_id = lua_tointeger(L,-1);
-    char* drv = freq_getDriver(cpu_id);
-    if (drv)
-        lua_pushstring(L, drv);
-    else
-        lua_pushnil(L);
+    const int socket_id = lua_tointeger(L,-2);
+    const uint64_t freq = lua_tointeger(L,-1);
+    int err = freq_setUncoreFreqMin(socket_id, freq);
+    lua_pushinteger(L, err);
+    return 1;
+}
+
+static int
+lua_likwid_getUncoreFreqMin(lua_State* L)
+{
+    const int socket_id = lua_tointeger(L,-1);
+    uint64_t freq = freq_getUncoreFreqMin(socket_id);
+    lua_pushinteger(L, freq);
+    return 1;
+}
+
+static int
+lua_likwid_setUncoreFreqMax(lua_State* L)
+{
+    const int socket_id = lua_tointeger(L,-2);
+    const uint64_t freq = lua_tointeger(L,-1);
+    int err = freq_setUncoreFreqMax(socket_id, freq);
+    lua_pushinteger(L, err);
+    return 1;
+}
+
+static int
+lua_likwid_getUncoreFreqMax(lua_State* L)
+{
+    const int socket_id = lua_tointeger(L,-1);
+    uint64_t freq = freq_getUncoreFreqMax(socket_id);
+    lua_pushinteger(L, freq);
     return 1;
 }
 
@@ -2688,8 +2820,10 @@ luaopen_liblikwid(lua_State* L){
     lua_register(L, "likwid_memSweepDomain", lua_likwid_memSweepDomain);
     // Pinning functions
     lua_register(L, "likwid_pinProcess", lua_likwid_pinProcess);
+    lua_register(L, "likwid_pinThread", lua_likwid_pinThread);
     // Helper functions
     lua_register(L, "likwid_setenv", lua_likwid_setenv);
+    lua_register(L, "likwid_unsetenv", lua_likwid_unsetenv);
     lua_register(L, "likwid_getpid", lua_likwid_getpid);
     lua_register(L, "likwid_access", lua_likwid_access);
     lua_register(L, "likwid_startProgram", lua_likwid_startProgram);
@@ -2698,6 +2832,7 @@ luaopen_liblikwid(lua_State* L){
     lua_register(L, "likwid_catchSignal", lua_likwid_catch_signal);
     lua_register(L, "likwid_getSignalState", lua_likwid_return_signal_state);
     lua_register(L, "likwid_waitpid", lua_likwid_waitpid);
+    lua_register(L, "likwid_sendSignal", lua_likwid_send_signal);
     // Verbosity functions
     lua_register(L, "likwid_setVerbosity", lua_likwid_setVerbosity);
     // Marker API functions
@@ -2729,7 +2864,6 @@ luaopen_liblikwid(lua_State* L){
     lua_register(L, "likwid_markerRegionMetric", lua_likwid_markerRegionMetric);
     // CPU frequency functions
     lua_register(L, "likwid_getCpuClockCurrent", lua_likwid_getCpuClockCurrent);
-    lua_register(L, "likwid_setCpuClockCurrent", lua_likwid_setCpuClockCurrent);
     lua_register(L, "likwid_getCpuClockMin", lua_likwid_getCpuClockMin);
     lua_register(L, "likwid_setCpuClockMin", lua_likwid_setCpuClockMin);
     lua_register(L, "likwid_getCpuClockMax", lua_likwid_getCpuClockMax);
@@ -2738,7 +2872,12 @@ luaopen_liblikwid(lua_State* L){
     lua_register(L, "likwid_setGovernor", lua_likwid_setGovernor);
     lua_register(L, "likwid_getAvailFreq", lua_likwid_getAvailFreq);
     lua_register(L, "likwid_getAvailGovs", lua_likwid_getAvailGovs);
-    lua_register(L, "likwid_getDriver", lua_likwid_getDriver);
+    lua_register(L, "likwid_setTurbo", lua_likwid_setTurbo);
+    lua_register(L, "likwid_getTurbo", lua_likwid_getTurbo);
+    lua_register(L, "likwid_setUncoreFreqMin", lua_likwid_setUncoreFreqMin);
+    lua_register(L, "likwid_getUncoreFreqMin", lua_likwid_getUncoreFreqMin);
+    lua_register(L, "likwid_setUncoreFreqMax", lua_likwid_setUncoreFreqMax);
+    lua_register(L, "likwid_getUncoreFreqMax", lua_likwid_getUncoreFreqMax);
     // setuid&friends
     lua_register(L, "likwid_getuid", lua_likwid_getuid);
     lua_register(L, "likwid_geteuid", lua_likwid_geteuid);
