@@ -5,14 +5,14 @@
  *
  *      Description:  Get NUMA topology from procfs and sysfs
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -174,7 +174,7 @@ nodeProcessorList(int node, uint32_t** list)
 //    int unitSize = (int) (sizeof(unsigned long)*8);
     int unitSize = (int) 32; /* 8 nibbles */
 
-    *list = (uint32_t*) malloc(MAX_NUM_THREADS * sizeof(uint32_t));
+    *list = (uint32_t*) malloc(cpuid_topology.numHWThreads * sizeof(uint32_t));
     if (!(*list))
     {
         return -ENOMEM;
@@ -210,7 +210,7 @@ nodeProcessorList(int node, uint32_t** list)
                 {
                     if (val&(1UL<<j))
                     {
-                        if (count < MAX_NUM_THREADS)
+                        if (count < cpuid_topology.numHWThreads)
                         {
                             (*list)[count] = (j+cursor);
                         }
@@ -295,7 +295,6 @@ int proc_numa_init(void)
     int err = 0;
     uint32_t i;
     uint64_t nrCPUs = 0;
-
     if (numaInitialized > 0 || numa_info.numberOfNodes > 0)
         return 0;
 
@@ -336,7 +335,6 @@ int proc_numa_init(void)
         numa_info.nodes[i].numberOfProcessors = 0;
         numa_info.nodes[i].numberOfDistances = nodeDistanceList(i, numa_info.numberOfNodes, &numa_info.nodes[i].distances);
     }
-
     if (err == 0)
         numaInitialized = 1;
     return err;
