@@ -5,14 +5,14 @@
  *
  *      Description:  Module implementing Intel RAPL interface
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -55,6 +55,8 @@ power_init(int cpuId)
     uint64_t flags;
     int i;
     int err;
+    uint32_t unit_reg = MSR_RAPL_POWER_UNIT;
+    int numDomains = NUM_POWER_DOMAINS;
 
     /* determine Turbo Mode features */
     double busSpeed;
@@ -74,43 +76,75 @@ power_init(int cpuId)
     power_info.uncoreMaxFreq = 0;
     power_info.perfBias = 0;
 
-    switch (cpuid_info.model)
+    switch (cpuid_info.family)
     {
-        case SANDYBRIDGE:
-        case IVYBRIDGE:
-        case HASWELL:
-        case SANDYBRIDGE_EP:
-        case IVYBRIDGE_EP:
-        case HASWELL_EP:
-        case ATOM_SILVERMONT_E:
-        case ATOM_SILVERMONT_Z1:
-        case ATOM_SILVERMONT_Z2:
-        case ATOM_SILVERMONT_F:
-        case ATOM_SILVERMONT_AIR:
-        case ATOM_SILVERMONT_GOLD:
-        case BROADWELL:
-        case BROADWELL_E3:
-        case BROADWELL_E:
-        case BROADWELL_D:
-        case HASWELL_M1:
-        case HASWELL_M2:
-        case SKYLAKE1:
-        case SKYLAKE2:
-        case KABYLAKE1:
-        case KABYLAKE2:
-        case XEON_PHI_KNL:
-            power_info.hasRAPL = 1;
-            break;
-        case ATOM_SILVERMONT_C:
-            power_info.hasRAPL = 1;
-            /* The info_regs list needs an update for Silvermont Type C
-               because it uses another info register */
-            info_regs[PKG] = MSR_PKG_POWER_INFO_SILVERMONT;
-            break;
-        default:
-            DEBUG_PLAIN_PRINT(DEBUGLEV_INFO, NO RAPL SUPPORT);
-            return 0;
+        case P6_FAMILY:
+            switch (cpuid_info.model)
+            {
+                case SANDYBRIDGE:
+                case IVYBRIDGE:
+                case HASWELL:
+                case SANDYBRIDGE_EP:
+                case IVYBRIDGE_EP:
+                case HASWELL_EP:
+                case ATOM_SILVERMONT_E:
+                case ATOM_SILVERMONT_Z1:
+                case ATOM_SILVERMONT_Z2:
+                case ATOM_SILVERMONT_F:
+                case ATOM_SILVERMONT_AIR:
+                case ATOM_SILVERMONT_GOLD:
+                case BROADWELL:
+                case BROADWELL_E:
+                case BROADWELL_D:
+                case BROADWELL_E3:
+                case HASWELL_M1:
+                case HASWELL_M2:
+                case XEON_PHI_KNL:
+                    power_info.hasRAPL = 1;
+                    numDomains = NUM_POWER_DOMAINS - 1;
+                    break;
+                case SKYLAKE1:
+                case SKYLAKE2:
+                case SKYLAKEX:
+                case KABYLAKE1:
+                case KABYLAKE2:
+                    power_info.hasRAPL = 1;
+                    numDomains = NUM_POWER_DOMAINS;
+                    break;
+                case ATOM_SILVERMONT_C:
+                    power_info.hasRAPL = 1;
+                    /* The info_regs list needs an update for Silvermont Type C
+                       because it uses another info register */
+                    info_regs[PKG] = MSR_PKG_POWER_INFO_SILVERMONT;
+                    numDomains = NUM_POWER_DOMAINS - 1;
+                    break;
+
+                default:
+                    DEBUG_PLAIN_PRINT(DEBUGLEV_INFO, NO RAPL SUPPORT);
+                    return 0;
+                    break;
+            }
             break;
+        case ZEN_FAMILY:
+            if (cpuid_info.model == ZEN_RYZEN)
+            {
+                cpuid_info.turbo = 0;
+                power_info.hasRAPL = 1;
+                numDomains = 2;
+                unit_reg = MSR_AMD17_RAPL_POWER_UNIT;
+                power_names[0] = "CORE";
+                power_names[1] = "PKG";
+                power_regs[0] = MSR_AMD17_RAPL_CORE_STATUS;
+                power_regs[1] = MSR_AMD17_RAPL_PKG_STATUS;
+
+                for (i = 0; i< NUM_POWER_DOMAINS; i++)
+                {
+                    limit_regs[i] = 0x0;
+                    policy_regs[i] = 0x0;
+                    perf_regs[i] = 0x0;
+                    info_regs[i] = 0x0;
+                }
+            }
     }
 
     perfmon_init_maps();
@@ -134,91 +168,112 @@ power_init(int cpuId)
     {
         busSpeed = 133.33;
     }
-    if (cpuid_info.turbo)
+
+    err = HPMread(cpuId, MSR_DEV, MSR_PLATFORM_INFO, &flags);
+    if (err == 0)
     {
-        err = HPMread(cpuId, MSR_DEV, MSR_PLATFORM_INFO, &flags);
-        if (err == 0)
+        power_info.baseFrequency = busSpeed * (double) extractBitField(flags,8,8);
+        power_info.minFrequency  = busSpeed * (double) extractBitField((flags>>(32)),8,8);
+
+        power_info.turbo.numSteps = cpuid_topology.numCoresPerSocket;
+        if (cpuid_info.model == WESTMERE_EX)
         {
-            power_info.baseFrequency = busSpeed * (double) extractBitField(flags,8,8);
-            power_info.minFrequency  = busSpeed * (double) extractBitField((flags>>(32)),8,8);
-
-            power_info.turbo.numSteps = cpuid_topology.numCoresPerSocket;
-            if (cpuid_info.model == WESTMERE_EX)
-            {
-                power_info.turbo.numSteps = 4;
-            }
-            power_info.turbo.steps = (double*) malloc(power_info.turbo.numSteps * sizeof(double));
-            if (!power_info.turbo.steps)
-            {
-                return -ENOMEM;
-            }
-
-            err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT, &flags);
-            if (err)
-            {
-                fprintf(stderr,"Cannot gather values from MSR_TURBO_RATIO_LIMIT,\n");
-            }
-            else
-            {
-                for (int i=0; i < power_info.turbo.numSteps; i++)
-                {
-                    if (i < 8)
-                    {
-                        power_info.turbo.steps[i] = busSpeed * (double) field64(flags,i*8, 8);
-                    }
-                    else
-                    {
-                        power_info.turbo.steps[i] = power_info.turbo.steps[7];
-                    }
-                }
-            }
-            if (power_info.turbo.numSteps > 8)
-            {
-                err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT1, &flags);
-                if (!err)
-                {
-                    for (int i=8; i < power_info.turbo.numSteps; i++)
-                    {
-                        if (i < 16)
-                        {
-                            power_info.turbo.steps[i] = busSpeed * (double) field64(flags,i*8, 8);
-                        }
-                        else
-                        {
-                            power_info.turbo.steps[i] = power_info.turbo.steps[15];
-                        }
-                    }
-                }
-            }
-            if (power_info.turbo.numSteps > 16)
-            {
-                err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT2, &flags);
-                if (!err)
-                {
-                    for (int i=16; i < power_info.turbo.numSteps; i++)
-                    {
-                        if (i < 24)
-                        {
-                            power_info.turbo.steps[i] = busSpeed * (double) field64(flags,i*8, 8);
-                        }
-                        else
-                        {
-                            power_info.turbo.steps[i] = power_info.turbo.steps[23];
-                        }
-                    }
-                }
-            }
+            power_info.turbo.numSteps = 4;
+        }
+        power_info.turbo.steps = (double*) malloc(power_info.turbo.numSteps * sizeof(double));
+        if (!power_info.turbo.steps)
+        {
+            return -ENOMEM;
+        }
+        memset(power_info.turbo.steps, 0, power_info.turbo.numSteps * sizeof(double));
+        err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT, &flags);
+        if (err)
+        {
+            fprintf(stderr,"Cannot gather values from MSR_TURBO_RATIO_LIMIT\n");
         }
         else
         {
-            fprintf(stderr,"Cannot gather values from MSR_PLATFORM_INFO,\n");
+            if (cpuid_info.model != SKYLAKEX)
+            {
+                unsigned long flag_vals[4];
+                int flag_idx = 0;
+                int reg_idx = 1;
+                int valid_idx = 0;
+                flag_vals[0] = flags;
+                if (power_info.turbo.numSteps > 8)
+                {
+                    err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT1, &flag_vals[1]);
+                    if (err)
+                    {
+                        flag_vals[1] = 0;
+                    }
+                }
+                if (power_info.turbo.numSteps > 16)
+                {
+                    err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT2, &flag_vals[2]);
+                    if (err)
+                    {
+                        flag_vals[2] = 0;
+                    }
+                }
+                if (power_info.turbo.numSteps > 24)
+                {
+                    err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT3, &flag_vals[3]);
+                    if (err)
+                    {
+                        flag_vals[3] = 0;
+                    }
+                }
+                power_info.turbo.steps[0] = busSpeed * (double) field64(flags,0, 8);
+                for (int i=1; i < power_info.turbo.numSteps; i++)
+                {
+                    if (i % 8 == 0)
+                    {
+                        flag_idx++;
+                        reg_idx = 0;
+                    }
+                    power_info.turbo.steps[i] = busSpeed * (double) field64(flag_vals[flag_idx],(i%8)*8, 8);
+                    if (power_info.turbo.steps[i] > 0)
+                        valid_idx = i;
+                    else
+                        power_info.turbo.steps[i] = power_info.turbo.steps[valid_idx];
+                }
+            }
+            else
+            {
+                uint64_t flags_cores = 0;
+                int insert = 0;
+                err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT_CORES, &flags_cores);
+                if (err)
+                {
+                    ERROR_PLAIN_PRINT(Cannot read MSR TURBO_RATIO_LIMIT_CORES);
+                    flags_cores = 0;
+                }
+                for (int i = 0; i < 8; i++)
+                {
+                    int num_cores_befores = 0;
+                    if (i > 0)
+                        num_cores_befores = field64(flags_cores,(i-1)*8, 8);
+                    int num_cores = field64(flags_cores,i*8, 8);
+                    double freq = busSpeed * (double) field64(flags, i*8, 8);
+                    for (int j = num_cores_befores; j < num_cores && insert < power_info.turbo.numSteps; j++)
+                    {
+                        power_info.turbo.steps[insert] = freq;
+                        insert++;
+                    }
+                }
+            }
         }
     }
+    else
+    {
+        fprintf(stderr,"Cannot gather values from MSR_PLATFORM_INFO\n");
+    }
 
     /* determine RAPL parameters */
     if ( power_info.hasRAPL )
     {
-        err = HPMread(cpuId, MSR_DEV, MSR_RAPL_POWER_UNIT, &flags);
+        err = HPMread(cpuId, MSR_DEV, unit_reg, &flags);
         if (err == 0)
         {
             double energyUnit;
@@ -232,7 +287,7 @@ power_init(int cpuId)
             {
                 energyUnit = 1.0 * (1 << ((flags >> 8) & 0x1F)) / 1000000;
             }
-            for (i = 0; i < NUM_POWER_DOMAINS; i++)
+            for (i = 0; i < numDomains; i++)
             {
                 power_info.domains[i].energyUnit = energyUnit;
                 power_info.domains[i].type = i;
@@ -247,12 +302,13 @@ power_init(int cpuId)
                 (cpuid_info.model == HASWELL_M2) ||
                 (cpuid_info.model == BROADWELL_D) ||
                 (cpuid_info.model == BROADWELL_E) ||
+                (cpuid_info.model == SKYLAKEX) ||
                 (cpuid_info.model == XEON_PHI_KNL))
             {
                 power_info.domains[DRAM].energyUnit = 15.3E-6;
             }
 
-            for(i = 0; i < NUM_POWER_DOMAINS; i++)
+            for(i = 0; i < numDomains; i++)
             {
                 err = HPMread(cpuId, MSR_DEV, power_regs[i], &flags);
                 if (err == 0)
@@ -288,6 +344,8 @@ power_init(int cpuId)
                         {
                             power_info.domains[i].minPower = (double) extractBitField(flags,15,16) * power_info.powerUnit;
                             power_info.domains[i].maxPower = (double) extractBitField(flags,15,32) * power_info.powerUnit;
+                            if (power_info.domains[i].minPower > power_info.domains[i].maxPower)
+                                power_info.domains[i].minPower = 0;
                             power_info.domains[i].maxTimeWindow = (double) extractBitField(flags,7,48) * power_info.timeUnit;
                         }
                     }
@@ -327,20 +385,23 @@ power_init(int cpuId)
         }
         else
         {
-            fprintf(stderr,"Cannot gather values from MSR_RAPL_POWER_UNIT, deactivating RAPL support\n");
+            fprintf(stderr,"Cannot gather values from unit register 0x%X, deactivating RAPL support\n", unit_reg);
             power_info.hasRAPL =  0;
         }
 
-        err = HPMread(cpuId, MSR_DEV, MSR_UNCORE_FREQ, &flags);
-        if (err == 0)
+        if (cpuid_info.isIntel)
         {
-            power_info.uncoreMinFreq = ((double)((flags >> 8) & 0xFFULL)) * busSpeed;
-            power_info.uncoreMaxFreq = ((double)(flags & 0xFF)) * busSpeed;
-        }
-        err = HPMread(cpuId, MSR_DEV, MSR_ENERGY_PERF_BIAS, &flags);
-        if (err == 0)
-        {
-            power_info.perfBias = flags & 0xF;
+            err = HPMread(cpuId, MSR_DEV, MSR_UNCORE_FREQ, &flags);
+            if (err == 0)
+            {
+                power_info.uncoreMinFreq = ((double)((flags >> 8) & 0xFFULL)) * busSpeed;
+                power_info.uncoreMaxFreq = ((double)(flags & 0xFF)) * busSpeed;
+            }
+            err = HPMread(cpuId, MSR_DEV, MSR_ENERGY_PERF_BIAS, &flags);
+            if (err == 0)
+            {
+                power_info.perfBias = flags & 0xF;
+            }
         }
         power_initialized = 1;
         return power_info.hasRAPL;
