@@ -6,13 +6,13 @@
  *      Description:  Overloaded library for pthread_create call. 
  *                    Implements pinning of threads together with likwid-pin.
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -40,6 +40,8 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <string.h>
+#include <unistd.h>
+#include <sys/syscall.h>
 
 #ifdef COLOR
 #include <textcolor.h>
@@ -49,6 +51,8 @@
 #define TOSTRING(x) STRINGIFY(x)
 #define LLU_CAST  (unsigned long long)
 
+#define gettid() syscall(SYS_gettid)
+
 extern int pthread_setaffinity_np(pthread_t thread, size_t cpusetsize, const cpu_set_t *cpuset);
 
 static char * sosearchpaths[] = {
@@ -73,6 +77,47 @@ static char * sosearchpaths[] = {
     } while(0)
 #endif
 
+static int *pin_ids = NULL;
+static int ncpus = 0;
+static uint64_t skipMask = 0x0;
+static int silent = 0;
+void __attribute__((constructor (103))) init_pthread_overload(void)
+{
+    char *str = NULL;
+    char *token = NULL, *saveptr = NULL;
+    char *delimiter = ",";
+    int i = 0;
+    static long avail_cpus = 0;
+    avail_cpus = sysconf(_SC_NPROCESSORS_CONF);
+    pin_ids = malloc(avail_cpus * sizeof(int));
+    memset(pin_ids, 0, avail_cpus * sizeof(int));
+    str = getenv("LIKWID_PIN");
+    if (str != NULL)
+    {
+        token = str;
+        while (token)
+        {
+            token = strtok_r(str,delimiter,&saveptr);
+            str = NULL;
+            if (token)
+            {
+                ncpus++;
+                pin_ids[i++] = strtoul(token, &token, 10);
+            }
+        }
+    }
+    str = getenv("LIKWID_SKIP");
+    if (str != NULL)
+    {
+        skipMask = strtoul(str, &str, 16);
+    }
+
+    if (getenv("LIKWID_SILENT") != NULL)
+    {
+        silent = 1;
+    }
+}
+
 int __attribute__ ((visibility ("default") ))
 pthread_create(pthread_t* thread,
         const pthread_attr_t* attr,
@@ -88,44 +133,16 @@ pthread_create(pthread_t* thread,
     static int ncalled = 0;
     static int overflow = 0;
     static int overflowed = 0;
-    static int silent = 0;
-    static int pin_ids[MAX_NUM_THREADS];
-    static uint64_t skipMask = 0x0;
-    static int ncpus = 0;
-    static int shepard = 0;
     static long online_cpus = 0;
+    static int shepard = 0;
     online_cpus = sysconf(_SC_NPROCESSORS_ONLN);
 
     /* On first entry: Get Evironment Variable and initialize pin_ids */
-    if (ncalled == 0)
+    if (ncalled == 0 && pin_ids != NULL)
     {
-        char *str;
-        char *token, *saveptr;
-        char *delimiter = ",";
-        int i = 0;
+        char* str = NULL;
         cpu_set_t cpuset;
 
-        str = getenv("LIKWID_SKIP");
-        if (str != NULL)
-        {
-            skipMask = strtoul(str, &str, 16);
-        }
-        else if ( skipMask == 0x0 )
-        {
-            dlerror();    /* Clear any existing error */
-            dlsym(RTLD_DEFAULT,"__kmpc_begin");
-
-            if (( dlerror()) == NULL)  {
-                skipMask = 0x1;
-            }
-        }
-
-
-        if (getenv("LIKWID_SILENT") != NULL)
-        {
-            silent = 1;
-        }
-
         if (!silent)
         {
             color_print("[pthread wrapper] \n");
@@ -134,17 +151,6 @@ pthread_create(pthread_t* thread,
         str = getenv("LIKWID_PIN");
         if (str != NULL)
         {
-            token = str;
-            while (token)
-            {
-                token = strtok_r(str,delimiter,&saveptr);
-                str = NULL;
-                if (token)
-                {
-                    ncpus++;
-                    pin_ids[i++] = strtoul(token, &token, 10);
-                }
-            }
             CPU_ZERO(&cpuset);
             CPU_SET(pin_ids[ncpus-1], &cpuset);
             ret = sched_setaffinity(getpid(), sizeof(cpu_set_t), &cpuset);
@@ -178,20 +184,39 @@ pthread_create(pthread_t* thread,
         FILE* fpipe;
         char cmd[512];
         char buff[512];
+        char file[256];
+        unsigned int ptr = ((void*)start_routine) - info.dli_fbase;
+
         buff[0] = '\0';
-        sprintf(cmd, "nm %s 2>/dev/null | grep %x ", info.dli_fname, ((void*)start_routine) - info.dli_fbase);
-        if ( !(fpipe = (FILE*)popen(cmd,"r")) )
-        {  // If fpipe is NULL
-            fprintf(stderr, "Problems");
-        }
-        char* t = fgets(buff, 512, fpipe);
-        char* tmp = strstr(buff, "monitor");
-        if (tmp != NULL)
+        snprintf(file, 255, "/tmp/likwidpin.%d", gettid());
+        snprintf(cmd, 511, "rm -f %s; nm %s 2>/dev/null | grep %x > %s",
+                 file, info.dli_fname, ptr, file);
+        ret = system(cmd);
+        if (!access(file, R_OK))
         {
-            shepard = 1;
-            skipMask |= 1ULL<<(ncalled);
+            fpipe = fopen(file, "r");
+            if (!fpipe)
+            {
+                fprintf(stderr, "Problems reading symbols for shepard thread detection\n");
+            }
+            else
+            {
+                char* t = fgets(buff, 512, fpipe);
+                char* tmp = strstr(buff, "monitor");
+                if (tmp != NULL)
+                {
+                    shepard = 1;
+                    skipMask |= 1ULL<<(ncalled);
+                }
+                fclose(fpipe);
+                snprintf(cmd, 511, "rm -f %s 2>/dev/null", file);
+                ret = system(cmd);
+            }
+        }
+        else
+        {
+            fprintf(stderr, "Problems reading symbols for shepard thread detection\n");
         }
-        pclose(fpipe);
     }
 
     /* Handle dll related stuff */
@@ -288,3 +313,8 @@ pthread_create(pthread_t* thread,
     return ret;
 }
 
+void __attribute__((destructor (103))) close_pthread_overload(void)
+{
+    free(pin_ids);
+}
+
