@@ -5,13 +5,13 @@
  *
  *      Description:  Implementation of timer module
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -448,13 +448,14 @@ timer_sleep(unsigned long usec)
     int status = -1;
     struct timespec req;
     struct timespec rem = {0,0};
+
     if (sleepbase == 0x0ULL)
     {
         init_sleep();
     }
     if (usec >= 1000000)
     {
-        status = sleep(usec / 1000000);
+        status = sleep(((usec-sleepbase)+500000) / 1000000);
     }
     else
     {
