@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the topology backends
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -60,6 +60,7 @@ static char* atom_22_str = "Intel Atom 22nm processor";
 static char* atom_silvermont_str = "Intel Atom (Silvermont) processor";
 static char* atom_airmont_str = "Intel Atom (Airmont) processor";
 static char* atom_goldmont_str = "Intel Atom (Goldmont) processor";
+static char* atom_goldmontplus_str = "Intel Atom (Goldmont Plus) processor";
 static char* nehalem_bloom_str = "Intel Core Bloomfield processor";
 static char* nehalem_lynn_str = "Intel Core Lynnfield processor";
 static char* nehalem_west_str = "Intel Core Westmere processor";
@@ -70,20 +71,24 @@ static char* sandybridge_ep_str = "Intel Xeon SandyBridge EN/EP processor";
 static char* haswell_str = "Intel Core Haswell processor";
 static char* haswell_ep_str = "Intel Xeon Haswell EN/EP/EX processor";
 static char* broadwell_str = "Intel Core Broadwell processor";
-static char* broadwell_e3_str = "Intel Xeon E3 Broadwell processor";
 static char* broadwell_d_str = "Intel Xeon D Broadwell processor";
+static char* broadwell_e3_str = "Intel Xeon E3 Broadwell processor";
 static char* broadwell_ep_str = "Intel Xeon Broadwell EN/EP/EX processor";
 static char* skylake_str = "Intel Skylake processor";
+static char* skylakeX_str = "Intel Skylake SP processor";
 static char* kabylake_str = "Intel Kabylake processor";
+static char* cannonlake_str = "Intel Cannonlake processor";
 static char* nehalem_ex_str = "Intel Nehalem EX processor";
 static char* westmere_ex_str = "Intel Westmere EX processor";
 static char* xeon_mp_string = "Intel Xeon MP processor";
 static char* xeon_phi_string = "Intel Xeon Phi (Knights Corner) Coprocessor";
 static char* xeon_phi2_string = "Intel Xeon Phi (Knights Landing) (Co)Processor";
-static char* barcelona_str = "AMD Barcelona processor";
-static char* shanghai_str = "AMD Shanghai processor";
-static char* istanbul_str = "AMD Istanbul processor";
-static char* magnycours_str = "AMD Magny Cours processor";
+static char* xeon_phi3_string = "Intel Xeon Phi (Knights Mill) (Co)Processor";
+static char* barcelona_str = "AMD K10 (Barcelona) processor";
+static char* shanghai_str = "AMD K10 (Shanghai) processor";
+static char* istanbul_str = "AMD K10 (Istanbul) processor";
+static char* magnycours_str = "AMD K10 (Magny Cours) processor";
+static char* thuban_str = "AMD K10 (Thuban) processor";
 static char* interlagos_str = "AMD Interlagos processor";
 static char* kabini_str = "AMD Family 16 model - Kabini processor";
 static char* opteron_sc_str = "AMD Opteron single core 130nm processor";
@@ -94,6 +99,7 @@ static char* athlon64_f_str = "AMD Athlon64 (AM2) Rev F 90nm processor";
 static char* athlon64_X2_g_str = "AMD Athlon64 X2 (AM2) Rev G 65nm processor";
 static char* athlon64_g_str = "AMD Athlon64 (AM2) Rev G 65nm processor";
 static char* amd_k8_str = "AMD K8 architecture";
+static char* amd_zen_str = "AMD K17 (Zen) architecture";
 static char* unknown_intel_str = "Unknown Intel Processor";
 static char* unknown_amd_str = "Unknown AMD Processor";
 
@@ -102,6 +108,7 @@ static char* short_atom = "atom";
 static char* short_pm = "pentiumm";
 static char* short_silvermont = "silvermont";
 static char* short_goldmont = "goldmont";
+static char* short_goldmontplus = "goldmontplus";
 static char* short_nehalem = "nehalem";
 static char* short_nehalemEX = "nehalemEX";
 static char* short_westmere = "westmere";
@@ -116,20 +123,23 @@ static char* short_ivybridge_ep = "ivybridgeEP";
 static char* short_sandybridge = "sandybridge";
 static char* short_sandybridge_ep = "sandybridgeEP";
 static char* short_skylake = "skylake";
+static char* short_skylakeX = "skylakeX";
 static char* short_kabylake = "skylake";
+static char* short_cannonlake = "cannonlake";
 static char* short_phi = "phi";
 static char* short_phi2 = "knl";
+static char* short_phi3 = "kml";
 static char* short_k8 = "k8";
 static char* short_k10 = "k10";
 static char* short_k15 = "interlagos";
 static char* short_k16 = "kabini";
+static char* short_zen = "zen";
 static char* short_unknown = "unknown";
 
 /* #####  EXPORTED VARIABLES  ########################################## */
 
 CpuInfo cpuid_info;
 CpuTopology cpuid_topology;
-int affinity_thread2tile_lookup[MAX_NUM_THREADS];
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
@@ -174,7 +184,6 @@ initTopologyFile(FILE* file)
         if (!tree_nodeExists(currentNode, i))
         {
             tree_insertNode(currentNode, i);
-            affinity_thread2tile_lookup[hwThreadPool[i].apicId] = hwThreadPool[i].coreId;
         }
     }
 }
@@ -637,18 +646,34 @@ topology_setName(void)
                     cpuid_info.short_name = short_skylake;
                     break;
 
+                case SKYLAKEX:
+                    cpuid_info.name = skylakeX_str;
+                    cpuid_info.short_name = short_skylakeX;
+                    break;
+
                 case KABYLAKE1:
                 case KABYLAKE2:
                     cpuid_info.name = kabylake_str;
                     cpuid_info.short_name = short_skylake;
                     break;
 
+                case CANNONLAKE:
+                    cpuid_info.name = cannonlake_str;
+                    cpuid_info.short_name = short_cannonlake;
+                    break;
+
                 case XEON_PHI_KNL:
                     cpuid_info.supportUncore = 1;
                     cpuid_info.name = xeon_phi2_string;
                     cpuid_info.short_name = short_phi2;
                     break;
 
+                case XEON_PHI_KML:
+                    cpuid_info.supportUncore = 1;
+                    cpuid_info.name = xeon_phi3_string;
+                    cpuid_info.short_name = short_phi3;
+                    break;
+
                 case NEHALEM_EX:
                     cpuid_info.name = nehalem_ex_str;
                     cpuid_info.short_name = short_nehalemEX;
@@ -694,9 +719,15 @@ topology_setName(void)
                     cpuid_info.short_name = short_silvermont;
                     break;
                 case ATOM_SILVERMONT_GOLD:
+                case ATOM_DENVERTON:
                     cpuid_info.name = atom_goldmont_str;
                     cpuid_info.short_name = short_goldmont;
                     break;
+                
+                case ATOM_GOLDMONT_PLUS:
+                    cpuid_info.name = atom_goldmontplus_str;
+                    cpuid_info.short_name = short_goldmontplus;
+                    break;
 
                 default:
                     cpuid_info.name = unknown_intel_str;
@@ -785,6 +816,10 @@ topology_setName(void)
                     cpuid_info.name = magnycours_str;
                     break;
 
+                case THUBAN:
+                    cpuid_info.name = thuban_str;
+                    break;
+
                 default:
                     cpuid_info.name = unknown_amd_str;
                     break;
@@ -802,6 +837,11 @@ topology_setName(void)
             cpuid_info.short_name = short_k16;
             break;
 
+        case ZEN_FAMILY:
+            cpuid_info.name = amd_zen_str;
+            cpuid_info.short_name = short_zen;
+            break;
+
         default:
             return EXIT_FAILURE;
             break;
@@ -812,10 +852,10 @@ topology_setName(void)
 const struct
 topology_functions topology_funcs = {
 #ifndef LIKWID_USE_HWLOC
-    .init_cpuInfo = cpuid_init_cpuInfo,
-    .init_cpuFeatures = cpuid_init_cpuFeatures,
-    .init_nodeTopology = cpuid_init_nodeTopology,
-    .init_cacheTopology = cpuid_init_cacheTopology,
+    .init_cpuInfo = proc_init_cpuInfo,
+    .init_cpuFeatures = proc_init_cpuFeatures,
+    .init_nodeTopology = proc_init_nodeTopology,
+    .init_cacheTopology = proc_init_cacheTopology,
     .close_topology = NULL,
 #else
     .init_cpuInfo = hwloc_init_cpuInfo,
@@ -860,7 +900,6 @@ void topology_setupTree(void)
                */
             //printf("Insert HWThread %d from Core %d at Socket %d\n", hwThreadPool[i].apicId, hwThreadPool[i].coreId, hwThreadPool[i].packageId);
             tree_insertNode(currentNode, hwThreadPool[i].apicId);
-            affinity_thread2tile_lookup[hwThreadPool[i].apicId] = hwThreadPool[i].coreId;
         }
 
     }
@@ -903,10 +942,6 @@ standard_init:
         sched_getaffinity(0,sizeof(cpu_set_t), &cpuSet);
         if (cpu_count(&cpuSet) < sysconf(_SC_NPROCESSORS_CONF))
         {
-            funcs.init_cpuInfo = proc_init_cpuInfo;
-            funcs.init_cpuFeatures = proc_init_cpuFeatures;
-            funcs.init_nodeTopology = proc_init_nodeTopology;
-            funcs.init_cacheTopology = proc_init_cacheTopology;
             cpuid_topology.activeHWThreads =
                 ((cpu_count(&cpuSet) < sysconf(_SC_NPROCESSORS_CONF)) ?
                 cpu_count(&cpuSet) :
@@ -1043,6 +1078,7 @@ print_supportedCPUs (void)
     printf("\t%s\n",skylake_str);
     printf("\t%s\n",atom_goldmont_str);
     printf("\t%s\n",xeon_phi2_string);
+    printf("\t%s\n",skylakeX_str);
     printf("\n");
     printf("Supported AMD processors:\n");
     printf("\t%s\n",opteron_sc_str);
@@ -1054,6 +1090,7 @@ print_supportedCPUs (void)
     printf("\t%s\n",magnycours_str);
     printf("\t%s\n",interlagos_str);
     printf("\t%s\n",kabini_str);
+    printf("\t%s\n",amd_zen_str);
     printf("\n");
 }
 
