@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the hwloc based topology backend
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Authors:  Thomas Roehl (tr), thomas.roehl@googlemail.com
  *
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -37,6 +37,7 @@
 
 #include <topology.h>
 #include <affinity.h>
+#include <cpuid.h>
 #ifdef LIKWID_USE_HWLOC
 #include <hwloc.h>
 #include <topology_hwloc.h>
@@ -48,6 +49,25 @@ hwloc_topology_t hwloc_topology = NULL;
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
+static int
+readCacheInclusiveIntel(int level)
+{
+    uint32_t eax = 0x0U, ebx = 0x0U, ecx = 0x0U, edx = 0x0U;
+    eax = 0x04;
+    ecx = level;
+    CPUID(eax, ebx, ecx, edx);
+    return edx & 0x2;
+}
+
+static int readCacheInclusiveAMD(int level)
+{
+    uint32_t eax = 0x0U, ebx = 0x0U, ecx = 0x0U, edx = 0x0U;
+    eax = 0x8000001D;
+    ecx = level;
+    CPUID(eax, ebx, ecx, edx);
+    return (edx & (0x1<<1));
+}
+
 #ifdef LIKWID_USE_HWLOC
 int
 likwid_hwloc_record_objs_of_type_below_obj(
@@ -87,9 +107,12 @@ hwloc_init_cpuInfo(cpu_set_t cpuSet)
     {
         setenv("HWLOC_HIDE_ERRORS", "1", 1);
     }
-    likwid_hwloc_topology_init(&hwloc_topology);
-    likwid_hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
-    likwid_hwloc_topology_load(hwloc_topology);
+    if (!hwloc_topology)
+    {
+        likwid_hwloc_topology_init(&hwloc_topology);
+        likwid_hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
+        likwid_hwloc_topology_load(hwloc_topology);
+    }
     obj = likwid_hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_SOCKET, 0);
 
     cpuid_info.model = 0;
@@ -206,7 +229,6 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
             {
                 hwThreadPool[id].coreId = 0;
                 hwThreadPool[id].packageId = 0;
-                affinity_thread2core_lookup[hwThreadPool[id].apicId] = hwThreadPool[id].coreId;
                 continue;
             }
         }
@@ -238,7 +260,6 @@ hwloc_init_nodeTopology(cpu_set_t cpuSet)
             }
         }
 #endif
-        affinity_thread2core_lookup[hwThreadPool[id].apicId] = hwThreadPool[id].coreId;
         while (obj->type != socket_type) {
             obj = obj->parent;
             if (!obj)
@@ -339,18 +360,31 @@ hwloc_init_cacheTopology(void)
         cachePool[id].threads = likwid_hwloc_record_objs_of_type_below_obj(
                         hwloc_topology, obj, HWLOC_OBJ_PU, NULL, NULL);
 #if defined(__x86_64) || defined(__i386__)
-        while (!(info = likwid_hwloc_obj_get_info_by_name(obj, "inclusiveness")) && obj->next_cousin)
+        if (obj->infos_count > 0)
         {
-            obj = obj->next_cousin; // If some PU/core are not bindable because of cgroup, hwloc may not know the inclusiveness of some of their cache.
+            while (!(info = likwid_hwloc_obj_get_info_by_name(obj, "inclusiveness")) && obj->next_cousin)
+            {
+                // If some PU/core are not bindable because of cgroup, hwloc may
+                // not know the inclusiveness of some of their cache.
+                obj = obj->next_cousin;
+            }
+            if(info)
+            {
+                cachePool[id].inclusive = info[0]=='t';
+            }
+            else if (cpuid_info.isIntel)
+            {
+                cachePool[id].inclusive = readCacheInclusiveIntel(obj->attr->cache.depth);
+            }
         }
-        if(info)
+        else
         {
-            cachePool[id].inclusive = info[0]=='t';
-        }
-        else if (cpuid_info.isIntel)
-        {
-            DEBUG_PLAIN_PRINT(DEBUGLEV_ONLY_ERROR, Cannot read cache inclusiveness);
-            cachePool[id].inclusive = 0;
+            // If a CPU is already pinned, it might not get the cache infos,
+            // so we get the inclusiveness through CPUID
+            if (cpuid_info.isIntel)
+                cachePool[id].inclusive = readCacheInclusiveIntel(obj->attr->cache.depth);
+            else
+                cachePool[id].inclusive = readCacheInclusiveAMD(obj->attr->cache.depth);
         }
 #endif
 #if defined(_ARCH_PPC)
@@ -372,7 +406,8 @@ hwloc_close(void)
 {
     if (hwloc_topology)
     {
-        hwloc_topology_destroy(hwloc_topology);
+        likwid_hwloc_topology_destroy(hwloc_topology);
+        hwloc_topology = NULL;
     }
 }
 #else
