@@ -5,14 +5,14 @@
  *
  *      Description:  Interface to the procfs/sysfs based topology backend
  *
- *      Version:   4.2
- *      Released:  22.12.2016
+ *      Version:   4.3.1
+ *      Released:  04.01.2018
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2018 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -392,6 +392,11 @@ proc_init_cpuFeatures(void)
             cpuid_info.featureFlags |= (1<<HTT);
             strcat(cpuid_info.features, "HTT ");
         }
+        else if (strncmp(cptr,"avx512", 6) == 0 && !(cpuid_info.featureFlags & (1<<AVX512)))
+        {
+            cpuid_info.featureFlags |= (1<<AVX512);
+            strcat(cpuid_info.features, "AVX512 ");
+        }
         cptr = strtok(NULL, delimiter);
     }
 
@@ -433,7 +438,6 @@ proc_init_nodeTopology(cpu_set_t cpuSet)
         {
             bstring src = bread ((bNread) fread, fp);
             hwThreadPool[i].coreId = ownatoi(bdata(src));
-            affinity_thread2core_lookup[hwThreadPool[i].apicId] = hwThreadPool[i].coreId;
             fclose(fp);
         }
         bdestroy(file);
@@ -622,6 +626,7 @@ proc_init_cacheTopology(void)
                 break;
             case K16_FAMILY:
             case K15_FAMILY:
+            case ZEN_FAMILY:
                 cachePool[i].inclusive = readCacheInclusiveAMD(cachePool[i].level);
                 break;
             /* For K8 and K10 it is known that they are inclusive */
